/**
 * @(#)BarServer.java 1.0
 * @Author Tognon Stefano 1.0 (1998) all the work
 *
 * This software was developing at Padova University in collaboration with
 * Padova CNR Ladseb in Italy.
 * THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
 *
 * This program creates the server (and the client) that are able to perform 
 * a robots cooperative bar pushing task using behavior based agents control.
 * See the "Tognon Stefano, Trasporto collettivo di oggetti tramite robot mobili,
 * Tesi di Laurea, Universit degli Studi di Padova, Marzo 1999"
 * for more details.
 */

package simulator;

import java.util.*;
import java.io.*;
import java.lang.*;
import simulator.*;


/**
 * This <i>class</i> rappresent a 2D vector with some operation
 */ 
class Position {

  double x,y;

  /**
   * Initialize vector with initial value
   *
   * @param a the x value
   * @param b the y value
   */
  public Position(double a, double b) {
    this.x=a;
    this.y=b;
  }

  /**
   * Initialize vector with another vector
   *
   * @param p the another vector
   */
  public Position(Position p) {
    x=p.x;
    y=p.y; 
  }

  /**
   * Find the angle between this vector and the passed vector
   *
   * @param P the passed vector
   * @result the angle
   */
  public double findAngle(Position P) {
    return(Math.atan2(P.y,P.x)-Math.atan2(this.y,this.x));
  }

  /**
   * Calculate the distance beetween this vector and the passed vector
   *
   * @param P the passed vector
   * @result the distance
   */
  public double dist(Position P) {
    return (Math.sqrt((P.y-this.y)*(P.y-this.y)+(P.x-this.x)*(P.x-this.x)));
  }

  /**
   * Calculate the sum vector of this vector and the passed vector
   *
   * @param P the passed vector
   * @result the sum vector 
   */
  public Position sum(Position P) {
    Position pos=new Position(0,0);
    pos.x=this.x+P.x;
    pos.y=this.y+P.y;
    return pos;
  }

  /**
   * Calculate the difference between this vector and the passed vector
   *
   * @param P the passed vector
   * @result the difference vector 
   */
  public Position diff(Position P) {
    Position pos=new Position(0,0);
    pos.x=P.x-this.x;
    pos.y=P.y-this.y;
    return pos;
  }

  /**
   * Multiply the vector for a costant value
   *
   * @param a the costant value
   * @result the modified resulting vector
   */
  public Position multiply(double a) {
    Position pos=new Position(0,0);
    pos.x=a*this.x;
    pos.y=a*this.y;
    return pos;
  }

  /**
   * Performs a cross product of vector
   *
   * @param p the 2 vector to use in the calcule
   * @return the result us a number
   */
  public double crossProduct(Position p) {
    return x*p.y-p.x*y;
  } 
}


/**
 * Contain costant information about simulation object
 */
class Fisical {
  /**
   * Length of the bar
   */
  public static double barLength=2; // 2m

  /**
   * Radius of the bar
   */
  public static final double barRadius=0.025; // 2,5cm
  
  /**
   * Mass of the bar
   */
  public static final double barMass=1; // 1Kg 

  /**
   * Dimension of the cube (length, height, depth)
   */
  public static double boxDim=0.1;  // 10cm 

  /**
   * Mass of the cube
   */
  public static double boxMass=0.5; // 0.5Kg
  
  /**
   * Box static attrition of rotation (wood on stone) 
   */
  public static final double boxStaticRot=0.6; 
  
  /**
   * Box static attrition of translation (wood on stone)
   */
  public static final double boxStaticTra=0.6; 
  
  /**
   * Box dinamic attrition of rotation (wood on stone) 
   */
  public static final double boxDinamicRot=0.5; 
  
  /** 
   * Box dinamic attrition of translation (wood on stone)
   */  
  public static final double boxDinamicTra=0.5; 

  /**
   * Radius of the robot
   */
  public static final double robotRadius=0.12; // 12cm

  /**
   * Height of the robot
   */
  public static final double robotHeight=0.08; // 8cm
  
  /**
   * Mass of the robot support for bar
   */
  public static final double robotSupportMass=0.2; // 0.2Kg
  
  /**
   * Mass of the robot counterbalance for bar 
   */
  public static final double robotCounterbalanceMass=0.05; // 0.05Kg 
    
  /**
   * Radius of the sensor 1
   */
  public static final double sensorRadius=0.015; // 1,5cm

  /**
   * Length of the sensor 1
   */
  public static final double sensorLength=0.1; // 10cm

  /**
   * Repose length of the sensor 1 
   */
  public static final double sensor1Repose=0.1; // 10cm
  
  /**
   * Repose length of the sensor 2
   */ 
  public static final double sensor2Repose=0.07; //7cm

 /**
   * Length of removal 1 of sensor by mass center of robot
   */ 
  public static final double robotRemoval1=sensor2Repose; 
  
  /**
   * Length of removal 2 of sensor by mass center of robot
   */ 
  public static final double robotRemoval2=0.02; //2cm;
  
  /**
   * Elastic costant of sensor 1
   */
  public static final double sensor1Costant=160; // 160 N/m
  
  /**
   * Elastic costant of sensor 2
   */
  public static final double sensor2Costant=200; // 200 N/m 
  
  /**
   * Attrition coefficient of support in direction of sensor 1
   * metal su metal (dry condition)
   */
  public static final double coefAttrition1=0.2;
  
  /**
   * Attrition coefficient of support in direction of sensor 2
   * metal su metal (dry condition)
   */
  public static final double coefAttrition2=0.2; 
  
  /**
   * Attrition coefficient of prismatic couple in orrizzontal
   * metal su metal (dry condition)
   */
  public static final double coefAttrition3=0.2;
  
  /**
   * Attrition coefficient of prismatic couple in vertical
   * metal su metal (dry condition)
   */  
  public static final double coefAttrition4=0.2;
  
  /**
   * Moment attrition coefficient of rotoidal couple
   * metal su metal
   */
  public static final double coefAttrition5=5E-5; // 5e-5 m 
  
  /**
   * Moment attrition coefficient of rotoidal couple in the support
   * metal su metal
   */
  public static final double coefAttrition6=5E-5; // 0.01 m 
  
  /**
   * gravity acceleration
   */
  public static final double g=9.81; // 9.81m/s
  
  /**
   * Weight of prismatic couple in orrizzontal
   */
  public static final double prismaticWeight=0.2*g;
  
  /**
   * Wieght of box in N
   */
  public static double boxWeight=boxMass*g;
  
  /**
   * Max number of robot for one bar
   */
  public static final int maxRobot=20; 
}


/**
 * Contains information about the position of a box
 */
class BoxState {
  /**
   * The x,y position of the center of mass of box
   */
  public Position massCenter;

  /**
   * The orientation angle from x axle
   */
  public double xAngle;

  /**
   * True if the box is collided to the bar
   */
  public boolean collisionBar;

  /**
   * True if the box has the same angle of the bar
   */
  public boolean attachedBar;
  
  /**
   * Contain the relative position from bar massCenter of the point that box corner
   * is in contact with the bar. It also can contain relative position of the mass
   * center perpendicular to the bar when <code>attachedBar</code> is true.
   * NaN means that there's no contact
   */
  public double contactPoint;
  
  /**
   * If true means that the box is stopped, so there's static attrition
   */
  public boolean lock;

  /**
   * Identification number for box
   */
   public int ID;

  /**
   * Initialize position of box 
   *
   * @param p the x,y position of mass center
   * @param a the rotation angle from x axle
   * @param i an identifier for the box
   */
  public BoxState(Position p, double a, int i) {
    massCenter=p;
    xAngle=a; 
    ID=i;
    collisionBar=false;
    attachedBar=false;
    lock=true;
    contactPoint=Double.NaN; // there's no contact
   }

  /**
   * Initialize box position using another box
   *
   * @param box the box to copy
   */
  public BoxState(BoxState box) {
    massCenter=new Position(box.massCenter); //create new copy, not a reference
    xAngle=box.xAngle;
    ID=box.ID;
    collisionBar=box.collisionBar;
    attachedBar=box.attachedBar;
    lock=box.lock;
    contactPoint=box.contactPoint;
  }
}


/**
 * Contains information about the position of a bar
 */
class BarState {
  /**
   * The x,y position of the center of mass of bar
   */
  public Position massCenter;

  /**
   * The orientation angle from x axle
   */
  public double xAngle;

  /**
   * Initialize position of bar
   * 
   * @param p the x,y position of mass center
   * @param a the rotation angle from x axle
   */
  public BarState(Position p, double a) {
    massCenter=p;
    xAngle=a; 
   }

   /**
    * Initialize position of bar
    *
    * @param b the <code>BarState</code> where copy information
    */
   public BarState(BarState b) {
     massCenter=b.massCenter;
     xAngle=b.xAngle;
   }
}


/**
 * Contain information about the position of the sensor
 */
class SensorState {
  /**
   * The x,y position of the mass center
   */
  Position massCenter;
  
  /**
   * The orientation angle from x axle
   */
  double xAngle;
  
  /**
   * Initialize position of the sensor
   * 
   * @param p the x,y position of mass center
   * @param a the rotation angle from x axle
   */
  public SensorState(Position p, double a) {
    massCenter=p;
    xAngle=a;
  }    
}


/**
 * Contains information about position/velocity of agent
 */
class RobotState {
  /**
   * Contains actual position of the mass robot center
   */
  Position massCenter;  

  /**
   * Contains angle rotation about x axle (but  is the same of the y axle and
   * the perpendicular of the bar)
   */ 
  double xAngle;

  /**
   * Contains actual velocity of agents
   */
  Position massCenterVel;

  /**
   * Contains relative position of robot sensor in respenct of the bar mass center 
   */
  double relativePos;  
  
  /**
   * Contain actual length of sensor 1
   */ 
  double sensor1Length=Fisical.sensor1Repose; 
  
  /**
   * Contain actual length of sensor 2 
   */ 
  double sensor2Length=Fisical.sensor2Repose; 

  /**
   * Result force that the bar and support apply to robot
   */
  double resultForce;
  
  /**
   * Contains actual behavior we are executing (it's use for changing robot color)
   */ 
  int behavior;
  
  /**
   * Initilize actual position and other information of the robot agent
   *
   * @param p the actual position
   * @param a the angle
   * @param v the actual velocity
   * @param r relative position of robot sensor in respect of bar
   * @param f result force that bar and support apply to robot
   */ 
  public RobotState(Position p, double a, Position v, double r, double f) {
    massCenter=p;
    xAngle=a;
    massCenterVel=v;
    relativePos=r;
    resultForce=f;
  }  
}

/**
 * Remember a sliding window of old value
 */
class OldValue {
  /**
   * The number of element to remember
   */
  protected int N;

  /**
   * The actual size of sliding window
   */
  protected int n;

  /**
   * The storage of element to remember
   */
  protected double[] old;

  /**
   * Build an empty window of value
   */
  public OldValue(int size) {
    N=size;
    n=1;
    old=new double[N];
    old[0]=0;           // first default value
  }  

  /**
   * Get the oldest value
   *
   * @return the oldest value
   */
  public double get() {
    return old[0];  
  }

  /**
   * Set the youngest value
   * 
   * @param el the youngest value 
   */
  public void set(double el) {
    int i;

    if (n<N) old[n++]=el;
    else {
      for (i=1; i<N; i++) {
        old[i-1]=old[i];
      }
      old[N-1]=el;
    }
  }
}


/**
 * Execute the task of the robot pushing bar
 */
class RobotAgent extends Agent {

  /**
   * Contains actual velocity of the mass robot center
   */
  Position actualVel;  
  
  /**
   * Contains actual position of the mass robot center
   */
  Position actualPos;

  /**
   * Contains goal position of the mass robot center
   */
  Position goal;

  /**
   * Contains angle rotation about x axle (but  is the same of the y axle and
   * the perpendicular of the bar)
   */ 
  double angle;
  
  /**
   * Contains final angle rotation about x axle
   */
  double goalAngle;
  
  /**
   * tile where change message with server
   */
  Queue[] agentQueue; 
  
  /**
   * Progressive identifier for robot (must be 1,2,...)
   */
  int I;
  
  /**
   * Name of the agent
   */
  String name;
  
  /**
   * Force that sensor 1 support (in Newton)
   */
  double force1=0;
  
  /**
   * Force that sensor 2 support (in Newton)
   */
  double force2=0;
  
  /**
   * Force that sensor 1 was supporting previus time
   */
  double old1;
  
  /**
   * Force that sensor 2 was supporting previus time
   */
  double old2;  
  
  /**
   * Min velocity for the robot 
   */
  public final static double Vmin=0.1/2; // 0,1 m/sec
  
  /**
   * Nominal velocity for the robot
   */
  public final static double Vnom=0.3/2; // 0,3 m/sec
  
  /**
   * Max velocity for the robot
   */
  public final static double Vmax=0.5/2;  // 0,5 m/sec 
  
  /**
   * Percentual of velocity to slowly readapt toward Vnom
   */
  public final static double Vperc=0.5; // 5%
  
  /**
   * Max error that we want for achieving goal
   */
  public final static double epsGoal=0.005; // 5 mm

  /**
   * Min distance for robot and goal that is to considerate near
   */
  public final static double epsNearGoal=0.05; // 5 cm 
  
  /**
   * Min variation of force in first sensor that activate behavior 
   */
  public final static double epsForce1=0.35; // 0.5 N 
  
  /**
   * Min force of second sensor that activate behavior
   */
  public final static double epsForce2=1.05; // 1.05 N
  
  /**
   * Max angle to turn when we have max force in second sensor
   */
  public final static double alfaMax=4*Math.PI/180; // 4
  
  /**
   * Min angle to turn when we have min force in second sensor
   */
  public final static double alfaMin=0.5*Math.PI/180; // 0.5  
  
  /**
   * angle to turn when behavior of second sensor is active 
   */
  public double alfa;
  
  /**
   * Max force of second sensor use for turning with max angle
   */
  public final static double force2Max=Fisical.sensor2Costant*(Fisical.sensor2Repose-0.03); 
  
  /**
   * Min force of second sensor use for turning with min angle 
   */
  public final static double force2Min=epsForce2; 

  /**
   * Max delta force of first sensor for changing velocity
   */
  public final static double force1Max=force2Max; // 15N 

  /**
   * Min delta force of first sensor for changing velocity
   */
  public final static double force1Min=epsForce1;
  
  /**
   * Type of action to performe (-1=reduce vel.,1=add velocity)
   */
  public int actionType; 
  
  /**
   * A sliding window of old value
   */
  OldValue old;
   
  /**
   * Initilize actual position and goal position of the robot agent
   *
   * @param i the identifier for the agent (must be 1,2,...)
   * @param name the name of agent 
   * @param p the actual position
   * @param a the angle
   * @param g the goal position
   * @param ga the goal angle
   * @param pq the queue for changing messages
   * @param ct the type of control for behavior that reduce force2
   */ 
  public RobotAgent(int i, String name, Position p, double a, Position g, double ga, Queue[] pq, int ct) {
    I=i;
    this.name=name;
    actualPos=p;
    angle=a;
    goal=g;
    goalAngle=ga;
    agentQueue=pq;
    actionType=ct; 
    old=new OldValue(3);
    setPriority(MAX_PRIORITY);
    start();
  }  

  /**
   * Executes behavior-based stategy for the robot
   */
  public void run() {
    boolean achieveGoal=false; 
    String msg="";
    double delta;
    double teta;   // angle of mass center in direction of goal respect x axle;
    double module=0; // module of actual velocity
    int behavior;  // indicate how behavior we are executing 
    actualVel=new Position(0,0); // initial null velocity
    BufferedWriter f;
  try {
    f=new BufferedWriter(new FileWriter("c:/temp/tmp/AG"+I));
   
    while (!achieveGoal) {
      this.suspend();
      // get position of robot
      agentQueue[0].put(name,name+" "+I+" GET POS", 0);
      msg=agentQueue[I].get(name,I);
      actualPos.x=(Float.valueOf(msg.substring(0,msg.indexOf(" ")))).floatValue();
      actualPos.y=(Float.valueOf(msg.substring(msg.indexOf(" "),msg.indexOf("  ")))).floatValue();
      angle=(Float.valueOf(msg.substring(msg.indexOf("  "),msg.length()))).floatValue();
      
      // get force of sensor 1
      agentQueue[0].put(name,name+" "+I+" GET FOR1", 0);
      msg=agentQueue[I].get(name,I);
      old1=old.get();
      force1=(Float.valueOf(msg)).floatValue();
      old.set(force1); 
      
      // get force of sensor 2
      agentQueue[0].put(name,name+" "+I+" GET FOR2", 0);
      msg=agentQueue[I].get(name,I);
      old2=force2;
      force2=(Float.valueOf(msg)).floatValue();

      //planning: behaviors control
      if ((Math.abs(actualPos.x-goal.x)>epsGoal)|(Math.abs(actualPos.y-goal.y)>epsGoal)) {
        module=Math.sqrt(actualVel.x*actualVel.x+actualVel.y*actualVel.y);
        // calculate actual angle of mass center to goal for x axle
        teta=Math.atan((goal.y-actualPos.y)/(goal.x-actualPos.x));
        if ((goal.x-actualPos.x)<0) teta+=Math.PI;

        if (Math.abs(force2)>epsForce2) {
          // calculate the angle of turning
          alfa=alfaMin+(alfaMax-alfaMin)/(force2Max-force2Min)*(Math.abs(force2)-force2Min); 
          delta=Vmin+(Vmax-Vmin)/(force2Max-force2Min)*(Math.abs(force2)-force2Min);
          delta/=7;                    // we don't need all delta
          // adjust sign of velocity according with the correct control type
          if (actionType==-1) delta=-delta;
          if (force2>0) alfa=-alfa;    // adjust direction of turning  
          alfa=0;
          module+=delta;               // update velocity module
          if (Math.abs(old1-force1)>epsForce1) {
            delta=Vmin+(Vmax-Vmin)/(force1Max-force1Min)*(Math.abs(old1-force1));
            delta/=4.5;                    // we don't need all delta
            if (force1-old1>0) delta=-delta;  // adjust sign of velocity
          }
          module+=delta;
          if (force1<-1) module+=Vnom/4;
          if (module>Vmax) module=Vmax;
          if (module<Vmin) module=Vmin;
          if (((goal.x-actualPos.x)<epsNearGoal) & ((goal.y-actualPos.y)<epsNearGoal))
            module=Vmin;
          // set new velocity
          actualVel.x=module*Math.cos(alfa+teta);
          actualVel.y=module*Math.sin(alfa+teta);          
          behavior=2; 
          // new velocity => new direction
          angle+=alfa;
        } else {
            if (Math.abs(old1-force1)>epsForce1) {
              delta=Vmin+(Vmax-Vmin)/(force1Max-force1Min)*(Math.abs(force1)-force1Min);
              delta/=4;                       // we don't need all delta
              if (force1-old1>0) delta=-delta;  // adjust sign of velocity

              module+=delta;               // update velocity module
              if (force1<-1) module+=Vnom/5;

              if (module>Vmax) module=Vmax;
              if (module<Vmin) module=Vmin;   
              if (((goal.x-actualPos.x)<epsNearGoal) & ((goal.y-actualPos.y)<epsNearGoal))
                module=Vmin;
              // calculate new direction using teta
              angle=teta-Math.PI/2;
              // calculate x/y component using new direction of robot
              actualVel.x=module*Math.cos(teta);
              actualVel.y=module*Math.sin(teta); 
              behavior=1; 
            } else {
                // readapts the module of the speed slowly toward Vnom
                if (module<Vnom) module+=(Vnom-module)*Vperc;
                if (module>Vnom) module-=(module-Vnom)*Vperc;
                // but if we are near goal use Vmin
                if (((goal.x-actualPos.x)<epsNearGoal) & ((goal.y-actualPos.y)<epsNearGoal))
                   module=Vmin;
                // calculate new direction using teta
                angle=teta-Math.PI/2;
                // calculate x/y component using new direction of robot
                actualVel.x=module*Math.cos(teta);
                actualVel.y=module*Math.sin(teta);
                behavior=0; 
              }
          }
      } else {
          // turn the angle to the rigth position
          if (angle>2*Math.PI) angle-=2*Math.PI;
          if (Math.abs(Math.sin(angle)-Math.sin(goalAngle))<0.04) {
             angle=goalAngle;
             achieveGoal=true;
          } else angle-=(angle-goalAngle)/2;            
          actualVel.x=0;     
          actualVel.y=0;
          behavior=3; 
        }
      //Actuating
      agentQueue[0].put(name,name+" "+I+" PUT VEL "+actualVel.x+" "+actualVel.y,0);  
      agentQueue[0].put(name,name+" "+I+" PUT ANGLE "+angle,0);
      agentQueue[0].put(name,name+" "+I+" PUT BEH "+behavior,0);
      System.out.println("AGENT "+I+"("+behavior+"): "+force1+" "+force2+" "+module);
      f.write(behavior+" "+force1+" "+force2+" "+module+"\n");
      f.flush();
    }  
   } catch (IOException e) {}
  }
}



/**
 * Impements the method that calculate position of robot and send the vrml
 * scene
 */
class BarSendVRML extends GenericSendVRML {

  /**
   * Contains a list of robots state position
   */
  RobotState[] robot=new RobotState[Fisical.maxRobot];

  /**
   * Contains a list of sensors state position  
   */
  SensorState[] sensor=new SensorState[Fisical.maxRobot];
   
  /**
   * Contains a list of non collided box state position
   */
  BoxState[] box=new BoxState[Fisical.maxRobot];

  /**
   * Contains a list of collided with bar box state position
   */
  BoxState[] boxColl=new BoxState[Fisical.maxRobot]; // collided box

  /** 
   * Number of robots
   */
  int N;
  
  /**
   * Number of non collided boxs
   */
  int M; // number of box
   
  /**
   * number of collided box  
   */
  int K;
   
  /**
   * Use to stimate new solution angle
   */
  double delta=0;  

  /**
   * Calculate an approssimate sign of a parameter using a function that is continuos
   * and has the first and second differential continuos (so it can be used by 
   * Newton-Rapson algorithm
   *
   * @param arg the argument of function
   * @result the approssimate sign of argument
   */
  public double sign(double arg) {
    final double eps=0.001;
    final double eps_=1/eps;
    if (arg<=-eps) return -1;
    if (arg>=+eps) return +1;
    double ris=(eps_*arg-2/Math.PI*Math.cos(Math.PI/2*(eps-arg)*eps_))*Math.sin(Math.PI/2*(arg-eps)*eps_);
    if (ris<=1.e-8) return 0; // eliminate numeric error in Gauss sistem resolution
      else return ris;
  }
  
  /**
   * Calculate first differential of approssimate sign function in the passed parameter
   *
   * @param arg the argument 
   * @result the value of differential
   */
  public double sign1(double arg) {
    final double eps=0.001;
    final double eps_=1/eps;
    
    if (arg>=eps || arg<=-eps) return 0;
      else return (2*eps_*(1-Math.cos(Math.PI/2*(arg-eps)*eps_)*Math.cos(Math.PI/2*(arg-eps)*eps_)));
  } 
  
  /**
   * Calculates the point that is perpendicular to the bar from the box center of mass
   *
   * @param box the mass center of the box
   * @param bar the state of bar
   * @param m the coefficient of the rect y=mx+q that represent the bar
   * @param q the coefficient of the rect y=mx+q that represent the bar
   * @result the calculated point  
   */
  public Position perpendicularPoint(Position box, BarState bar, double m, double q) {
    Position point;
    if (Double.isInfinite(m)) {
      point=new Position(bar.massCenter.x, box.y);
    } else {
        point=new Position(box);
        double dist=(box.y-(m*box.x+q))/Math.sqrt(1+m*m);
        point.x+=dist*Math.sin(bar.xAngle);
        point.y+=-dist*Math.cos(bar.xAngle);
      }  
    return point;  
  }  

  /**
   * Get the relative position of one point respect the mass center of the bar
   * NB the point are the external of the bar
   *
   * @param point the position of the point 
   * @param bar the mass center of bar
   * @param alfa the angle of bar respect x axle
   */
  public double getRelative(Position point, BarState bar) {
    double x=bar.massCenter.x-Fisical.barRadius*Math.sin(bar.xAngle);
    double y=bar.massCenter.y+Fisical.barRadius*Math.cos(bar.xAngle);
    double l=Math.sqrt(Math.pow(point.x-x,2)+Math.pow(point.y-y,2));
    if ((Math.abs(x+l*Math.cos(bar.xAngle)-point.x)>0.01) ||
       (Math.abs(y+l*Math.sin(bar.xAngle)-point.y)>0.01)) return -l;
      else return l; 
  }
  
  /**
   * Calculate the sign of the argument
   *
   * @param n the double argument
   * @result 0/+1/-1
   */
  public int signDouble(double n) {
    if (n>0) return 1;
    if (n<0) return -1;
    return 0;
  } 
  
  /**
   * Calculates the two points solution of box corner that collided to the bar 
   *
   * @param m coefficient of rect y=mx+q
   * @param q coefficient of rect y=mx+q
   * @param bar the state of the bar
   * @param box the state of the box
   * @param p1 the first solution
   * @param p2 the second solution 
   * @result false if the box is parallel and attached to the bar
   */ 
  public boolean getCollidedPoints(double m, double q, BarState bar, BoxState box, Position p1, Position p2) {
    double a,b,c; // coefficient of ax^2+bx+c=0
    double l=Fisical.boxDim/Math.sqrt(2);     // actual distance
    Position p=perpendicularPoint(box.massCenter,bar,m,q);
    if (Math.sqrt(Math.pow(p.x-box.massCenter.x,2)+Math.pow(p.y-box.massCenter.y,2))<=Fisical.boxDim/2) 
      return false;

    // calculate the two solution of equation  
    if (Double.isInfinite(m)) {
        // bar is parallel of Y axle
        a=Math.pow(box.massCenter.x-bar.massCenter.x,2);
        b=box.massCenter.y;
        c=box.massCenter.y*box.massCenter.y-l+a;
        p1.y=b+Math.sqrt(b-a);
        p2.y=b-Math.sqrt(b-a);
        p1.x=bar.massCenter.x;
        p2.x=bar.massCenter.x;  
    } else { 
        a=1+m*m;
        b=-2*(box.massCenter.x+box.massCenter.y*m-m*q);
        c=Math.pow(box.massCenter.x,2)+Math.pow(box.massCenter.y,2)
          -l*l+q*q-2*box.massCenter.y*q;   
        p1.x=(-b+Math.sqrt(b*b-4*a*c))/(2*a);
        p2.x=(-b-Math.sqrt(b*b-4*a*c))/(2*a);
        p1.y=m*p1.x+q;
        p2.y=m*p2.x+q;
      }
    return true;  
  } 
   
  /**
   * Calculates the rotation of the box that is done by the bar changing position.
   * It is done by rotate the box until the distance between one point of the bar
   * and the box mass center is the same of precedent configuration. The toward of
   * rotation is calculated considering that the point will be at the same part of
   * the perpendicular between box mass center and the bar.
   *
   * @param oldBar the old state of the bar
   * @param newBar the new state of the bar
   * @param oldBox the old box position
   * @param newBox the state of the box that will be changed
   * @result true if the box rotation is ok, false if the box is completly parallel
   *         to the bar
   */ 
  public boolean boxRotation(BarState oldBar, BarState newBar, BoxState oldBox, BoxState newBox) {
    double m,q;   // coefficient of rect y=mx+q
    Position pNew;   // perpendicular point for new bar position
    Position pOld;   // perpendicular point for old bar position
    Position p1=new Position(0,0);  // first solution of equation
    Position p2=new Position(0,0);  // second solution of equation
    Position vectorSide1; // vector for contact point to box c.m. (old)
    Position vectorSide2; // vector for contact point to box c.m. (new)
    Position p=new Position(oldBar.massCenter.x
                           +oldBox.contactPoint*Math.cos(oldBar.xAngle)
                           -Fisical.barRadius*Math.sin(oldBar.xAngle),
                            oldBar.massCenter.y
                           +oldBox.contactPoint*Math.sin(oldBar.xAngle)
                           +Fisical.barRadius*Math.cos(oldBar.xAngle));  // old point of contact
    double p1_;  // first solution relative position 
    double p2_;  // second solution relative position
    double alfa; // angle of box turning
  
    // calculate coefficient or rect that rapresent the bar plus the fisical thickness
    m=Math.tan(newBar.xAngle);
    q=m*(-newBar.massCenter.x+Fisical.barRadius*Math.sin(newBar.xAngle))
      +newBar.massCenter.y+Fisical.barRadius*Math.cos(newBar.xAngle);    
    if (!getCollidedPoints(m, q, newBar, newBox, p1, p2))
      return false;  // the box is parallel and attached to the bar                          
    // choose the right solution ...
    // calculates perpendicular points from box mass center to external of bar
    pNew=perpendicularPoint(newBox.massCenter,newBar,m,q);
    m=Math.tan(oldBar.xAngle);
    q=m*(-oldBar.massCenter.x+Fisical.barRadius*Math.sin(oldBar.xAngle))
      +oldBar.massCenter.y+Fisical.barRadius*Math.cos(oldBar.xAngle);
    pOld=perpendicularPoint(oldBox.massCenter,oldBar,m,q);
    
    int s=signDouble(getRelative(pOld, oldBar)-oldBox.contactPoint);
    // calculate relative position of perpendicular points respect bar mass center
    p1_=getRelative(p1, newBar);
    p2_=getRelative(p2, newBar);
    // calculate relative position of point respect bar mass center
    double d=getRelative(pNew, newBar);
    // calculate angle of rotation of box 
    vectorSide1=p.diff(oldBox.massCenter);
    if (s==signDouble(d-p1_)) {
      newBox.contactPoint=p1_;
      //alfa=2*Math.asin(Math.sqrt(Math.pow(p.x-p1.x,2)+Math.pow(p.y-p1.y,2))/(Fisical.boxDim*Math.sqrt(2)));
      vectorSide2=p1.diff(newBox.massCenter);
      alfa=Math.atan2(vectorSide1.y,vectorSide1.x)-Math.atan2(vectorSide2.y,vectorSide2.x);
      if (s<=0) newBox.xAngle-=alfa;
        else newBox.xAngle+=alfa;
      return true; 
    }
    if (s==signDouble(d-p2_)) {
      newBox.contactPoint=p2_;
      //alfa=2*Math.asin(Math.sqrt(Math.pow(p.x-p2.x,2)+Math.pow(p.y-p2.y,2))/(Fisical.boxDim*Math.sqrt(2)));
      vectorSide2=p2.diff(newBox.massCenter);
      alfa=Math.atan2(vectorSide1.y,vectorSide1.x)-Math.atan2(vectorSide2.y,vectorSide2.x);
      if (s>=0) newBox.xAngle-=alfa;
        else newBox.xAngle+=alfa;
      return true;
    }
   return false;
  }
  
  /**
   * Calculate the numeric solution of the non-linear equations (Fx=0, Fy=0,
   * M=0, sensor...) using Newton-Raphson algorithm
   *
   * @param BR the initial stimate solution
   * @param eps the max error we want 
   * @param old1
   * @param old2
   * @param oldTeta
   * @result the solution for the bar state
   */
  public BarState non_linear_Newton_Raphson(BarState BS, double eps, double[] old1, 
	              double[] old2, double[] oldTeta) {
    double deltaL, deltaM;
    double deltaOldL, deltaOldM;
    double deltaTeta;              // oldTeta-teta 
    double deltaA;                 // beta_i-alfa
    double deltaK1;                // sensor1Costant
    double Ccos, Csin;             // (sensor1Costant)*cos(alfa) [sin(alfa)]
    double C2cos, C2sin;           // (sensor2Costant)*cos(beta_i) [sin(beta_i)]  
    double Xcos, Xsin;             // cos(alfa) [sin(alfa)]
    double Rcos, Rsin;             // cos(beta_i) [sin(beta_i)]
    double Kr1;                    // sensor1Costant*robotRelativePos 
    double Kr2;                    // sensor2Costant*robotRelativePos 
    double mua1N, mua2N;           // coefAttrition1*robotResultForce[i]
    double teta;                   // PI/2-beta_i+alfa  
    double boxCoeff=0;             // box attrition coefficient
    double[][] J=new double[2*N+3][2*N+4];
    int i, j, k;
    BarState sol=BS;

    double temp;
    double[] ris=new double[2*N+3];  // contains the result of jacobian-sistem

    boolean first=true;  // indicate if it is the first solution calculated
    int nIter=0;         // number of iteration done
    int nEps=0;          // number of required iteration
    boolean error=false; // use if jacobian is null
    do {
      nIter++;
      for (i=0; i<=2*N+2; i++) 
        for (j=0; j<=2*N+3; j++)
          J[i][j]=0.0;

      Xsin=Math.sin(sol.xAngle);
      Xcos=Math.cos(sol.xAngle);     
      for (i=1; i<=N; i++) { 
        Csin=Fisical.sensor1Costant*Xsin;
        Ccos=Fisical.sensor1Costant*Xcos;
            
        Rcos=Math.cos(robot[i].xAngle);
        Rsin=Math.sin(robot[i].xAngle);
        
        teta=Math.PI/2-robot[i].xAngle+sol.xAngle;
                
        deltaL=robot[i].sensor1Length-Fisical.sensor1Repose;
        deltaM=robot[i].sensor2Length-Fisical.sensor2Repose;       
        deltaOldL=old1[i]-robot[i].sensor1Length;
        deltaOldM=old2[i]-robot[i].sensor2Length;
        deltaTeta=oldTeta[i]-teta;
        deltaA=robot[i].xAngle-sol.xAngle;
        deltaK1=Fisical.sensor1Costant*deltaL;
        
        C2sin=Fisical.sensor2Costant*Rsin;
        C2cos=Fisical.sensor2Costant*Rcos;
        
        Kr1=Fisical.sensor1Costant*robot[i].relativePos;
        Kr2=Fisical.sensor2Costant*robot[i].relativePos;

        mua1N=Fisical.coefAttrition1*robot[i].resultForce;
        mua2N=Fisical.coefAttrition2*robot[i].resultForce;
        
        // J[0][0]+=0;                                 // forces contribution
        // J[0][1]+=0;                                 // forces contribution
        J[0][2]+=
            +Ccos*deltaL                               // force 1 contribution	                     
            -sign(deltaOldL)*mua1N*Xcos                // attrition 1 contribution
            +(sign1(deltaTeta)*Xcos+sign(deltaTeta)*Xsin)*sign(deltaK1)*
             deltaK1*Fisical.coefAttrition5/robot[i].sensor1Length // moment attrition 1
            +(sign1(deltaTeta)*Xcos+Xsin*sign(deltaTeta))*Fisical.coefAttrition6*
             robot[i].resultForce/robot[i].sensor1Length; // moment attrition 2 
        J[0][i+2]+=
            +Csin                                      // force 1 contribution
            +sign1(deltaOldL)*mua1N*Xsin               // attrition 1 contribution      
            +sign(deltaOldM)*Fisical.coefAttrition4*Rcos*Fisical.sensor1Costant*
             (sign1(deltaK1)*deltaK1+sign(deltaK1))    // attrition 3 contribution
            -(sign1(deltaK1)*deltaK1+
             sign(deltaK1)*Fisical.sensor1Repose/robot[i].sensor1Length)*
             Xcos*Fisical.coefAttrition5*Fisical.sensor1Costant/robot[i].sensor1Length*
             sign(deltaTeta)                           // moment attrition 1
            +sign(deltaTeta)*Xcos*Fisical.coefAttrition6*robot[i].resultForce/
             (robot[i].sensor1Length*robot[i].sensor1Length); // moment attrition 2
        J[0][i+N+2]+=
            -C2cos                                     // force 2 contribution 
            -sign1(deltaOldM)*mua2N*Rcos               // attrition 2 contribution
            -sign1(deltaOldM)*Rcos*(Fisical.coefAttrition3*Fisical.prismaticWeight+
             Fisical.coefAttrition4*sign(deltaK1)*deltaK1); // attrition 3 contribution  
        J[0][2*N+3]-=
            +Csin*deltaL                               // force 1 contribution 
            -C2cos*deltaM                              // force 2 contribution  
            -sign(deltaOldL)*mua1N*Xsin                // attrition 1 contribution
            +sign(deltaOldM)*mua2N*Rcos                // attrition 2 contribution
            +sign(deltaOldM)*Rcos*(Fisical.coefAttrition3*Fisical.prismaticWeight+
             Fisical.coefAttrition4*sign(deltaK1)*deltaK1) // attrition 3 contribution
            -sign(deltaTeta)*sign(deltaK1)*deltaK1*Xcos*
             Fisical.coefAttrition5/robot[i].sensor1Length // moment attrition 1  
            -sign(deltaTeta)*Xcos*Fisical.coefAttrition6*
             robot[i].resultForce/robot[i].sensor1Length; // moment attrition 2             

        // J[1][0]+=0;                                 // forces contribution                      
        // J[1][1]+=0;                                 // forces contribution 
        J[1][2]+=
            +Csin*deltaL                               // force 1 contribution
            -sign(deltaOldL)*mua1N*Xsin                // attrition 1 contribution 
            +(sign1(deltaTeta)*Xsin-Xcos*sign(deltaTeta))*sign(deltaK1)*
             deltaK1*Fisical.coefAttrition5/robot[i].sensor1Length // moment attrition 1 
            +(sign1(deltaTeta)*Xsin-Xcos*sign(deltaTeta))*Fisical.coefAttrition6*
             robot[i].resultForce/robot[i].sensor1Length; // moment attrition 2 
        J[1][i+2]+=
            -Ccos                                      // force 1 contribution
            -sign1(deltaOldL)*mua1N*Xcos               // attrition 1 contribution 
            +sign(deltaOldM)*Fisical.coefAttrition4*Rsin*Fisical.sensor1Costant*
             (sign1(deltaK1)*deltaK1+sign(deltaK1))    // attrition 3 contribution  
            -(sign1(deltaK1)*deltaK1+
             sign(deltaK1)*Fisical.sensor1Repose/robot[i].sensor1Length)*
             sign(deltaTeta)*Xsin*Fisical.coefAttrition5*Fisical.sensor1Costant/
             robot[i].sensor1Length                    // moment attrition 1  
            +sign(deltaTeta)*Xsin*Fisical.coefAttrition6*robot[i].resultForce/
             (robot[i].sensor1Length*robot[i].sensor1Length); // moment attrition 2 
        J[1][i+N+2]+=
            -C2sin                                     // force 2 contribution    
            -sign1(deltaOldM)*mua2N*Rsin               // attrition 2 contribution               
            -sign1(deltaOldM)*Rsin*(Fisical.coefAttrition3*Fisical.prismaticWeight+
             Fisical.coefAttrition4*sign(deltaK1)*deltaK1); // attrition 3 contribution
        J[1][2*N+3]-=
            -Ccos*deltaL                               // force 1 contribution  
            -C2sin*deltaM                              // force 2 contribution
            +sign(deltaOldL)*mua1N*Xcos                // attrition 1 contribution
            +sign(deltaOldM)*mua2N*Rsin                // attrition 2 contribution
            +sign(deltaOldM)*Rsin*(Fisical.coefAttrition3*Fisical.prismaticWeight+
             Fisical.coefAttrition4*sign(deltaK1)*deltaK1) // attrition 3 contribution
            -sign(deltaTeta)*sign(deltaK1)*deltaK1*Xsin*
             Fisical.coefAttrition5/robot[i].sensor1Length // moment attrition 1
            -sign(deltaTeta)*Xsin*Fisical.coefAttrition6*
             robot[i].resultForce/robot[i].sensor1Length; // moment attrition 2
                    
        // J[2][0]+=0;                                 // forces contribution
        // J[2][1]+=0;                                 // forces contribution
        J[2][2]+=
            +Kr2*deltaM*Math.cos(deltaA)               // force 2 contribution
            -sign(deltaOldM)*mua2N*Math.cos(deltaA)*robot[i].relativePos // att. 2
            -sign(deltaOldM)*Math.cos(deltaA)*robot[i].relativePos*
             (Fisical.coefAttrition3*Fisical.prismaticWeight+
             Fisical.coefAttrition4*sign(deltaK1)*deltaK1); // attrition 3 contribution
        J[2][i+2]+=
            -Kr1                                       // force 1 contribution
            -sign1(deltaOldL)*mua1N*robot[i].relativePos // attrition 1 contribution
            +sign(deltaOldM)*Fisical.coefAttrition4*Math.sin(deltaA)*robot[i].relativePos*
             (sign1(deltaK1)*deltaK1+sign(deltaK1));  // attrition 3 contribution
        J[2][i+N+2]+=
            -Kr2*Math.sin(deltaA)                      // force 2 contribution 
            -sign1(deltaOldM)*mua2N*Math.sin(deltaA)*robot[i].relativePos // att.2
            -sign1(deltaOldM)*Math.sin(deltaA)*robot[i].relativePos*
             (Fisical.coefAttrition3*Fisical.prismaticWeight+
             Fisical.coefAttrition4*sign(deltaK1)*deltaK1); // attrition 3 contribution
        J[2][2*N+3]-=
            -Kr1*deltaL                                // force 1 contribution  
            -Kr2*deltaM*Math.sin(deltaA)               // force 2 contribution               
            +sign(deltaOldL)*mua1N*robot[i].relativePos // attrition 1 contribution
            +sign(deltaOldM)*mua2N*Math.sin(deltaA)*robot[i].relativePos // att.2
            +sign(deltaOldM)*Math.sin(deltaA)*robot[i].relativePos*
             (Fisical.coefAttrition3*Fisical.prismaticWeight+
             Fisical.coefAttrition4*sign(deltaK1)*deltaK1); // attrition 3 contribution         

        J[i+2][0]=1;
        J[i+2][1]=0;
        J[i+2][2]=-robot[i].relativePos*Xsin
                  +robot[i].sensor1Length*Xcos;
        J[i+2][i+2]=Xsin;
        J[i+2][i+N+2]=-Rcos;
        J[i+2][2*N+3]-=+sol.massCenter.x+robot[i].relativePos*Xcos
                       +robot[i].sensor1Length*Xsin
                       +(Fisical.robotRemoval1-robot[i].sensor2Length)*Rcos
                       +Fisical.robotRemoval2*Rsin
                       -robot[i].massCenter.x;

        J[i+N+2][0]=0;
        J[i+N+2][1]=1;
        J[i+N+2][2]=+robot[i].relativePos*Xcos
                    +robot[i].sensor1Length*Xsin;
        J[i+N+2][i+2]=-Xcos;
        J[i+N+2][i+N+2]=-Rsin;            
        J[i+N+2][2*N+3]-=+sol.massCenter.y+robot[i].relativePos*Xsin
                         -robot[i].sensor1Length*Xcos
                         +(Fisical.robotRemoval1-robot[i].sensor2Length)*Rsin
                         -Fisical.robotRemoval2*Rcos
                         -robot[i].massCenter.y;
      }
      
      // insert contribution of box
      for (i=0; i<K; i++) {
        if (boxColl[i].lock && boxColl[i].attachedBar) boxCoeff=Fisical.boxStaticTra;
        if (boxColl[i].lock && !boxColl[i].attachedBar) boxCoeff=Fisical.boxStaticRot;
        if (!boxColl[i].lock && boxColl[i].attachedBar) boxCoeff=Fisical.boxDinamicTra;
        if (!boxColl[i].lock && !boxColl[i].attachedBar) boxCoeff=Fisical.boxDinamicRot;
        J[0][2]+=boxCoeff*Fisical.boxWeight*Xcos;
        J[0][2*N+3]-=boxCoeff*Fisical.boxWeight*Xsin;
        J[1][2]+=boxCoeff*Fisical.boxWeight*Xsin;
        J[1][2*N+3]-=-boxCoeff*Fisical.boxWeight*Xcos;
        J[2][2*N+3]-=-boxCoeff*Fisical.boxWeight*boxColl[i].contactPoint;
      }
      
     // search if actual solution is ok                      
      temp=0;
      for (i=1; i<=2*N+2; i++) {
        temp+=J[i][2*N+3]*J[i][2*N+3];         
      }
      if (temp<(eps*eps)) return sol;          
                 
      // solve linear resulting sistem with Gauss algorithm
      for (i=0; i<2*N+2; i++) { 
        if (Math.abs(J[i][i])<=0.1E-10) {               // we can't divide, so...
          stop:
          for (k=i+1; k<=2*N+2; k++) {   // search for a new line...
            if (J[k][i]!=0) {            // without 0 and...
              for (j=0; j<=2*N+3; j++) { // change the lines
                temp=J[i][j];
                J[i][j]=J[k][j];
                J[k][j]=temp;  
              } 
              break stop;
            }
          } 
        }

        for (j=i+1; j<=2*N+2; j++) {
          temp=-J[j][i]/J[i][i];
          if (temp!=0) {
            for (k=i; k<=2*N+3; k++) {
              //J[i][k]*=temp;
              //J[j][k]+=J[i][k];
              J[j][k]+=J[i][k]*temp;
            }
          } 
        }
      }

      for (i=2*N+2; i>=0; i--) {
        temp=0;
        if (i!=2*N+2) {
          for (j=i+1; j<=2*N+2; j++) 
            temp+=-J[i][j]*ris[j];           
        }
        ris[i]=(J[i][2*N+3]+temp)/J[i][i];
      }

      if ((Double.isNaN(ris[2*N+2]) || Double.isInfinite(ris[2*N+2]))
                                   && !error) { // was the jacobian null ?
        error=true;
        sol.xAngle+=0.05;                       // try with a new value
        nEps=1; 
        nIter=0;  
      } else {                                  // updates actual results
          sol.massCenter.x+=ris[0];
          sol.massCenter.y+=ris[1];
          sol.xAngle+=ris[2]; 
          for (i=1; i<=N; i++) {
            robot[i].sensor1Length+=ris[i+2];
            robot[i].sensor2Length+=ris[i+N+2];
          }
          if (first) {                           // calculates ||error||
            first=false;
            temp=ris[0]*ris[0]+ris[1]*ris[1]+ris[2]*ris[2];
            for (i=1; i<=N; i++) {
              temp+=ris[i+2]*ris[i+2]+ris[i+N+2]*ris[i+N+2];         
            }
            temp=Math.sqrt(temp);
            nEps=2+(int)Math.round(Math.log(temp/eps)/Math.log(2));
            System.out.println("temp="+temp+" Neps="+nEps);
          }
        }
    } while (nIter<=nEps); 
    return sol;
  }

  /**
   * Stimate the position of the bar to make the solution found by
   * Newton-Raphson to be calcolated fast.
   *
   * @param actual actual bar state position 
   * @result the stimate new position of bar
   */
  public BarState stimatePosition(BarState actual) {
    int i;
    BarState sol=new BarState(new Position(0,0),actual.xAngle);
    for (i=1; i<=N; i++) {
      sol.massCenter=sol.massCenter.sum(robot[i].massCenterVel.multiply(
           server.printerTime/1000.0)); 
    }
    sol.massCenter=sol.massCenter.multiply(1./N);
    sol.massCenter=sol.massCenter.sum(actual.massCenter);
    return sol;
  }

  /**
   * Determine if there's intersection between (p1,p2) and (p3,p4) rectangles
   * (bounding box of original line)
   *
   * @param p1 first point of rectangle 1
   * @param p2 second point (opposite) of rectangle 1
   * @param p3 first point of rectangle 2
   * @param p4 second point (opposite) of rectangle 2
   * @result true if there's intersection
   */  
  public boolean quickRejection(Position p1, Position p2, Position p3, Position p4) {
    double x1,x2,y1,y2; // coord for quick rejection 1
    double x3,x4,y3,y4; // coord for quick rejection 2 

    x1=Math.min(p1.x,p2.x);
    x2=Math.max(p1.x,p2.x);
    y1=Math.min(p1.y,p2.y);
    y2=Math.max(p1.y,p2.y);

    x3=Math.min(p3.x,p4.x);
    x4=Math.max(p3.x,p4.x);
    y3=Math.min(p3.y,p4.y);
    y4=Math.max(p3.y,p4.y);

    return (x2>=x3 && x4>=x1 && y2>=y3 && y4>=y1);    
  }

  /** 
   * Search for intersection of line (p1,p2) and (p3,p4) using the cross
   * product algorithm
   *
   * @param p1 first point of first line
   * @param p2 second point of fisrt line
   * @param p3 first point of second line
   * @param p4 second point of second line
   * @return true if there's intersection
   */
  public boolean crossProduct(Position p1, Position p2, Position p3, Position p4) {
    Position a1,a2;
    double r1,r2;
    a1=new Position(p3.x-p1.x,p3.y-p1.y);
    a2=new Position(p2.x-p1.x,p2.y-p1.y);
    r1=a1.crossProduct(a2);
    a1=new Position(p4.x-p1.x,p4.y-p1.y);
    r2=a1.crossProduct(a2);
    if ((r1>0 && r2<0) || (r1<0 && r2>0)) return true;
    if (r1==0) return true;
    if (r2==0) return true;
    return false;       
  }
  
  /**
   * Calculates the position of a point in the bar that is traslated from simmetric
   * axle becouse it is at the external of bar.
   *
   * @param first the point to shift
   * @param xAngle the angle of bar respect x axle
   * @return the corect shifting point
   */
  public Position shiftRadius(Position first, double xAngle) {
    Position second=new Position(first);
    Position shift=new Position(-Fisical.barRadius*Math.sin(xAngle),
                                Fisical.barRadius*Math.cos(xAngle));
    second=second.sum(shift);
    return second;
  }  
  
  /**
   * Detect if the bar has collided to boxes object.
   * All the collision are detected and the return value contains the index of
   * collided boxes.
   * The algoritms is based on quick rejection and bounding boxs.
   *
   * @param bar actual bar state position
   * @return the indexes of box that was collided or -1 for nothing
   */
  public int[] collisionDetect(BarState bar) {
    int i;                     // variable for cycle 
    int pos=0;                 // index position of resilt array
    int result[]=new int[M+1]; // contains the result of serching 
    for (i=0; i<=M; i++)       // initialize output value to never find collision 
      result[i]=-1;
    Position p1=new Position(bar.massCenter); // first external vertex of bar
    Position p2=new Position(bar.massCenter); // second external vertex of bar
    Position p3,p4;
    Position a1=new Position(0,0);  // first vertex of box 
    Position a2=new Position(0,0);  // second vertex of box
    Position a3=new Position(0,0);  // third vertex of box
    Position a4=new Position(0,0);  // fourth vertex of box
    
    // calculate the extrems coord. of vertex of bar
    p1.x+=-Fisical.barLength/2*Math.cos(bar.xAngle);
    p1.y+=-Fisical.barLength/2*Math.sin(bar.xAngle);
    p2.x+=Fisical.barLength/2*Math.cos(bar.xAngle);
    p2.y+=Fisical.barLength/2*Math.sin(bar.xAngle); 
    p1=shiftRadius(p1, bar.xAngle);
    p2=shiftRadius(p2, bar.xAngle);

    for (i=0; i<M; i++) {
      // quick-rejection of box
      p3=new Position(box[i].massCenter.x-(Fisical.boxDim*Math.sqrt(2)/2),
                     box[i].massCenter.y+(Fisical.boxDim*Math.sqrt(2)/2));
      p4=new Position(box[i].massCenter.x+(Fisical.boxDim*Math.sqrt(2)/2),
                     box[i].massCenter.y-(Fisical.boxDim*Math.sqrt(2)/2));

      if (!quickRejection(p1,p2,p3,p4)) continue;
      
      findBoxVertexes(box[i],a1,a2,a3,a4);   

      // find if one side of box collided with the bar
      if (quickRejection(p1,p2,a1,a2)) {
        if (crossProduct(p1,p2,a1,a2)) {
          result[pos++]=i; 
          result[M]=1;     // there's collision
          continue;
        }
      }
      if (quickRejection(p1,p2,a2,a3)) {
        if (crossProduct(p1,p2,a2,a3)) {
          result[pos++]=i;
          result[M]=1;     // there's collision
          continue;
        }
      } 
      if (quickRejection(p1,p2,a3,a4)) {
        if (crossProduct(p1,p2,a3,a4)) {
          result[pos++]=i;
          result[M]=1;     // there's collision
          continue;
        }         
      }
      if (quickRejection(p1,p2,a4,a1)) {
        if (crossProduct(p1,p2,a4,a1)) {
          result[pos++]=i;
          result[M]=1;     // there's collision
          continue;
        }
      }
    }    
    return result;
  } 
  
  /**
   * Determine if the passed box has collided with the bar.
   * The used algorithm is like in <code>collisionDetect</code> methode
   * @see BarSendVRML#collisionDetect
   *
   * @param box the state of the box who search for collision
   * @param bar the state of the bar
   * @return true if there's a collision
   */
  public boolean collidedBox(BoxState box, BarState bar) {
    Position p1=new Position(bar.massCenter); // first external vertex of bar
    Position p2=new Position(bar.massCenter); // second external vertex of bar
    Position a1=new Position(0,0);  // first vertex of box 
    Position a2=new Position(0,0);  // second vertex of box
    Position a3=new Position(0,0);  // third vertex of box
    Position a4=new Position(0,0);  // fourth vertex of box
    
    // calculate the extrems coord. of vertex of bar
    p1.x+=-Fisical.barLength/2*Math.cos(bar.xAngle);
    p1.y+=-Fisical.barLength/2*Math.sin(bar.xAngle);
    p2.x+=Fisical.barLength/2*Math.cos(bar.xAngle);
    p2.y+=Fisical.barLength/2*Math.sin(bar.xAngle);
    p1=shiftRadius(p1, bar.xAngle);
    p2=shiftRadius(p2, bar.xAngle);
   
    findBoxVertexes(box,a1,a2,a3,a4);   
    
    // find if one side of box collided with the bar
    if (quickRejection(p1,p2,a1,a2)) {
      if (crossProduct(p1,p2,a1,a2)) {
        return true;
      }
    }
    if (quickRejection(p1,p2,a2,a3)) {
      if (crossProduct(p1,p2,a2,a3)) {
        return true;
      }
    }
    if (quickRejection(p1,p2,a3,a4)) {
      if (crossProduct(p1,p2,a3,a4)) {
        return true;
      }         
    }   
    if (quickRejection(p1,p2,a4,a1)) {
      if (crossProduct(p1,p2,a4,a1)) {
        return true;
      }
    }
    return false;
  }  
  
  /**
   * Calculates the 4 vertexes of the box using fisical dimension and orientation
   *
   * @param box the state of the box
   * @param a1 first vertex
   * @param a2 second vertex
   * @param a3 third vertex
   * @param a4 fourth vertex
   */
  public void findBoxVertexes(BoxState box, Position a1, Position a2, Position a3, Position a4) {
    double sinA, cosA;
    sinA=Math.sin(box.xAngle);
    cosA=Math.cos(box.xAngle);
    // found the 4 vertexs of box
    a1.x=box.massCenter.x+Fisical.boxDim/2*(-sinA+cosA);       
    a1.y=box.massCenter.y+Fisical.boxDim/2*(sinA+cosA);
    a2.x=box.massCenter.x+Fisical.boxDim/2*(sinA+cosA);
    a2.y=box.massCenter.y+Fisical.boxDim/2*(sinA-cosA);
    a3.x=box.massCenter.x+Fisical.boxDim/2*(sinA-cosA);
    a3.y=box.massCenter.y+Fisical.boxDim/2*(-sinA-cosA);
    a4.x=box.massCenter.x+Fisical.boxDim/2*(sinA-cosA);
    a4.y=box.massCenter.y+Fisical.boxDim/2*(-sinA+cosA);
  }
  
  /**
   * Interpolate the position of bar using linear algorithm
   * 
   * @param oldBar first position of bar (k=0)
   * @param newBar second position of bar (k=1)
   * @param k the interpolator coefficient (0..1)
   * @return the interpolated position
   */
  public BarState interpolation(BarState oldBar, BarState newBar, double k) {
    BarState sol=new BarState(
                  new Position(k*(newBar.massCenter.x-oldBar.massCenter.x)+oldBar.massCenter.x,
                               k*(newBar.massCenter.y-oldBar.massCenter.y)+oldBar.massCenter.y),
                  k*(newBar.xAngle-oldBar.xAngle)+oldBar.xAngle);
    return sol;                            
  }
  
  /**
   * Adjust the position of sensor to be valid with the new position of the bar.
   * This is made by calculate the point where sensor1 and sensor2 become joined.
   *
   * @param bar the state of bar 
   */ 
  public void adjustRobotSensor(BarState bar) {
    int i; // local variable for cycle
    Position p=new Position(0,0);  // point where sensor1 is attached to the bar
    Position f=new Position(0,0);  // point where sensor2 is attached to the bar
    for (i=1; i<=N; i++) {
      p.x=bar.massCenter.x+robot[i].relativePos*Math.cos(bar.xAngle);
      p.y=bar.massCenter.y+robot[i].relativePos*Math.sin(bar.xAngle);
      f.x=robot[i].massCenter.x-Fisical.robotRemoval1*Math.cos(robot[i].xAngle)
                               -Fisical.robotRemoval2*Math.sin(robot[i].xAngle);
      f.y=robot[i].massCenter.y-Fisical.robotRemoval1*Math.sin(robot[i].xAngle)
                               +Fisical.robotRemoval2*Math.cos(robot[i].xAngle);                         
      if (Math.cos(robot[i].xAngle)!=0) { 
        robot[i].sensor1Length=(p.y-f.y-Math.tan(robot[i].xAngle)*(p.x-f.x))
               /(Math.tan(robot[i].xAngle)*Math.sin(bar.xAngle)+Math.cos(bar.xAngle));
        robot[i].sensor2Length=((p.x-f.x)+robot[i].sensor1Length*Math.sin(bar.xAngle))
              /Math.cos(robot[i].xAngle);
      } else {
          robot[i].sensor1Length=-(p.x-f.x)/Math.sin(bar.xAngle);
          robot[i].sensor2Length=(p.y-f.y-robot[i].sensor1Length*Math.cos(bar.xAngle))
              /Math.sin(robot[i].xAngle);
        }
    }
  }
  
  /**
   * Move the bar back until to the point that there's the minimum boxes that collided
   *
   * @param oldBar old position of bar
   * @param newBar new position of bar
   * @param index the index of collided box
   * @return the rigth position of bar
   */
  public BarState moveBackBar(BarState oldBar, BarState newBar, int[] index) {
    Position p1=new Position(0,0); // first point solution of equation
    Position p2=new Position(0,0); // second point solution of equation
    Position a1=new Position(0,0); // first box vertex
    Position a2=new Position(0,0); // second box vertex
    Position a3=new Position(0,0); // third box vertex
    Position a4=new Position(0,0); // fourth box vertex
    double d1,d2,d3,d4;            // linear distance of point
    double m,q;                    // coefficient of equation y=mx+q
    final int maxIteration=20;     // max number of iteration for interpolation
    BarState bar=new BarState(newBar);
    BarState temp=new BarState(newBar);
    double interp=0.5;             // interpolation coefficient  
    boolean inside=true;           // true if piece of bar is inside box
    boolean result;                // result of collision searching
    int j,i;
    
    // use dicotomic algorithm to find point
    for (j=2; j<=maxIteration; j++) {
      temp=interpolation(oldBar,newBar,interp);
System.out.println(interp);
      result=false;
      for (i=0; i<M; i++) {
        if (index[i]!=-1) {
          result|=collidedBox(box[index[i]],temp);
          if (result) break;      // if there's one collision break;
        }
      }
      if (result) { 
        if (j!=maxIteration)           // no update interp if we are at the last iteration
          interp-=1./(Math.pow(2,j));  // now we probably can be out of collision   
      } else {
          interp+=1./(Math.pow(2,j-1));  // now we are out of boxes, return to previus pos.
         }
    }
    // search for collided box and mark them 
    for (i=0; i<M; i++) {  
      if (index[i]!=-1) {
        if (collidedBox(box[index[i]],temp)) {
          box[index[i]].collisionBar=true;
        }
      }
    }
    bar=interpolation(oldBar,newBar,interp);

    // calculate coefficient or rect that rapresent the bar plus the fisical thickness
    m=Math.tan(bar.xAngle);
    q=m*(-bar.massCenter.x+Fisical.barRadius*Math.sin(bar.xAngle))
      +bar.massCenter.y+Fisical.barRadius*Math.cos(bar.xAngle);
    
    for (i=0; i<M; i++) {
      // find possible points of contact 
      if (index[i]!=-1 && box[index[i]].collisionBar) {
        if (!getCollidedPoints(m, q, bar, box[index[i]] ,p1, p2)) {
          // the box is parallel and attached to the bar
          box[index[i]].contactPoint=getRelative(perpendicularPoint(box[index[i]].massCenter,bar,m,q),bar);
          box[index[i]].attachedBar=true;
        } else {
            a1=new Position(0,0); // first box vertex
            a2=new Position(0,0); // second box vertex
            a3=new Position(0,0); // third box vertex
            a4=new Position(0,0); // fourth box vertex
            // try to find the vertex points of box that is the nearest of bar rect...
            findBoxVertexes(box[index[i]], a1, a2, a3, a4);
            // ... by calculates the distance ...
            if (Double.isInfinite(m)) {
              d1=Math.abs(a1.y-bar.massCenter.y);
              d2=Math.abs(a2.y-bar.massCenter.y);
              d3=Math.abs(a3.y-bar.massCenter.y);
              d4=Math.abs(a4.y-bar.massCenter.y);
            } else {
                d1=Math.abs(m*a1.x+q-a1.y);
                d2=Math.abs(m*a2.x+q-a2.y);
                d3=Math.abs(m*a3.x+q-a3.y);
                d4=Math.abs(m*a4.x+q-a4.y);  
              }
            // ... and choosing the minor
            if (Math.min(d1,d2)==d1 && Math.min(d1,d3)==d1 && Math.min(d1,d4)==d1) 
              box[index[i]].contactPoint=getRelative(a1, bar);
            if (Math.min(d2,d1)==d2 && Math.min(d2,d3)==d2 && Math.min(d2,d4)==d2) 
              box[index[i]].contactPoint=getRelative(a2, bar);
            if (Math.min(d3,d1)==d3 && Math.min(d3,d2)==d3 && Math.min(d3,d4)==d3) 
              box[index[i]].contactPoint=getRelative(a3, bar);
            if (Math.min(d4,d1)==d4 && Math.min(d4,d2)==d4 && Math.min(d4,d3)==d4) 
              box[index[i]].contactPoint=getRelative(a4, bar);   
          }
      }
    } 
    // adjust sensors of robots to make valid this new position of the bar
    adjustRobotSensor(bar);
    return bar;
  }
  

  /**
   * Adjust position of all boxes to reflect new bar position
   *
   * @param old the old state of bar
   * @param bar the actual state of bar
   */
  public void adjustBoxesPosition(BarState old, BarState bar) {
    int i;         // cycle variable
    int n;
    double m,q;    // coefficient of equation y=mx+q
    double alfa;   // alfa angle of Perkins formula
    double beta_i; // beta initial angle of Perkins formula
    double beta_f; // beta final angle of Perkins formula
    double lMax;   // max distance for having complete rotation (Perkins)
    double dist;   // distance from box c.m. to perpendicular point in bar
    double temp;   
    double temp_i; 
    double temp_f;
    boolean mustAttached; // if box will be to considerate attached
    Position p;  // position of perpendicular point
    Position pointContact1;  // position on contact point respect origin (old)
    Position pointContact2_; // position of contact point if it was attached (new)
    Position vectorContact1; // vector that go from point of contact to c.m. of object (old) 
    Position vectorBar1;     // vector that go from bar c.m. to object c.m. (old)
    Position vectorMotion;   // vector of bar motion direction (calculate for one point) 
    BoxState oldBox;         // old position of box

    for (i=0; i<K; i++) {
      if (boxColl[i].attachedBar) {  // was the box "attached" to the bar ?
        boxColl[i].massCenter.x=bar.massCenter.x
                               -Fisical.barRadius*Math.sin(bar.xAngle) 
                               +boxColl[i].contactPoint*Math.cos(bar.xAngle)
                               -Fisical.boxDim/2*Math.sin(bar.xAngle);
        boxColl[i].massCenter.y=bar.massCenter.y
                               +Fisical.barRadius*Math.cos(bar.xAngle)
                               +boxColl[i].contactPoint*Math.sin(bar.xAngle)                         
                               +Fisical.boxDim/2*Math.cos(bar.xAngle);
        boxColl[i].xAngle-=(old.xAngle-bar.xAngle);                       
      } else {
          // calculates some useful vector for the Perkins translation formula  
          pointContact1=new Position(old.massCenter.x+boxColl[i].contactPoint*Math.cos(old.xAngle),
                                     old.massCenter.y+boxColl[i].contactPoint*Math.sin(old.xAngle));
          vectorContact1=pointContact1.diff(boxColl[i].massCenter);
          vectorBar1=old.massCenter.diff(new Position(old.massCenter.x-Fisical.barLength/2*Math.cos(old.xAngle),
                                                      old.massCenter.y-Fisical.barLength/2*Math.sin(old.xAngle)));
          pointContact2_=new Position(bar.massCenter.x+boxColl[i].contactPoint*Math.cos(old.xAngle),
                                      bar.massCenter.y+boxColl[i].contactPoint*Math.sin(old.xAngle));
          vectorMotion=pointContact1.diff(pointContact2_);
          alfa=vectorMotion.findAngle(vectorBar1);
          beta_i=vectorContact1.findAngle(vectorMotion);
          beta_f=Math.PI-alfa-Math.acos(1/Math.sqrt(2));   
          temp=Fisical.boxDim/Math.sqrt(2);
          temp_i=Math.sin(alfa+beta_i);
          temp_f=Math.sin(alfa+beta_f);
          lMax=temp/Math.sin(alfa)*(
                -temp_f-temp/2*Math.log(Math.abs((1+temp_f)/(1-temp_f)))
                +temp_i+temp/2*Math.log(Math.abs((1+temp_i)/(1-temp_i))));
          // calculate coefficient or rect that rapresent the bar plus the fisical thickness
          m=Math.tan(old.xAngle);
          q=m*(-old.massCenter.x+Fisical.barRadius*Math.sin(old.xAngle))
            +old.massCenter.y+Fisical.barRadius*Math.cos(old.xAngle);
          // calculates perpendicular points from box mass center to external of bar
          p=perpendicularPoint(boxColl[i].massCenter,old,m,q);  
          dist=Math.sqrt(Math.pow(p.x-boxColl[i].massCenter.x,2)+
                         Math.pow(p.y-boxColl[i].massCenter.y,2));
          n=(int)((lMax/Math.sqrt(Math.pow(vectorMotion.x,2)+Math.pow(vectorMotion.y,2))));
          if (n<=0) n=1;
          delta=(dist-Fisical.boxDim/2)/n;
System.out.println("\n"+delta*n+" "+n);
          //update position of box center of mass
          temp=Math.sqrt(vectorMotion.x*vectorMotion.x+vectorMotion.y*vectorMotion.y);
          p=new Position(vectorMotion.x/temp,vectorMotion.y/temp);
          oldBox=new BoxState(boxColl[i]);
          if ((Math.abs(dist-Fisical.boxDim/2)<=0.0033)||(dist<Fisical.boxDim/2)) 
             mustAttached=true;  // prevent indesiderate not allignement
          else { 
            boxColl[i].massCenter.x+=vectorMotion.x-p.x*delta;
            boxColl[i].massCenter.y+=vectorMotion.y-p.y*delta;            
            mustAttached=false;
          }
          if (mustAttached || (!boxRotation(old,bar,oldBox,boxColl[i]))) { // if the box become parallel to the bar
            boxColl[i].attachedBar=true; // now we are "attached"
            // calculate coefficient or rect that rapresent the bar plus the fisical thickness
            m=Math.tan(bar.xAngle);
            q=m*(-bar.massCenter.x+Fisical.barRadius*Math.sin(bar.xAngle))
              +bar.massCenter.y+Fisical.barRadius*Math.cos(bar.xAngle);
            // calculates perpendicular points from box mass center to external of bar
            p=perpendicularPoint(boxColl[i].massCenter,bar,m,q);  
            boxColl[i].contactPoint=getRelative(p, bar);
            boxColl[i].massCenter.x=bar.massCenter.x
                              -Fisical.barRadius*Math.sin(bar.xAngle) 
                              +boxColl[i].contactPoint*Math.cos(bar.xAngle)
                              -Fisical.boxDim/2*Math.sin(bar.xAngle);
            boxColl[i].massCenter.y=bar.massCenter.y
                              +Fisical.barRadius*Math.cos(bar.xAngle)
                              +boxColl[i].contactPoint*Math.sin(bar.xAngle)                         
                              +Fisical.boxDim/2*Math.cos(bar.xAngle);
            // N.B. This is an approssimation; if you use a VRML rappresentation 
            // of the box that is not simmetric you must see indesiderate jump of
            // rotation                   
            boxColl[i].xAngle=bar.xAngle;                  
          }
        }
    }   
  }
  
  /**
   * Calculate the position of all object in the simulation ambient
   */
  public void runTask() {
    String str="";
    String name="";
    String groupName="";
    String objectName="";
    boolean thereStatic;  // indicate if there's one "static" object
    BoxState tempBox; 
    BarState old;                                    // old state of bar    
    BarState bar=new BarState(new Position(0,0),0);  // actual state of bar
    int i;       // local variable for cicly
    int index[]; // index of collided boxes
    double ang;

    // suppose there's no objects
    M=0;
    N=0;
    K=0;      
    Enumeration objectSet=state.keys();

    // extract object to be updated from all object in vrml scene
    // and divide them
    while (objectSet.hasMoreElements()) {
      name=(String)objectSet.nextElement();
      if (name.indexOf("ROB")!=-1) {
        robot[new Integer(name.substring(name.indexOf("ROB")+3,name.length())).intValue()]=
             (RobotState)state.get(name);
        N++; 
      }
      if (name.indexOf("SEN")!=-1)
        sensor[new Integer(name.substring(name.indexOf("SEN")+3,name.length())).intValue()]=
             (SensorState)state.get(name);
      if (name.indexOf("BOX")!=-1) {
        tempBox=(BoxState)state.get(name);
        if (tempBox.collisionBar) {
          boxColl[K]=(BoxState)state.get(name);
          K++;
        } else {
           box[M]=(BoxState)state.get(name);
           M++;
          }
      } 
      if (name.indexOf("BAR")!=-1) {
        bar=(BarState)state.get(name);
        groupName=name.substring(0,name.indexOf("BAR"));
      }  
    }

    if (N==0) return; // there's nothing to updates

     // store actual value of sensors length and angle before updating position
     double[] old1=new double[N+1];     // contains old value of sensor1 or robot 1..N
     double[] old2=new double[N+1];     // contains old value of sensor2 of robot 1..N
     double[] oldTeta=new double [N+1]; // contains old teta value of robot 1..N
     for (i=1; i<=N; i++) {
       old1[i]=robot[i].sensor1Length;
       old2[i]=robot[i].sensor2Length;
       oldTeta[i]=Math.PI/2-robot[i].xAngle+bar.xAngle;
     }

    // update position of robot
    for (i=1; i<=N; i++) {
      robot[i].massCenter=robot[i].massCenter.sum(robot[i].massCenterVel.multiply(
            server.printerTime/1000.0)); 
      state.put(groupName+"ROB"+String.valueOf(i),robot[i]); 
      //System.out.println(i+" "+robot[i].massCenter.x+"[]"+robot[i].massCenter.y);
    }
 
    // calculate position of bar using Newton-Raphson numeric resolution
    // of non-linear sistem of equation (Fx=0, Fy=0, M=0)
    ang=bar.xAngle;    
    BarState stimate=stimatePosition(bar);
    stimate.xAngle+=delta;
    System.out.println("actual "+bar.massCenter.x+" "+bar.massCenter.y+" "+bar.xAngle);
    System.out.println("stimate "+stimate.massCenter.x+" "+stimate.massCenter.y+" "+stimate.xAngle);
    old=new BarState(bar);
    bar=non_linear_Newton_Raphson(stimate, 0.001, old1, old2, oldTeta);
    
    // search if the static objects are still static 
    thereStatic=false;  // suppose there's no static object
    for (i=0; i<K; i++) {
      if (boxColl[i].lock && !collidedBox(boxColl[i],bar)) {
        System.out.println("THERE'S STATIC OBJECT");
        bar=new BarState(old);  // the previus position was right
        adjustRobotSensor(bar); // set the robots sensors to right position
        thereStatic=true;       // there's one static object
        break;
      }  
    }

    // search if there's new collisions...
    if (!thereStatic && ((index=collisionDetect(bar)))[M]!=-1) {
      System.out.println("BAR BACK");
      bar=moveBackBar(old, bar, index); //... yes, move back the bar
      // adjust position of the other boxes
      adjustBoxesPosition(old,bar); // this never adjust box[index] becouse now it's not in boxColl
    } else {                 // we can make static value to dinamic value             
        if (!thereStatic) { // if there's not static
          for (i=0; i<K; i++) {
            if (boxColl[i].lock) boxColl[i].lock=false;
          }
          adjustBoxesPosition(old,bar); 
        }
      }
    ;
    delta=bar.xAngle-ang;
    
    System.out.println("effective "+bar.massCenter.x+"  "+bar.massCenter.y+"  "+bar.xAngle);
    state.put(groupName+"BAR",bar);

    // calculate new position of sensor 
    Position temp, temp1; 
    double alfa; 
    for (i=1; i<=N; i++) {
      temp=new Position(bar.massCenter.x+robot[i].relativePos*Math.cos(bar.xAngle),
                        bar.massCenter.y+robot[i].relativePos*Math.sin(bar.xAngle));
      sensor[i].massCenter=temp;
      sensor[i].massCenter.x+=(Fisical.sensorLength/2)*Math.sin(Math.PI-bar.xAngle);
      sensor[i].massCenter.y+=(Fisical.sensorLength/2)*Math.cos(Math.PI-bar.xAngle);     
      sensor[i].xAngle=bar.xAngle;
      state.put(groupName+"SEN"+String.valueOf(i),sensor[i]);
      state.put(groupName+"ROB"+String.valueOf(i),robot[i]);
    }
    //for (i=0; i<K; i++) {
    //  state.put(groupName+"OGG"+boxColl[i].ID,boxColl[i]);
    //} 
  }

  /**
   * Send the new vrml scene to all the connected clients
   */
  public void sendVRML() {
    String objectName="";
    String str="";
    Enumeration objectSet=state.keys();

    while (objectSet.hasMoreElements()) {
      objectName=(String)objectSet.nextElement();
      if (objectName.endsWith("BAR")) {
        BarState BA=(BarState)state.get(objectName);
        str+="# MODIFY "+objectName+" SFVEC3F set_translation "
             +BA.massCenter.x+" 0 "+(-BA.massCenter.y)+"\n";
        str+="# MODIFY "+objectName+" SFROTATION set_rotation 0 1 0 "
             +BA.xAngle+"\n";
      } 
      if (objectName.indexOf("BOX")!=-1) {
        BoxState BO=(BoxState)state.get(objectName);
        if (BO.collisionBar) {
          str+="# MODIFY "+objectName+" SFVEC3F set_translation "
               +BO.massCenter.x+" 0 "+(-BO.massCenter.y)+"\n";
          str+="# MODIFY "+objectName+" SFROTATION set_rotation 0 1 0 "
               +BO.xAngle+"\n";
        }
      }                     
      if (objectName.indexOf("ROB")!=-1) {
        RobotState RS=(RobotState)state.get(objectName);
        str+="# MODIFY "+objectName+" SFVEC3F set_translation "
             +RS.massCenter.x+" 0 "+(-RS.massCenter.y)+"\n";
        str+="# MODIFY "+objectName+" SFROTATION set_rotation 0 1 0 "
             +RS.xAngle+"\n";
        switch (RS.behavior) {
          case 0:
            str+="# MODIFY "+objectName+"color"+" SFCOLOR set_diffuseColor 0 0 1 \n";
            break;
          case 1:
            str+="# MODIFY "+objectName+"color"+" SFCOLOR set_diffuseColor 1 1 0 \n";
            break;
          case 2:
            str+="# MODIFY "+objectName+"color"+" SFCOLOR set_diffuseColor 0 1 0 \n";
            break;
          case 3:
            str+="# MODIFY "+objectName+"color"+" SFCOLOR set_diffuseColor 0 1 1 \n";
            break;    
        }
      }
      if (objectName.indexOf("SEN")!=-1) {
        SensorState ST=(SensorState)state.get(objectName);
        str+="# MODIFY "+objectName+" SFVEC3F set_translation "
             +ST.massCenter.x+" 0 "+(-ST.massCenter.y)+"\n";
        str+="# MODIFY "+objectName+" SFROTATION set_rotation 0 1 0 "
             +ST.xAngle+"\n";
      }
    }  
    broadcast(str);
  }
}


/**
 * Add the group of robot for simulation based for file
 */
class AddFileClientCommand extends ClientCommand {
  /**
   * Initialize object for ADDFILE command
   */
  public AddFileClientCommand() {
    super("ADDFILE   : add object and start simulation",
          "ADDFILE group_name description_file : add object and start simulation\n"+
          "group_name=[string]=the name assigned to the group\n"+
          "description_file=[string]=the name of file contains the description of the simulation"); 
  }

  /**
   * Add robots for simulation using file information.
   * This is the structure of the file (.sim extension): <p> 
   * <pre>
   * 1line:   bar_wrl_file Xcm Ycm Xangle  Xcm_goal Ycm_Goal Xangle_Goal
   * 2line:   number_of_pushing_robot robot_pushing_wrl sensor_wrl
   * 3line:   relative_position_from_bar_cm
   * ...
   * nline:   number_of_box box_wrl_file
   * N+1line: Xcm Ycm Xangle
   * ...
   * </pre>
   * 
   *
   * @param ST the parameter for simulation
   * @param connection the instance of <code>Connection</code>
   * @return false becouse simulation must continue
   */
  public boolean runCommand(StringTokenizer ST, Connection connection) {
    int nRob=0;
    int i=0;
    int j=0;
    int k=0;
    Nodes n;

    int boxsNumber;
    String boxFileName;
    float[][] boxPosition; // (x, y, angle), (x, y, angle)...

    int robotsNumber;
    String robotFileName;
    float[] robotRelativePos;

    String sensorFileName;
   
    final String startFileName="Start.wrl";
    final String goalFileName="Goal.wrl";

    String barFileName;
    float[] barStartInfo=new float[3]; // x, y, angle
    float[] barEndInfo=new float[3];   // x, y, angle
 
    String vrmlString=""; 
    String vrmlNode="";
    String s;
    StringTokenizer S;

    connection.SV.suspend();  // suspend simulation until we had create all objects

    if (!Tokens(ST,2,connection)) return false;
    String groupName=ST.nextToken();
    n=(Nodes)connection.nodesList.get(groupName+String.valueOf(0));
    if (n==null) {
      String fileName=ST.nextToken()+".sim";
      try {
        BufferedReader inputFile=new BufferedReader(new FileReader(fileName));

        // analize first line
        s=inputFile.readLine();
        S=new StringTokenizer(s, " ", false);
        if (!Tokens(S,7,connection)) {
          inputFile.close();  
          connection.SV.resume(); 
          return false;
        }       
        barFileName=S.nextToken()+".wrl";
        barStartInfo[0]=(new Float(S.nextToken())).floatValue();
        barStartInfo[1]=(new Float(S.nextToken())).floatValue();
        barStartInfo[2]=(new Float(S.nextToken())).floatValue();      
        barEndInfo[0]=(new Float(S.nextToken())).floatValue();
        barEndInfo[1]=(new Float(S.nextToken())).floatValue();
        barEndInfo[2]=(new Float(S.nextToken())).floatValue();

        // analize second line
        s=inputFile.readLine();
        S=new StringTokenizer(s, " ", false);
        if (!Tokens(S,3,connection)) {
          inputFile.close();
          connection.SV.resume();
          return false;
        }
        robotsNumber=(new Integer(S.nextToken())).intValue();
        robotFileName=S.nextToken()+".wrl";
        sensorFileName=S.nextToken()+".wrl";
        robotRelativePos=new float[robotsNumber];
    
        // analize 3, 4 ... lines
        boolean positive=false;
        boolean negative=false;
        for (i=0; i<robotsNumber; i++) {
          s=inputFile.readLine();
          S=new StringTokenizer(s, " ", false);
          if (!Tokens(S,1,connection)) {
            inputFile.close();
            connection.SV.resume();
            return false;
          }
          robotRelativePos[i]=(new Float(S.nextToken())).floatValue();          
          if (robotRelativePos[i]>0) positive=true;
                                  else negative=true;
          if (Math.abs(robotRelativePos[i])>(Fisical.barLength/2)) {
            connection.out.println("Invalid relative position");
            connection.out.flush();
            inputFile.close();
            connection.SV.resume();
            return false;
          }         
          if ((i>0) && (robotRelativePos[i]<robotRelativePos[i-1])) {
            connection.out.println("Insert robot from sx to dx position");
            connection.out.flush();
            inputFile.close();
            connection.SV.resume();
            return false;   
          }
        }
        if ((!positive) || (!negative)) {
           connection.out.println("Invalid robot position");
           connection.out.flush();
           inputFile.close();
           connection.SV.resume();
           return false;
        }
        
        // analize next lines
        s=inputFile.readLine();
        S=new StringTokenizer(s, " ", false);  
        if (!Tokens(S,2,connection)) {
          inputFile.close();
          connection.SV.resume();
          return false;
        }
        boxsNumber=(new Integer(S.nextToken())).intValue();
        boxFileName=S.nextToken()+".wrl";
        boxPosition=new float[3][boxsNumber];     
        for (i=0; i<boxsNumber; i++) {
          s=inputFile.readLine();
          S=new StringTokenizer(s, " ", false);  
          if (!Tokens(S,3,connection)) {
            inputFile.close();
            connection.SV.resume();
            return false;
          }
          boxPosition[0][i]=(new Float(S.nextToken())).floatValue();
          boxPosition[1][i]=(new Float(S.nextToken())).floatValue();
          boxPosition[2][i]=(new Float(S.nextToken())).floatValue();
        } 
        inputFile.close(); 

        //create box object and vrml rappresentation:
        String groupBox=""; 
        for (i=0;i<boxsNumber;i++) {
          groupBox=groupName+"BOX"+String.valueOf(i);
          vrmlNode="# NEW "+groupBox+" "+
                  // connection.readFile(boxFileName)+
                  "DEF Box Group { children [ Transform {"+
                  "translation 0 0.055 0 children [ Shape {"+
                  "appearance Appearance { material Material {"+
                  "diffuseColor 0 1 0 } } geometry Box {"+
                  "size "+String.valueOf(Fisical.boxDim)+" 0.1 "
                  +String.valueOf(Fisical.boxDim)+
                  " } } ] } ] }"+"\n";
          vrmlString+=vrmlNode;
          n=new Nodes(vrmlNode,connection.clientName);
          connection.nodesList.put(groupBox,n);
          connection.state.put(groupBox, new BoxState(
                 new Position(boxPosition[0][i],boxPosition[1][i]),boxPosition[2][i],i));
          vrmlString+="# MODIFY "+groupBox+" SFVEC3F set_translation "+
                      boxPosition[0][i]+" 0 "+(-boxPosition[1][i])+"\n";
          vrmlString+="# MODIFY "+groupBox+" SFROTATION set_rotation 0 1 0 "+boxPosition[2][i]+"\n";
        } 
        connection.broadcast(vrmlString);

        // create bar object and vrml rappresentation:
        String groupBar=groupName+"BAR"; 
        vrmlString="# NEW "+groupBar+" "+
                   //+connection.readFile(barFileName)
                   "DEF Bar Group { children [ Transform { translation 0 0.065 0 "+
                   "rotation 1 1 0  3.14 children [ Shape { appearance Appearance { "+
                   "material Material { diffuseColor 1 0 0 } } geometry Cylinder { "+
                   "radius 0.025 height "+String.valueOf(Fisical.barLength)+
                   " } } ] } ] }\n";
        n=new Nodes(vrmlString,connection.clientName);
        connection.nodesList.put(groupBar,n);
        connection.state.put(groupBar, new BarState(
                new Position(barStartInfo[0], barStartInfo[1]), barStartInfo[2]));
        vrmlString+="# MODIFY "+groupBar+" SFVEC3F set_translation "+
                      barStartInfo[0]+" 0 "+(-barStartInfo[1])+"\n";          
        vrmlString+="# MODIFY "+groupBar+" SFROTATION set_rotation 0 1 0 "+barStartInfo[2]+"\n"; 
        connection.broadcast(vrmlString);

        // calculate fisical result of gravity bar action for all robots
        double [][] J=new double[robotsNumber][robotsNumber+1];
        double mTemp=0;
        double sTemp=0;
        double[] robotResultForce=new double[robotsNumber];
        
        for (i=0; i<robotsNumber; i++)
          robotResultForce[i]=0;

        switch (robotsNumber) {
          case 2:
            robotResultForce[1]=Fisical.barMass*Fisical.g/(1-robotRelativePos[1]/robotRelativePos[0]); 
            robotResultForce[0]=-robotResultForce[1]*(robotRelativePos[1]/robotRelativePos[0]);
            break;
          case 3:
            // calculate the mass of the first peace of bar
            mTemp=Fisical.barMass*(robotRelativePos[1]-(-Fisical.barLength/2))/Fisical.barLength;
            // calculate the position of mass center of first peace of bar
            sTemp=(-Fisical.barLength/2+robotRelativePos[1])/2;
            robotResultForce[1]=mTemp*Fisical.g/(1+(robotRelativePos[1]-sTemp)/(sTemp-robotRelativePos[0])); 
            robotResultForce[0]=robotResultForce[1]*(robotRelativePos[1]-sTemp)/(sTemp-robotRelativePos[0]);
            // calculate the mass of the second peace of bar
            mTemp=Fisical.barMass-mTemp;
            // calculate the position of mass center of second peace of bar
            sTemp=(robotRelativePos[1]+Fisical.barLength/2)/2;
            robotResultForce[2]=mTemp*Fisical.g/(1+(robotRelativePos[2]-sTemp)/(sTemp-robotRelativePos[1])); 
            robotResultForce[1]+=robotResultForce[2]*(robotRelativePos[2]-sTemp)/(sTemp-robotRelativePos[1]);
            break; 
          default:
            // calculate the mass of the first peace of bar
            mTemp=Fisical.barMass*(robotRelativePos[1]-(-Fisical.barLength/2))/Fisical.barLength;
            // calculate the position of mass center of first peace of bar
            sTemp=(-Fisical.barLength/2+robotRelativePos[1])/2;
            robotResultForce[1]=mTemp*Fisical.g/(1+(robotRelativePos[1]-sTemp)/(sTemp-robotRelativePos[0])); 
            robotResultForce[0]=robotResultForce[1]*(robotRelativePos[1]-sTemp)/(sTemp-robotRelativePos[0]);
            
            for (i=1; i<robotsNumber-2; i++) {
              mTemp=Fisical.barMass*(robotRelativePos[i+1]-robotRelativePos[i])/Fisical.barLength;
              robotResultForce[i]+=mTemp/2*Fisical.g;
              robotResultForce[i+1]+=mTemp/2*Fisical.g;
            } 

            // calculate the mass of the last peace of bar
            mTemp=Fisical.barMass*(Fisical.barLength/2-robotRelativePos[robotsNumber-2])/Fisical.barLength;
            // calculate the position of mass center of last peace of bar
            sTemp=(robotRelativePos[robotsNumber-2]+Fisical.barLength/2)/2;
            robotResultForce[robotsNumber-1]=mTemp*Fisical.g/(1+(robotRelativePos[robotsNumber-1]-sTemp)/(sTemp-robotRelativePos[robotsNumber-2])); 
            robotResultForce[robotsNumber-2]+=robotResultForce[robotsNumber-1]*(robotRelativePos[robotsNumber-1]-sTemp)/(sTemp-robotRelativePos[robotsNumber-2]);
            break;
        }
        
        for (i=0; i<robotsNumber; i++)
          System.out.println("ResultForce="+robotResultForce[i]);
        
        // add other contribute
        for (i=0; i<robotsNumber; i++)
          robotResultForce[i]+=(Fisical.robotSupportMass+
                  Fisical.robotCounterbalanceMass)*Fisical.g;

        for (i=0; i<robotsNumber; i++)
          System.out.println("TotalResultForce="+robotResultForce[i]);

        // create robots and sensors object and vrml rappresentation
        String groupRobot=""; 
        String groupSensor="";
        String groupSignal="";
        vrmlString="";
        Position p,p1,p2; // vary position for start situation
        Position p_,p1_;  // vary position for end situation
        double delta;     // difference of start and end angle 
        int type=0;         // type of control
                
        for (i=0; i<robotsNumber; i++) {
          groupRobot=groupName+"ROB"+String.valueOf(i+1);
          // vrmlNode="# NEW "+groupRobot+" "+connection.readFile(robotFileName)+"\n";
          vrmlNode="# NEW "+groupRobot+" DEF Cilindro Transform {} \n";
          vrmlNode+="# ADDMFNODE "+groupRobot+"k "+groupRobot+" set_children "
                     +"Transform { translation 0 0.052 0 } \n";
          vrmlNode+="# ADDMFNODE "+groupRobot+"kk "+groupRobot+"k set_children "
                     +"Shape { appearance Appearance {material Material { diffuseColor 0 0 1 }} geometry Cylinder { radius .06 height .1 }}"//\n";       
                     +" Transform { translation 0 0.058 0 rotation 1 1 0  3.14 "
                     +"children [ Shape { appearance Appearance { material Material { diffuseColor 1 0 0}} geometry Cylinder { radius 0.02 height 0.09 }}]}\n";
          vrmlNode+="# ADDSFNODE "+groupRobot+"kkk "+groupRobot+"kk set_appearance "
                     +"Appearance { material Material {}}\n";
          vrmlNode+="# ADDSFNODE "+groupRobot+"color "+groupRobot+"kkk set_material "
                     +"Material { diffuseColor 0 0 1 }\n";
          vrmlString+=vrmlNode;
          n=new Nodes(vrmlNode,connection.clientName);
          connection.nodesList.put(groupRobot,n);

          // set p to robot/bar start relative position
          p=new Position(barStartInfo[0], barStartInfo[1]);
          p.x+=robotRelativePos[i]*Math.cos(barStartInfo[2]);
          p.y+=robotRelativePos[i]*Math.sin(barStartInfo[2]);
          // set p1 to robot start position
          p1=new Position(p);
          p1.x+=(Fisical.sensor1Repose+Fisical.robotRemoval2)*Math.sin(Math.PI-barStartInfo[2]);
          p1.y+=(Fisical.sensor1Repose+Fisical.robotRemoval2)*Math.cos(Math.PI-barStartInfo[2]); 
          // set p_ to robot/bar goal position
          p_=new Position(barEndInfo[0], barEndInfo[1]); 
          p_.x+=robotRelativePos[i]*Math.cos(barEndInfo[2]);
          p_.y+=robotRelativePos[i]*Math.sin(barEndInfo[2]);
          // set p1_ to goal position
          p1_=new Position(p_);
          p1_.x+=(Fisical.sensor1Repose+Fisical.robotRemoval2)*Math.sin(Math.PI-barEndInfo[2]);
          p1_.y+=(Fisical.sensor1Repose+Fisical.robotRemoval2)*Math.cos(Math.PI-barEndInfo[2]); 
                
          connection.state.put(groupRobot, new RobotState(p1, barStartInfo[2], new Position(0,0), robotRelativePos[i], robotResultForce[i]));
          connection.presentMessage[i+1]=new Semaphore(0,"");
          connection.availableCells[i+1]=new Semaphore(connection.dimQueue,"");
          connection.processesQueue[i+1]=new Queue(connection.dimQueue,connection.availableCells,connection.presentMessage);

          // calculate the type of control for robot
          delta=barEndInfo[2]-barStartInfo[2];
          if (robotRelativePos[i]>=0 && delta>0) type=1;
          if (robotRelativePos[i]<0 && delta>0) type=-1;
          if (robotRelativePos[i]>=0 && delta<=0) type=-1;
          if (robotRelativePos[i]<0 && delta<=0) type=1;
                
          RobotAgent A=new RobotAgent(i+1, groupRobot, p1, barStartInfo[2], p1_, barEndInfo[2], connection.processesQueue, type);
          connection.processesList.put(groupRobot,A);

          vrmlString+="# MODIFY "+groupRobot+" SFVEC3F set_translation "+p1.x+" 0 "+(-p1.y)+"\n";
          vrmlString+="# MODIFY "+groupRobot+" SFROTATION set_rotation 0 1 0 "+barStartInfo[2]+"\n";
          
          groupSensor=groupName+"SEN"+String.valueOf(i+1);
          vrmlNode="# NEW "+groupSensor+" "+connection.readFile(sensorFileName)+"\n";
          vrmlString+=vrmlNode;
          n=new Nodes(vrmlNode,connection.clientName);
          connection.nodesList.put(groupSensor,n);
          p2=new Position(p);
          p2.x+=(Fisical.sensorLength/2)*Math.sin(Math.PI-barStartInfo[2]);
          p2.y+=(Fisical.sensorLength/2)*Math.cos(Math.PI-barStartInfo[2]);
          connection.state.put(groupSensor, new SensorState(p2,  barStartInfo[2]));                   
          vrmlString+="# MODIFY "+groupSensor+" SFVEC3F set_translation "+p2.x+" 0 "+(-p2.y)+"\n";
          vrmlString+="# MODIFY "+groupSensor+" SFROTATION set_rotation 0 1 0 "+barStartInfo[2]+"\n"; 
        } 
        
        // create start and goal signal vrml
        groupSignal=groupName+"START";
        vrmlNode="# NEW "+groupSignal+" "+connection.readFile(startFileName)+"\n";        
        vrmlString+=vrmlNode;
        n=new Nodes(vrmlNode, connection.clientName);
        connection.nodesList.put(groupSignal,n);
        vrmlString+="# MODIFY "+groupSignal+" SFVEC3F  set_translation "+barStartInfo[0]+" 0 "+(-barStartInfo[1])+"\n";
        vrmlString+="# MODIFY "+groupSignal+" SFROTATION set_rotation 0 1 0 "+barStartInfo[2]+"\n";          
        
        groupSignal=groupName+"GOAL";
        vrmlNode="# NEW "+groupSignal+" "+connection.readFile(goalFileName)+"\n";
        vrmlString+=vrmlNode;
        n=new Nodes(vrmlNode, connection.clientName);
        connection.nodesList.put(groupSignal,n);
        vrmlString+="# MODIFY "+groupSignal+" SFVEC3F set_translation "+barEndInfo[0]+" 0 "+(-barEndInfo[1])+"\n";
        vrmlString+="# MODIFY "+groupSignal+" SFROTATION set_rotation 0 1 0 "+barEndInfo[2]+"\n";

        connection.broadcast(vrmlString);           

      } catch (FileNotFoundException e) {
         System.out.println("[ERROR] File "+fileName+" not found "+e);
        }
        catch(IOException IOe) {
          System.out.println("[ERROR] Don't able to read file"+fileName+" "+IOe);
        }
    } else {
        connection.out.println("The group "+groupName+" already exists!!!");
        connection.out.flush();
      }
    connection.SV.resume();
    return false;
  }

  /**
   * Determines if there's the expected number of tokens
   *
   * @param ST the tokens
   * @param n the number of expected token
   * @return true if ok
   */
  public boolean Tokens(StringTokenizer ST, int n, Connection connection) {
    if (ST.countTokens()<n) {
      connection.out.println("Invalid number of parameter");
      connection.out.flush();
      return false; 
    }
    return true;
  }
}


/**
 * Del the group of robot of simulation
 */
class DelFileClientCommand extends ClientCommand {
  /**
   * Initialize object for DELFILE command
   */
  public DelFileClientCommand() {
    super("DELFILE   : stop simualtion and delate objects",
          "DELFILE group_name : stop simulation and delete objects\n"+
          "group_name=[string]=the name of group to delete"); 
  } 

  /*
   * Execute the del group command
   * 
   * @param ST the parameter for simulation
   * @param connection the instance of <code>Connection</code>
   * @return false becouse simulation must continue
   */
  public boolean runCommand(StringTokenizer ST, Connection connection) {
    Nodes n;

    connection.SV.suspend();
    String groupName=ST.nextToken();
    if (connection.processesList.containsKey(groupName+"ROB1")) {
      n=(Nodes)connection.nodesList.get(groupName+"ROB1");
      if (n.userName.equals(connection.clientName)) {
        int i=1;
        RobotAgent A=((RobotAgent)connection.processesList.get(groupName+"ROB"+String.valueOf(i)));
        while (A!=null) {
          A.stop();
          connection.processesList.remove(groupName+"ROB"+String.valueOf(i));
          connection.nodesList.remove(groupName+"ROB"+String.valueOf(i));
          connection.state.remove(groupName+"ROB"+String.valueOf(i));
          connection.broadcast("# DEL "+groupName+"ROB"+String.valueOf(i));

          connection.nodesList.remove(groupName+"SEN"+String.valueOf(i));
          connection.state.remove(groupName+"SEN"+String.valueOf(i));
          connection.broadcast("# DEL "+groupName+"SEN"+String.valueOf(i));
          i++;
          A=(RobotAgent)connection.processesList.get(groupName+"ROB"+String.valueOf(i));
        }
        i=0;
        BoxState B=((BoxState)connection.state.get(groupName+"BOX"+String.valueOf(i)));
        while (B!=null) {
          connection.nodesList.remove(groupName+"BOX"+String.valueOf(i));
          connection.state.remove(groupName+"BOX"+String.valueOf(i));
          connection.broadcast("# DEL "+groupName+"BOX"+String.valueOf(i));   
          i++;
          B=((BoxState)connection.state.get(groupName+"BOX"+String.valueOf(i)));                 
        }
        connection.nodesList.remove(groupName+"START");
        connection.broadcast("# DEL "+groupName+"START");
        connection.nodesList.remove(groupName+"GOAL");
        connection.broadcast("# DEL "+groupName+"GOAL");
        connection.nodesList.remove(groupName+"BAR");
        connection.state.remove(groupName+"BAR");
        connection.broadcast("# DEL "+groupName+"BAR");
      } else {
          connection.out.println("Can't erase the group becouse it's of another uses!!!");
        }
    } else {
        connection.out.println("The group don't exist");
      }
    connection.SV.resume();
    return false;
  }
} 


/**
 * Set some fisical parameter of the simulator objects
 */
class FisicalClientCommand extends ClientCommand {
  /**
   * Initialize object for FISICAL command
   */
  public FisicalClientCommand() {
    super("FISICAL   : change some fisical parameter of objects",
          "FISICAL n val : change the n parameter to value\n"+
          "n=1: change box mass (value=Kg)\n"+
          "n=2: change box x,y dimension (value=cm)\n"+
          "n=3: change bar length (value=m)\n"+
          "if you use an 1.1 browser, you can use the applet rigth control instead"); 
  } 

  /*
   * Execute the del group command
   * 
   * @param ST the parameter for simulation
   * @param connection the instance of <code>Connection</code>
   * @return false becouse simulation must continue
   */
  public boolean runCommand(StringTokenizer ST, Connection connection) {
    int n;   // contain the number of parameter to change
    double value; // the new value for the parameter
    if (ST.countTokens()!=2) {
      connection.out.println("Invalid number of parameter");
      connection.out.flush();
      return false;  
    }
    n=(new Integer(ST.nextToken())).intValue();
    value=(new Double(ST.nextToken())).doubleValue();
    switch (n) {
      case 1:
        if (value<0.1 || value>25) {
          connection.out.println("Value for box mass is out of limit (0,1..25kg)");
          return false; 
        }
        Fisical.boxMass=value; 
        Fisical.boxWeight=value*Fisical.g; 
        connection.broadcast("Box mass changes to "+value+" Kg");
        connection.broadcast("%% 1 "+String.valueOf(Fisical.boxMass));
        break;
      case 2:
        if (value<5 || value>100) {
          connection.out.println("Value for box dimension is out of limit (5..100cm)");
          return false; 
        }
        Fisical.boxDim=value/100; 
        connection.broadcast("Box dimendion changes to "+value+" cm");
        connection.broadcast("%% 2 "+String.valueOf(Fisical.boxDim));
        break;
      case 3:
        if (value<1 || value>3) {
          connection.out.println("Value for bar length is out of limit (1..3m)");
          return false; 
        }
        Fisical.barLength=value; 
        connection.broadcast("Bar length changes to "+value+" m");
        connection.broadcast("%% 3 "+String.valueOf(Fisical.barLength));
        break;
    }
    return false;
  }
}


/**
 * Implement the ambient for robot bar boxs puhing
 */
class BarAmbient extends Ambient {
  public void buildStatic(Connection connection) {
    String staticVRML;

    staticVRML="# NEW STATIC_1 "+connection.readFile("Coord.wrl")+"\n";        
    //n=new Nodes(staticVRML, "STATIC_OWNER");
    //connection.nodesList.put(groupSignal,n);
    staticVRML+="# MODIFY STATIC_1 SFVEC3F  set_translation 0.1 0 -0.2 \n";
    staticVRML+="# MODIFY STATIC_1 SFROTATION set_rotation 0 1 0 0\n";               
    connection.out.println(staticVRML);
    connection.out.flush();
  }
}


/**
 * Implements the <i>class</i> that performes the server ambient
 */
class BarServerAmbient extends GenericServerAmbient {
 /**
   * Search (and execute) for tile message as:
   * <ul>
   * <li> GET POS   : get position of agents
   * <li> GET FOR1  : get force of sensor 1
   * <li> GET FOR2  : get force of sensor 2
   * <li> PUT VEL   : put new velocity of agents 
   * <li> PUT ANGLE : put new direction of robot
   * <li> PUT BEH   : put actual behavior we are using 
   * </ul>
   */
  public void runTask() {
    String msg;                // message read
    String procName;           // name of the sender
    String tok;                // actual token of string 
    StringTokenizer ST;        // string tokens of message
    int proc;                  // number associated with the sender  
    RobotState RS;             // robot state of sender agent  
  
    msg=clientQueue[0].get("Server",0);   
    ST=new StringTokenizer(msg," ",false);
    //find the sender
    procName=ST.nextToken();
    proc=(new Integer(ST.nextToken())).intValue();
    
    tok=ST.nextToken();
    if (tok.equals("GET")) {
      tok=ST.nextToken();
      RS=(RobotState)state.get(procName);
      if (tok.equals("POS")) {       
        clientQueue[proc].put("Server",String.valueOf(RS.massCenter.x)+" "+
	         String.valueOf(RS.massCenter.y)+"  "+
	         String.valueOf(RS.xAngle),proc);
      }
      if (tok.equals("FOR1")){
        clientQueue[proc].put("Server",String.valueOf(-Fisical.sensor1Costant*
                             (RS.sensor1Length-Fisical.sensor1Repose)),proc);
      }
      if (tok.equals("FOR2")){
        clientQueue[proc].put("Server",String.valueOf(-Fisical.sensor2Costant*
                             (RS.sensor2Length-Fisical.sensor2Repose)),proc);
      }
    }
    if (tok.equals("PUT")) {
      tok=ST.nextToken();
      RS=(RobotState)state.get(procName);
      if (tok.equals("VEL")) {
        RS.massCenterVel.x=(new Float(ST.nextToken())).floatValue();
        RS.massCenterVel.y=(new Float(ST.nextToken())).floatValue();
        state.put(procName,RS);
      }
      if (tok.equals("ANGLE")) {
        RS.xAngle=(new Float(ST.nextToken())).floatValue();
        state.put(procName,RS);      
      }
      if (tok.equals("BEH")) {
        RS.behavior=(new Integer(ST.nextToken())).intValue();
        state.put(procName,RS);
      }
    }
  }
} 

class BarServer extends Server {
  public BarServer(String argv[]) {
    super(argv);
  }
  
  protected void init() {
    // initialize all undefined variable
    about="Server for Robot Bar Boxs Pushing";
    resolutionTime=100;
    printerTime=100;
    agentsTime=200; 
    timeToSend=1;
    realTime=1;//30;
    GSV=new BarSendVRML();
    GSA=new BarServerAmbient();
    staticAmb=new BarAmbient();

    // insert client command available
    ExitClientCommand C1=new ExitClientCommand();
    clientCommands.put(C1.getName(),C1); 

    WhoClientCommand C2=new WhoClientCommand();
    clientCommands.put(C2.getName(),C2);

    NameClientCommand C3=new NameClientCommand();
    clientCommands.put(C3.getName(),C3);

    HelpClientCommand C4=new HelpClientCommand();
    clientCommands.put(C4.getName(),C4);

    AddClientCommand C5=new AddClientCommand();
    clientCommands.put(C5.getName(),C5);

    DelClientCommand C6=new DelClientCommand();
    clientCommands.put(C6.getName(),C6);

    AddFileClientCommand C7=new AddFileClientCommand();
    clientCommands.put(C7.getName(),C7);  

    DelFileClientCommand C8=new DelFileClientCommand();
    clientCommands.put(C8.getName(),C8);

    ModifyClientCommand C9=new ModifyClientCommand();
    clientCommands.put(C9.getName(),C9);

    FisicalClientCommand C10=new FisicalClientCommand();
    clientCommands.put(C10.getName(),C10);
  } 

  public static void main(String argv[]){
    new BarServer(argv);
  } 
}

