/**
 * @(#)TransportServer.java 1.1
 * @Author Adolfo Benin   1.0 (1997) all the work
 * @Author Tognon Stefano 1.1 (1998) only reconversion to simulator ambient 1.1
 *
 * This software was developing at Padova University in collaboration with
 * Padova CNR Ladseb in Italy.
 * THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
 *
 * This program creates the server (and the client) that are able to perform 
 * a robots cooperative transporting task using behavior based agents control.
 * See the "Adolfo Benin, Costruzione di un ambiente di simulazione per lo
 * studio del trasporto cooperativo di un oggetto mediante robot mobili,
 * Tesi di Laurea, Universit degli Studi di Padova, Marzo 1998" for more
 * details.
 */

package simulator;

import simulator.*;
import java.util.*;

/**
 * This <i>class</i> rappresent a 2D vector with some operation
 */ 
class Position {

  double x,y;

  /**
   * Initialize vector with initial value
   *
   * @param a the x value
   * @param b the y value
   */
  public Position(double a, double b) {
    this.x=a;
    this.y=b;
  }

  /**
   * Find the angle between this vector and the passed vector
   *
   * @param P the passed vector
   * @result the angle
   */
  public double findAngle(Position P) {
    return(Math.atan2(P.y-this.y,P.x-this.x));
  }

  /**
   * Calculate the distance beetween this vector and the passed vector
   *
   * @param P the passed vector
   * @result the distance
   */
  public double dist(Position P) {
    return (Math.sqrt((P.y-this.y)*(P.y-this.y)+(P.x-this.x)*(P.x-this.x)));
  }

  /**
   * Calculate the sum vector of this vector and the passed vector
   *
   * @param P the passed vector
   * @result the sum vector 
   */
  public Position sum(Position P) {
    Position pos=new Position(0,0);
    pos.x=this.x+P.x;
    pos.y=this.y+P.y;
    return pos;
  }

  /**
   * Calculate the difference between this vector and the passed vector
   *
   * @param P the passed vector
   * @result the difference vector 
   */
  public Position diff(Position P) {
    Position pos=new Position(0,0);
    pos.x=P.x-this.x;
    pos.y=P.y-this.y;
    return pos;
  }

  /**
   * Multiply the vector for a costant value
   *
   * @param a the costant value
   * @result the modified resulting vector
   */
  public Position multiply(double a) {
    Position pos=new Position(0,0);
    pos.x=a*this.x;
    pos.y=a*this.y;
    return pos;
  }
}


/**
 * this <i>class</i> contains information obout one agents in the simulator
 */
class AgentState {
  /**
   * position of agents is fixed SR
   */
  public Position posAg;     

  /**
   * velocity of agents 
   */ 
  public Position vel;  

  /**
   * relative position of end_effector
   */
  public Position posEndEff; 

  /**
   * Angle of robot in respect of basicentric SR
   */
  public double alpha;       
          
  /**
   * Initialize object with actual state of agents
   */                                          
  public AgentState(Position posAg, Position vel, Position posEndEff, float alpha) {
    this.posAg=posAg;
    this.vel=vel;
    this.posEndEff=posEndEff;
    this.alpha=alpha;
  }
}


/**
 * This <i>class</i> rappresent an object of simulation ambient
 */  
class ObjectState {
  /**
   * Position of object in fixed SR
   */
  public Position posObj;

  /**
   * Roration of object in respect of fixed SR
   */
  public float theta;    
  
  /**
   * Initilize object with object state
   *
   * @param posObj the position of object
   * @param theta the rotation of object
   */
  public ObjectState(Position posObj, float theta) {
    this.posObj=posObj;
    this.theta=theta;
  }
}  


/**
 * this is the agent that reppresent the behavior of robot
 */
class TransportAgent extends Agent {
  protected int I;
  protected int N;

  protected Queue[] agentQueue;
  protected String nameAg;
  
  private Position goal;
  private Position startPos;
  private double rot;
  private double Vmax=0.03;
  
  public TransportAgent(int i, String name, Queue[] agentQueue, Position goal, double rot, Position startPos) {
    nameAg=name;
    I=i;
    this.agentQueue=agentQueue;
    this.goal=goal;
    this.rot=rot;
    this.startPos=startPos;
    System.out.println(name+" StartPos x ==> "+startPos.x);
    System.out.println(name+" StartPos y ==> "+startPos.y);
    this.goal=this.goal.sum(new Position(startPos.x*Math.cos(rot)-startPos.y*Math.sin(rot),
                               startPos.x*Math.sin(rot)+startPos.y*Math.cos(rot)));
    System.out.println(name+" Goal x ==> "+this.goal.x);
    System.out.println(name+" Goal y ==> "+this.goal.y);   
    this.setPriority(MAX_PRIORITY);
    this.start();
  }
  
  /**
   * Execute a task of one agent robots
   */
  public void run() {
    int count=0;
    String msg="";
    Position H=new Position(0,0);
    Position P=new Position(0,0);
    Position V=new Position(0,0);
    boolean achieveGoal=false;
    double Eps=0.0025;
    int i;
    
    System.out.println("Agent running "+I);
    while (!achieveGoal) {
      this.suspend();
      System.out.println("AG"+I+" Start");
      // Sensing
      agentQueue[0].put("Ag"+I,nameAg+" "+I+" GET HAND",0);
      
      msg=agentQueue[I].get("AG"+I,I);
      H.x=(Float.valueOf(msg.substring(0,msg.indexOf(" ")))).floatValue();
      H.y=(Float.valueOf(msg.substring(msg.indexOf(" "),msg.length()))).floatValue();
      
      agentQueue[0].put("Ag"+I,nameAg+" "+I+" GET POS",0);

      msg=agentQueue[I].get("AG"+I,I);
      P.x=(Float.valueOf(msg.substring(0,msg.indexOf(" ")))).floatValue();
      P.y=(Float.valueOf(msg.substring(msg.indexOf(" "),msg.length()))).floatValue();
     
      // Planning
      if ((Math.abs(P.x-goal.x)>Eps) | (Math.abs(P.y-goal.y)>Eps)) {
        if ((Math.abs(H.x)>0.015) | (Math.abs(H.y)>0.015)) {
          V=H.multiply(10/3);
        } else {
            if ((Math.abs(P.x-goal.x)<=0.03) & (Math.abs(P.y-goal.y)<=0.03)) {
              V=P.diff(goal).multiply(10/3);
            } else {
                double norma=(P.diff(goal)).dist(new Position(0,0));
                V=P.diff(goal).multiply(Vmax/norma);
                V=V.sum(H.multiply(10/3));
              }
          }    
      } else {
          System.out.println("Achieving Goal!!!");
          achieveGoal=true;
          V.x=0;
          V.y=0;
        }
      //Actuating
      agentQueue[0].put("Ag"+I,nameAg+" "+I+" PUT VEL "+V.x+" "+V.y,0);
   
      System.out.println("AG"+I+" End");
    }
    V.x=0;
    V.y=0;
    agentQueue[0].put("Ag"+I,nameAg+" "+I+" PUT VEL "+V.x+" "+V.y,0);
  }
}


/**
 * Impements the method that calculate position of robot and send the vrml
 * scene
 */
class TransportSendVRML extends GenericSendVRML {
  private Position B0=new Position(0,0);
  private Position B1=new Position(0,0);
                        
  private Position vettDiff=new Position(0,0);
  private double num,den;
  private double alpha;
  private float theta;
  private int numProc;


  /**
   * Calculate the position of all object in the simulation ambient
   */
  public void runTask() {
    String str="";
    String name="";
    String objectName="";
      
    Enumeration objectSet=state.keys();
      
    // update state using actual velocity, modify robot position, calculate
    // new baricenter
    B0=B1;
    B1=new Position(0.0,0.0);
    numProc=0;
    while (objectSet.hasMoreElements()) {
      name=(String)objectSet.nextElement();
      if (name.indexOf("OGG")==-1) {
        numProc++;
        AgentState SA=(AgentState)state.get(name);
        SA.posAg=SA.posAg.sum(SA.vel.multiply(server.printerTime/1000.0));
        state.put(name,SA);
        B1.x+=SA.posAg.x;
        B1.y+=SA.posAg.y;
      } else {
          objectName=name;
        }
    }
    if (numProc != 0) {
      B1.x/=numProc;
      B1.y/=numProc;
      vettDiff=B0.diff(B1); 
    }
    //calculate inclination of object
    num=0.0;
    den=0.0;
    objectSet=state.keys();
    while (objectSet.hasMoreElements()) {
      name=(String)objectSet.nextElement();
      if (name.indexOf("OGG")==-1) {
        AgentState SA=(AgentState)state.get(name);
        alpha=SA.alpha;
        Position pos=vettDiff.diff(SA.posAg);
        num-=(SA.posAg.y*Math.cos(alpha)-SA.posAg.x*Math.sin(alpha));
        den-=(SA.posAg.y*Math.sin(alpha)+SA.posAg.x*Math.cos(alpha));
      }
    }
    theta=(float)Math.atan2(num,den);
    
    //set agents state
    if (!objectName.equals("")) {
      ObjectState SO=(ObjectState)state.get(objectName);
      SO.posObj=B1;
      SO.theta=theta;
      state.put(objectName,SO);
    }
     
    //calculate new position of End-Effector
    objectSet=state.keys();
    while (objectSet.hasMoreElements()) {
      name=(String)objectSet.nextElement();
      if (name.indexOf("OGG")==-1) {
        AgentState SA=(AgentState)state.get(name);
        alpha=SA.alpha+theta+Math.PI;
        Position pee=new Position(B1.x+Math.cos(alpha),B1.y+Math.sin(alpha));
        SA.posEndEff=pee;
        state.put(name,SA);
      }
    }
  }
 
  /**
   * Send the new vrml schene to all the connected clients
   */
  public void sendVRML() {
    String objectName="";
    String str="";
    Enumeration objectSet=state.keys();

    while (objectSet.hasMoreElements()) {
      objectName=(String)objectSet.nextElement();
      if (objectName.endsWith("OGG")) {
        ObjectState SO=(ObjectState)state.get(objectName);
        str+="# MODIFY "+objectName+" SFVEC3F set_translation "
             +SO.posObj.x+" 0 "+(-SO.posObj.y)+"\n";
        str+="# MODIFY "+objectName+" SFROTATION set_rotation 0 1 0 "
             +SO.theta+"\n";
      } else {
          AgentState SA=(AgentState)state.get(objectName);
          str+="# MODIFY "+objectName+" SFVEC3F set_translation "
               +SA.posAg.x+" 0 "+(-SA.posAg.y)+"\n";
          str+="# MODIFY "+objectName+" SFROTATION set_rotation 0 1 0 "
               +(SA.alpha+Math.PI+theta)+"\n";
        }                     
    }  
    broadcast(str);
    System.out.println("Baricenter 0 =("+(float)B0.x+","+(float)B0.y+")");
    System.out.println("Baricenter 1 =("+(float)B1.x+","+(float)B1.y+")");
  }
}


/**
 * Implements the <i>class</i> that performes the server ambient
 */
class TransportServerAmbient extends GenericServerAmbient {
  /**
   * Search (and execute) for tile message as:
   * <ul>
   * <li> GET POS  : get position of agents
   * <li> GET VEL  : get velocity of agents
   * <li> GET HAND : get relative delta of end_effector
   * <li> PUT VEL  : put new velocity of agents 
   * </ul>
   */
  public void runTask() {
    int proc;
    StringTokenizer ST;
    String tok="";
    String msg;
    String procName="";
    AgentState SA;
    
    msg=clientQueue[0].get("Server",0);   
    ST=new StringTokenizer(msg," ",false);

    //find the sender
    procName=ST.nextToken();
    proc=(new Integer(ST.nextToken())).intValue();
      
    tok=ST.nextToken();
    if (tok.equals("GET")) {
      tok=ST.nextToken();
      SA=(AgentState)state.get(procName);
      if (tok.equals("POS")) {
        clientQueue[proc].put("Server",String.valueOf(SA.posAg.x)+" "+String.valueOf(SA.posAg.y),proc);
      }
      if (tok.equals("VEL")) {
        clientQueue[proc].put("Server",String.valueOf(SA.vel.x)+" "+String.valueOf(SA.vel.y),proc);
      }
      if (tok.equals("HAND")) {
        double x,y;
        x=SA.posEndEff.x-SA.posAg.x;
        y=SA.posEndEff.y-SA.posAg.y;
        clientQueue[proc].put("Server",String.valueOf(x)+" "+String.valueOf(y),proc);
      }
    }  
    if (tok.equals("PUT")) {
      SA=(AgentState)state.get(procName);
      tok=ST.nextToken();
      if (tok.equals("VEL")) {
        SA.vel.x=(new Float(ST.nextToken())).floatValue();
        SA.vel.y=(new Float(ST.nextToken())).floatValue();
      }
      state.put(procName,SA);
    }
  } 
}


/**
 * Del a group of robots from simulation
 */
class DelgroupClientCommand extends ClientCommand {
  /**
   * Initilaize object for DELGROUP command
   */
  public DelgroupClientCommand() {
    super("DELGROUP delete robots",
          "DELGROUP group_name  : delete robots\n"+
          "group_name=[string]=the name of the group");
  } 

  /**
   * Del robot from simulation
   *
   * @param ST next token = name of group
   * @return false
   */
  public boolean runCommand(StringTokenizer ST, Connection connection) {
    Nodes n;

    connection.SV.suspend();
    String groupName=ST.nextToken();
    if (connection.processesList.containsKey(groupName+"1")) {
      n=(Nodes)connection.nodesList.get(groupName+"1");
      if (n.userName.equals(connection.clientName)) {
        int i=1;
        TransportAgent A=((TransportAgent)connection.processesList.get(groupName+String.valueOf(i)));
        while (A!=null) {
          A.stop();
          connection.processesList.remove(groupName+String.valueOf(i));
          connection.nodesList.remove(groupName+String.valueOf(i));
          connection.state.remove(groupName+String.valueOf(i));
          connection.broadcast("DEL "+groupName+String.valueOf(i));
          i++;
          A=(TransportAgent)connection.processesList.get(groupName+String.valueOf(i));
        }
        connection.nodesList.remove(groupName+"OGG");
        connection.state.remove(groupName+"OGG");
        connection.broadcast("# DEL "+groupName+"OGG");
      } else {
          connection.out.println("Can't erase the group becouse it's of another uses!!!");
        }
    } else {
        connection.out.println("The group don't exist");
      }
    connection.SV.resume();
    return false;
  }
}


/**
 * Add the group of robot for simulation
 */
class AddgroupClientCommand extends ClientCommand {
  /**
   * Initialize object for ADDGROUP command
   */
  public AddgroupClientCommand() {
    super("ADDGROUP  : adding robots",
          "ADDGROUP n_robot group_name wrl_file start_x start_y end_x end_y"+
          " angle  : adding robots\n"+
          "n_robot=[number]=number of robot in simualtion\n"+
          "group_name=[string]=name of group\n"+
          "wrl_file=[string]=name of vrml file\n"); 
  }

  /**
   * Add robot for simulation 
   *
   * @param ST the parameter for simulation
   * @return false becouse simulation must continue
   */
  public boolean runCommand(StringTokenizer ST, Connection connection) {
    int nRob=0;
    int i=0;
    Nodes n;
    String vrmlString=""; 
    float[] SP=new float[2];
    float[] EP=new float[2];

    connection.SV.suspend();  
 
    nRob=(new Integer(ST.nextToken())).intValue();
    String groupName=ST.nextToken();
    n=(Nodes)connection.nodesList.get(groupName+String.valueOf(0));
    if (n==null) {
      String groupType=ST.nextToken();

      //read initial position
      for (i=0;i<2;i++) {
        SP[i]=(Float.valueOf(ST.nextToken())).floatValue();
      }
      // SV.setStart(new Position(SP[0],SP[1]));
 
      //read final position
      for (i=0;i<2;i++) {
        EP[i]=(Float.valueOf(ST.nextToken())).floatValue();
      }
      float rot=(Float.valueOf(ST.nextToken())).floatValue();

      //build robots in VRML world
      double theta=(float)(2*Math.PI/nRob);
      double angle;
      vrmlString="";
      for (i=1;i<=nRob;i++) {
        vrmlString+="# NEW "+groupName+String.valueOf(i)+" "+connection.readFile(groupType+".wrl")+"\n";
        n=new Nodes(vrmlString,connection.clientName);
        connection.nodesList.put(groupName+String.valueOf(i),n);
        connection.presentMessage[i]=new Semaphore(0,"");
        connection.availableCells[i]=new Semaphore(connection.dimQueue,"");
        connection.processesQueue[i]=new Queue(connection.dimQueue,connection.availableCells,connection.presentMessage);
        angle=(theta*(i-1));
        connection.state.put(groupName+String.valueOf(i),new AgentState(
             (new Position(SP[0]+Math.cos(angle),SP[1]+Math.sin(angle))),
             (new Position(0,0)),
             (new Position(SP[0]+Math.cos(angle),SP[1]+Math.sin(angle))), (float)angle));            
        vrmlString+="# MODIFY "+groupName+String.valueOf(i)+" SFVEC3F set_translation "
                   +(SP[0]+Math.cos(angle))+" "+0+" "+(-(SP[1]+Math.sin(angle)))+"\n"
                   +"# MODIFY "+groupName+String.valueOf(i)+" SFROTATION set_rotation 0 1 0 "
                   +(float)(angle+Math.PI)+"\n";
      }
      connection.broadcast(vrmlString);

      //build object to transport in VRML world
      vrmlString="# NEW "+groupName+"OGG"+" "+connection.readFile("croce.wrl");
      n=new Nodes(vrmlString,connection.clientName);
      connection.nodesList.put(groupName+"OGG",n);
      connection.state.put(groupName+"OGG",new ObjectState(new Position(SP[0],SP[1]),0));
      connection.broadcast(vrmlString+"\n"+"# MODIFY "+groupName+"OGG"+" SFVEC3F set_translation "
                           +SP[0]+" "+0+" "+(-SP[1])+"\n");
      //Create Agents processes
      for (i=1;i<=nRob;i++) {
        angle=(2*Math.PI*(i-1)/nRob);
        TransportAgent A=new TransportAgent(i,groupName+String.valueOf(i),connection.processesQueue,
                             new Position(EP[0],EP[1]),rot,
                             new Position(Math.cos(angle),Math.sin(angle)));
        connection.processesList.put(groupName+String.valueOf(i),A);
      }
    } else {
        connection.out.println("The group "+groupName+" already exists!!!");
        connection.out.flush();
      }
    connection.SV.resume();
    return false;
  }
}


class TransportAmbient extends Ambient {
  public void buildStatic(Connection connection) {
  }
}

class TransportServer extends Server {
  public TransportServer(String argv[]) {
    super(argv);
  }
  
  protected void init() {
    // initialize all undefined variable
    about="Server of the ...";
    resolutionTime=100;
    printerTime=100;
    agentsTime=300; 
    timeToSend=1;// 4;
    realTime=1;
    GSV=new TransportSendVRML();
    GSA=new TransportServerAmbient();
    staticAmb=new TransportAmbient();

    // insert client command available
    ExitClientCommand C1=new ExitClientCommand();
    clientCommands.put(C1.getName(),C1); 

    WhoClientCommand C2=new WhoClientCommand();
    clientCommands.put(C2.getName(),C2);

    NameClientCommand C3=new NameClientCommand();
    clientCommands.put(C3.getName(),C3);

    HelpClientCommand C4=new HelpClientCommand();
    clientCommands.put(C4.getName(),C4);

    AddClientCommand C5=new AddClientCommand();
    clientCommands.put(C5.getName(),C5);

    DelClientCommand C6=new DelClientCommand();
    clientCommands.put(C6.getName(),C6);

    ModifyClientCommand C7=new ModifyClientCommand();
    clientCommands.put(C7.getName(),C7);  

    AddgroupClientCommand C8=new AddgroupClientCommand();
    clientCommands.put(C8.getName(),C8);

    DelgroupClientCommand C9=new DelgroupClientCommand();
    clientCommands.put(C9.getName(),C9);

    ModifyClientCommand C10=new ModifyClientCommand();
    clientCommands.put(C10.getName(),C10);
  } 

  public static void main(String argv[]){
    new TransportServer(argv);
  } 
}

