/**
 * @(#)Simulator.java 1.1
 * @Author 1.0 Adolfo Benin    (1997) first base version
 * @Author 1.1 Tognon Stefano  (1998) global restructuring+new features
 *
 * This software was developing at Padova University in collaboration with
 * Padova CNR Ladseb in Italy.
 * THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
 *
 * This package want to create an basic simulation ambient that can be
 * execute in a server computer and than some client can access to the
 * ambient using other computer connected to the server by INTERNET.
 * To use this product is necessary a Netscape browser and an VRML plug-in
 * that support EAI (Esternal Authority interface).
 * See the just available implementation of ambient for more details about
 * how can be made with this simulator.
 */

package simulator;

import java.lang.*;
import java.util.*;
import java.net.*;
import java.io.*;
import java.applet.*;
import java.awt.*;
import vrml.external.Node;
import vrml.external.Browser;
import vrml.external.exception.*;
import vrml.external.field.*;
import vrml.external.field.FieldTypes;
//import netscape.javascript.JSObject;


/**
 * This class is a multi-value semaphore with P and V operations
 * that performs a process suspension if there's no other available
 * resorces.
 * The P and V operation use a internal Java mutex to avoid conflicts.
 */
class Semaphore {
  /**
   * The actual number of available resorces
   */
  int value;

  /**
   * The actual number of process that are suspend
   */
  int N=0;

  /**
   * This string contain the Owner of the process that call a P or V
   * operation, but it is never used.
   */
  String O;

  /**
   * Initialize the semaphore with starting resorces and owner
   * string of process
   *
   * @param initSem  the initial resources number
   * @param Owner    a string for the owner (actualy never used)
   */
  public Semaphore(int initSem, String Owner) {
    value=initSem;
    O=Owner;
  }

  /**
   * Perform a P operation (if there's no other resorces the process will
   * be suspend)
   */
  public synchronized void P() {
    if (value>0) {
	   value--;
    } else {
        try {
          N++;
          wait();
        } catch (InterruptedException e){}
      }
  }

  /**
   * Perform a V operation (if there's suspended processes, it will resume
   * one of them)
   */
  public synchronized void V() {
    if (value==0) {
      if (N==0) {                 //there's no process in queue
        value++;
      } else {
          N--;
          notify();
        }
    } else value++;
  }
}



/**
 * This class implements a circular-shared-queue for string elements.
 * The acesses to the queue are im mutual exclusion to prevent theads conflict.
 * To prevent that the queue become full two semaphore array are required and
 * are used as producer-consumer. So the initialization of this semaphores
 * are made by user that can use for examples a queue of 10 elements, but
 * 6 of one type, and 4 of another type. When one of this type is finished
 * the process is suspend until the resources it needs become free.
 */
class Queue {
  /**
   * This is the array that contain the string values
   */
  private String[] Q;

  /**
   * This contain the dimansion of the queue
   */
  int numElem;

  /**
   * The actual number of elements that the queue contains
   */
  int countElem;

  /**
   * The pointer to head of queue
   */
  int head;

  /**
   * The pointer to tile of queue
   */
  int tile;

  /**
   * This is the array of semaphore that contains the available resources
   * of the index-array type
   */
  protected Semaphore[] availableCells;

  /**
   * This is the array of semaphore that contains the occupied resources
   * of the index-array type
   */
  protected Semaphore[] presentMessage;

  /**
   * the mutex to prevent conflict in shared queue
   */
  protected Semaphore mutex = new Semaphore(1,"");

  /**
   * Initialize the shared queue
   *
   * @param numElem        number of total element of the cell
   * @param availableCells the semaphore array (must be just initialized) for
   *                       available resources
   * @param presentMessage the semaphore array (must be just initialized) for
   *                       occupied resources
   * @see Queue#availableCells
   * @see Queue#presentMessage
   */
  public Queue(int numElem, Semaphore[] availableCells, Semaphore[] presentMessage) {
    this.numElem=numElem;
    Q=new String[numElem];
    head=0;
    tile=0;
    countElem=0;
    this.availableCells=availableCells;
    this.presentMessage=presentMessage;
  }

  /**
   * Put the string s in the queue using the dest index of array
   *
   * @param name the name of process that call put (actual never used)
   * @param s    the string to insert in the queue
   * @param dest the index of array (type of resources)
   */
  void put(String name, String s, int dest) {
    availableCells[dest].P();

    mutex.P();
    Q[head]=s;
    head=(head+1)%numElem;
    countElem++;
    mutex.V();

    presentMessage[dest].V();
  }

  /**
   * Get a string from the sorg index of array in the queue
   *
   * @param name the name of process that call put (actual never used)
   * @param sorg the index of array (type of resources)
   * @return the string from the queue
   */
  String get(String name, int sorg) {
    String Ret="";
    presentMessage[sorg].P();

    mutex.P();
    Ret=Q[tile];
    tile=(tile+1)%numElem;
    countElem--;
    mutex.V();

    availableCells[sorg].V();

    return Ret;
  }
}


/**
 * This class run a thread that activate the vrml printer and the agents
 * processes every simulation time.
 * It use a resolution time set by user to count for departed time and a system to
 * set if we use a real time, accelerated time or slow down time for simulation.
 */
class ProcessesHandler extends Thread {
  /**
   * Contains the list of agents processes
   */
  protected Hashtable processesList;

  /**
   * Contains the vrml printer proccess
   */
  Thread printer;

  /**
   * Contains the server ambient process
   */
  protected ServerAmbient serverAmbient;

  /**
   * This variable set the real time in the simulation with a 1 value.
   * If the value insert is better than the simulation is slow down, else the simulation
   * is fast. For example if <code>resolutionTime</code> is 100msec. and this is set to
   * 3, this implies that we update the simulation event us 100msec was departed, but
   * this is made in 300msec, so slow system can execute correctly the simulation even
   * if no real time is performed.
   */
  int realTime;

  /**
   * This is the resolution time in milliseconds of the process
   */
  int resolutionTime;

  /**
   * Every <code>printerTime</code> milliseconds time the printer is resumed
   * (this time must be multiple of <code>resolutionTime</code>)
   */
  int printerTime;

  /**
   * Every <code>agentsTime</code> milliseconds time the agents are resumed
   * (this time must be multiple of <code>resolutionTime</code>)
   */
  int agentsTime;

  /**
   * Initialize the processes handler and start thread
   *
   * @param processesList contains the list of agents processes
   * @param printer contains the vrml printer process
   * @param serverAmbient contains the server ambient
   * @param resolutionTime as defined in <i>class</i> variable
   * @param printerTime as defined in <i>class</i> variable
   * @param agentsTime as defined in <i>class</i> variable
   * @param realTime as defined in <i>class</i> variable
   */
  public ProcessesHandler(Hashtable processesList, Thread printer, ServerAmbient serverAmbient,
                          int resolutionTime, int printerTime, int agentsTime, int realTime) {
    this.processesList=processesList;
    this.printer=printer;
    this.serverAmbient=serverAmbient;
    this.resolutionTime=resolutionTime;
    this.printerTime=printerTime;
    this.agentsTime=agentsTime;
    this.realTime=realTime;
    this.start();
  }

  /**
   * resume the printer processes and the agents processes at every
   * simulation time.
   * The <code>resume</code> to <code>serverAmbient</code> seems to be
   * necessary becouse when it will activate with an <code>notify</code>
   * (and the process is suspend in a <code>queue</code>) not always
   * the process become active and so the agents processes will be wait
   * forever.
   */
  public void run() {
    int count=0;
    int i=0;

    while (true) {
      try {
        this.sleep(resolutionTime*realTime);
      } catch (Exception e) {}
      count+=resolutionTime;

      // resume the agents processes: they must complete their work
      // before that printer process update the state of the ambient
      if ((count%agentsTime)==0) {
        Enumeration ListProc = processesList.keys();
        while (ListProc.hasMoreElements()) {
          ((Thread)processesList.get(ListProc.nextElement())).resume();
	}
        serverAmbient.resume();
      }

      // resume the printer process
      if ((count%printerTime)==0) {
        printer.resume();
        serverAmbient.resume();
      }
    }
  }
}


/**
 * This is an <i>abstract class</i> created to be extended for making the
 * <code>SendVRML</code> <i>class</i> able to operate with your simulation
 * ambient and your client send VRML protocol.
 * The derived <i>class</i> must be like this:
 * <p><blockquote><pre>
 *    class MySendVRML extend GenericSendVRML{
 *      // this is the list of your variable
 *      ...
 *      public void runTask() {
 *        //this is the procedure called with late-bilding method
 *        //that you must use to your simulation strategy
 *      }
 *      public void sendVRML() {
 *        //this is the procedure called with late-bilding method
 *        //that you must use to your send VRML
 *      }
 *    }
 * </pre></blockquote>
 */
abstract class GenericSendVRML {
  /**
   * This contains the list of connected user and will be always used for
   * a simulation strategy in <code>runTask</code>. It's the same that you
   * can extract from <code>server</code> variable.
   */
  protected Hashtable connectedUsers;

  /**
   * This contains the list of the state of simulation and will be always
   * used for a simulation strategy in <code>runTask</code>. It's the same
   * that you can extract from <code>server</code> variable.
   */
  protected Hashtable state;

  /**
   * Contains the instance to the <code>Server</code> <i>class</i>
   */
  protected Server server;

  /**
   *  This initialize the internal variable
   *
   * @param server the instance to <code>Server</code>
   */
  public void init(Server server) {
    this.server=server;
    this.connectedUsers=server.connectedUsers;
    this.state=server.state;
  }

  /**
   * This is the procedure called by <code>SendVRML</code> <i>thread</i> when
   * we must updating the simulator state.
   * It's <i>abstract</i> so you must implements it.
   * This method should use like this: <p>
   * <ul>
   * <li>scan in <code>state</code> list for the objects that are present in
   * the simulator
   * <li>update the information of those objects respecting the fisical lows
   * <li>store objects in <code>state</code>
   * </ul>
   */
  public abstract void runTask();

  /**
   * This is the procedure called by <code>sendVRML</code> <i>thread</i> when
   * we send the VRML view to the connected clients.
   * It's abstract so you must implements it.
   * This method shuld use like this: <p>
   * <ul>
   * <li>scan in <code>state</code> list for the object that are present in
   * the simulator
   * <li>if you see that an object had changed the position (or some ather
   * caratteristic), you must send a string containing the new information
   * with <code>broadcast</code> routine. The string must respect the sintax
   * of <code>runCommand</code> of <code>Client</code> <i>class</i>
   * </ul>
   *
   * @see Client#runCommand
   */
  public abstract void sendVRML();

  /**
   * Broadcasting a string to all the connected clients
   *
   * @param s the string to broadcasting
   */
  public void broadcast (String s) {
    Enumeration e=connectedUsers.keys();
    while (e.hasMoreElements())	{
      String key=(String) e.nextElement();
      Connection thisClient=(Connection) connectedUsers.get(key);
      thisClient.out.println(s);
      thisClient.out.flush();
    }
  }
}


/**
 * This is the <i>class</i> that send a vrml dinamic view of the simulation.
 * It must execute this two tasks:
 * <ul>
 * <li> calculate the state of all object in the ambient using the actual
 * available data of them (velocity, time, ...)
 * <li> send how the vrml scene had changed to the connected clients
 * </ul>
 * To do this you must derived the <code>GenericSendVRML</code> <i>class</i>
 * and define the <code>runTask</code> and <code>sendVRML</code> method,
 * becouse this <i>class</i> refer to that methods.
 *
 * @see GenericSendVRML
 */
class SendVRML extends Thread {
  /**
   * Contains an istance of your send vrml procedures
   */
  protected GenericSendVRML GSV;

  /**
   * Contains an instance of <code>Server</code> <i>class</i>
   */
  protected Server server;

  /**
   * Indicate how many times we must attend before send a vrml scene to the
   * client. So the effective time to attend before send is:
   * <code>printerTime*timeToSend</code>
   *
   * @see ProcessesHandler#printerTime
   */
  protected int timeToSend;

  /**
   * Initialize the internal variable and start thread body
   *
   * @param server the istance of the server
   * @param GSV the istance of your send vrml
   */
  public SendVRML(Server server, GenericSendVRML GSV) {
    this.server=server;
    timeToSend=server.timeToSend;
    this.GSV=GSV;
    this.start();
  }

  /**
   * This thread calles the <code>runTask</code> method of
   * <code>GenericSendVRML</code> and after <code>timeToSend</code> times
   * calles the <code>sendVRML</code> method too.
   *
   * @see GenericSendVRML#runTask
   * @see GenericSendVRML#sendVRML
   * @see timeToSend
   */
  public void run() {
    int count=0;

    GSV.init(server);
    while (true) {
      this.suspend();
      count++;
      GSV.runTask();
      if (count%timeToSend==0) {
        GSV.sendVRML();
      }
    }
  }
}


/**
 * This is an <i>abstract class</i> to be implemented for you server of ambient.
 * So you must implements the <code>runTask</code> method. <p>
 * The rule of this <i>class</i> is that all the variables (position, velocity,
 * forces...) of all the dimamic objects (robots, ...) are to be stored in
 * the <code>state</code> <i>hashtable</i> and only this and
 * <code>GenericSendVRML</code>can manage the contents to prevent conflict with
 * thread agents.
 * Infact the agents that need to have/give some information about itself have
 * to require/send this information using a protocol (that you must implements)
 * that must use <code>clientQueue</code> tile. <p>
 * The convenction that must be observe is that the array position <i>0</i>
 * of tile is used to notify to this server that an agents want to set/get
 * information obout itself, and than it used a <i>i</i> array position of
 * tile to change information (so the <i>i</i> position must be different for
 * different agents).
 *
 * see @GenericSendVRML
 */
abstract class GenericServerAmbient {
  /**
   * Rappresents the dinamic state of the simulation ambient.
   * The information that are stored in it are required by the agents
   * for calculating new positions.
   */
  protected Hashtable state;

  /**
   * In this tile the agents send required about information useful to
   * calculate their position and the <code>ServerAmbient</code> store and
   * read information using this tile.
   */
  protected Queue[] clientQueue;

  /**
   * Initialize object variables
   *
   * @param state the state of all object in virtual ambient
   * @param clientQueue the tile where information must be send between
   *        agents and server
   */
  public void init(Hashtable state, Queue[] clientQueue) {
    this.state=state;
    this.clientQueue=clientQueue;
  }

  /**
   * This is abstract becouse you must implements it to have your server
   * ambient. Infact this method must like this:
   * <blockquote><pre>
   *     public void runTask() {
   *       String msg;
   *       StringTokenizer ST;
   *       int i;
   *
   *       msg=clientQueue[0].get("Server",0);
   *       ST=new StringTokenizer(Msg," ",false);
   *       // analize ST token to get information about command
   *       // and send/retrive information to/for agents
   *       if (ST.nextToken()=="GET") {
   *         ...
   *         clientQueue[i].put("Server",...);
   *       }
   *       ...
   *     }
   * </pre></blockquote><hr>
   */
  public abstract void runTask();
}


/**
 * This <i>class</i> is used to comunicate between the agents their
 * position (velocity...) and the position of all other object there's in the
 * simulator.
 * It used a <code>GenericServerAmbient</code> to do this, so you can
 * implement the function you need.
 *
 * @see GenericServerAmbient
 */
class ServerAmbient extends Thread {
  /**
   * Rappresents the dinamic state of the simulation ambient.
   * The information that are stored in it are required by the agents
   * for calculating new positions.
   */
  protected Hashtable state;

  /**
   * In this tile the agents send required about information useful to
   * calculate their position and the <code>ServerAmbient</code> store and
   * read information using this tile.
   */
  protected Queue[] clientQueue;

  /**
   * Instances your server ambient processes
   */
  protected GenericServerAmbient GSA;

  /**
   * Initialize object and start thread
   *
   * @param state the state of all object in virtual ambient
   * @param clientQueue the tile where information must be send between
   *        agents and server
   * @param GSA your class that execute the server ambient processes
   */
  public ServerAmbient(Hashtable state, Queue[] clientQueue, GenericServerAmbient GSA) {
    this.clientQueue=clientQueue;
    this.state=state;
    this.GSA=GSA;
    this.setPriority(NORM_PRIORITY+2);
    this.start();
  }

  /**
   * This thread loop in a <code>runTask</code> of
   * <code>GenericServerAmbient</code> for making your actions, but first
   * time it call your <code>init</code>.
   *
   * @see GenericServerAmbient
   */
  public void run() {
    GSA.init(state, clientQueue);
    while (true) {
      GSA.runTask();
    }
  }
}


/**
 * This <i>class</i> is provided becouse you can add some static
 * object in the simulator when a connection is made.
 * It's abstract so you must implement it.
 * There's only one method in this <i>class</i>:
 * <code>buildStatic</code> that is called by the
 * <code>Connection</code>'s initilization <i>class</i>
 */
abstract class Ambient {

  /**
   * This constructor is here only to remember you that your
   * <i>class</i> may have an initialization part that can be called
   * only in the <code>Server</code> initialization.
   */
  public Ambient () {
  }

  /**
   * This method is that build the static ambient.
   * You may create new vrml node and add them to <code>nodesList</code>
   * of <code>Connection</code>.
   *
   * @param connection a reference to <code>Connection</code> for
   *        using it's object.
   * @see Connection#nodesList
   */
  public abstract void buildStatic(Connection connection);
}


/**
 * Implements a base function of a server as a thread.
 * To use this class, you must extend it like this:
 * <p><code><pre>
 *     class MyServer extends Server{
 *       public MyServer(String argv[]) {
 *         super(argv);   // this is the constructor that call the super
 *       }
 *
 *       protected void init() {
 *          // initialize all undefined variable
 *          about="Server ...";
 *          resolutionTime=100;
 *          printerTime=100;
 *          agentsTime=300;
 *          timeToSend=4;
 *          GSV=new MySendVRML; //this is the implementation of GenericSendVRML
 *          GSA=new MyServerAmbient; //this is the implementation of GenericServerAmbient
 *          staticAmb=new MyAmbient(); //this is the implementation of Ambient
 *          ExitClientCommand C1=new ExitClientCommand();
 *          clientCommands.put(C1.getName(),C1); //insert the client commands
 *          ...
 *       }
 *
 *       public static void main(String[] argv){
 *         new MyServer(argv);
 *       }
 *     }
 * </pre></code>
 * <p>
 * How you must see your <i>class</i> must only initilize all the undefined
 * variables and creates your object that performes your commands (refer to
 * the other <i>abstract class</i> for details) for the simulation.
 *
 * @see SendVRML
 * @see GenericSendVRML
 * @see ServerAmbient
 * @see GenericServerAmbient
 * @see Connection
 * @see Ambient
 */
abstract class Server extends Thread {
  /**
   * This is undefined string avoided to contain the about information of
   * server program.
   */
  public static String about;

  /**
   * This variable set the real time in the simulation with a 1 value.
   * If the value insert is better than the simualtion is slow down, else the simulation
   * is fast. For example if <code>resolutionTime</code> is 100msec. and this is set to
   * 3, this implies that we update the simulation event us 100msec was departed, but
   * this is made in 300msec, so slow system can execute correctly the simulation even
   * if no real time is performed.
   */
  public static int realTime=1;

  /**
   * This is undefined integer avoided to contain the resolution of the time
   * processes handler
   */
  public static int resolutionTime;

  /**
   * This is undefined integer avoided to contain the time to resume the
   * printer process. It must be multiple of <code>resolutionTime</code>
   * variable.
   */
  public static int printerTime;

  /**
   * This is undefined integer avoided to contain the time to resume the
   * agents. It must be multiple of <code>resolutionTime</code> variable.
   */
  public static int agentsTime;

  /**
   * Indicate how many times we must attend before send a vrml scene to
   * client. So the effective time to attend before send is:
   * <code>printerTime*timeToSend</code>
   *
   * @see ProcessesHandler#printerTime
   */
  public int timeToSend;

  /**
   * This is undefined process that is to be used with <code>SendVRML</code>
   */
  public static GenericSendVRML GSV;

  /**
   * This is undefined process that is to be used with
   * <code>ServerAmbient</code>
   */
  public static GenericServerAmbient GSA;

  public static final  int DEFAULT_PORT = 6472;
  public static final  int maxNumProc   = 100;

  /**
   * Max dimension for the queue
   */
  public static final  int dimQueue     = 100000;

  /**
   * contains the port number uses to create a connection with socket
   */
  protected int port=DEFAULT_PORT;

  /**
   * Contains the server socket istance to permit the comunication
   */
  protected ServerSocket server_socket;

  /**
   * Contains the list of client users that are connected
   */
  protected Hashtable connectedUsers;

  /**
   * Contains the list of nodes of virtual world
   */
  protected Hashtable nodesList;

  /**
   * Contains the list of agents processes
   */
  protected Hashtable processesList;

  /**
   * Contains the process for serving an ambient
   */
  protected ServerAmbient  SA;

  /**
   * Contains the time handler of processes
   */
  protected ProcessesHandler GP;

  /**
   * This is the thread that updates the simulator state and send the VRML
   * view to the client
   */
  protected SendVRML SV;

  /**
   * This is the array of semaphore that contains the available resources
   * of the index-array type
   */
  protected Semaphore[] availableCells;

  /**
   * This is the array of semaphore that contains the occupied resources
   * of the index-array type
   */
  protected Semaphore[] presentMessage;

  /**
   * A queue of message changed by client agents and the server
   */
  protected Queue[] processesQueue;

  /**
   * This is the <i>abstract class</i> to build your static ambient
   */
  protected Ambient staticAmb;

  /**
   * Contains instance of the actual states of agents
   */
  protected Hashtable state;

  /**
   * Contains the list of commands that the client can execute. It must be
   * initiailzed by you when derived this class.
   */
   protected Hashtable clientCommands=new Hashtable();

  /**
   * This abstract method must be implemented by the derived class.
   * It is called by the <code>Server</code> constructor to permits to
   * define the elements that are not initialized.
   * So the derived class may have a constructor that calls Server's
   * and the late bilding process calls your just defined <code>init</code>.
   */
  protected abstract void init();

  /**
   * Analize the command passes as string
   *
   * @param args string that contain the server initial parameters
   */
  public Server(String args[]) {
    init();
    System.out.println(about);
    if (args.length==1) {
      try {
        port = Integer.parseInt(args[0]);
      } catch (NumberFormatException e) {   //if not int => error
          helpScreen();
        }
    } else port = DEFAULT_PORT;             //set port to default

    System.out.println ("Actual setting: "
                       + "\nport                     " + port
                       + "\n\n\n... Server is active and attends connection...");

    //activate the socket to listen for connection
    try	{
      server_socket = new ServerSocket(port);
    } catch (IOException e) {
	fail(e, "[ERROR] I don't be able to create the server-socket");
      }

    connectedUsers=new Hashtable();
    nodesList=new Hashtable();
    processesList=new Hashtable();
    availableCells=new Semaphore[maxNumProc];
    presentMessage=new Semaphore[maxNumProc];
    processesQueue=new Queue[maxNumProc];
    state=new Hashtable();
    this.setPriority(MIN_PRIORITY);
    this.start();
  }

  /**
   * Display a help screen
   */
  public static void helpScreen() {
    System.out.println (about+ "Usage:\n\n"
                             + "     java Server port \nwhere:\n"
                             + "     - port   =  server port number\n");
    System.exit(1);
  }

  /**
   * Execute the server thread: listen for a client connection and made it
   * The priority of this thread is set to MIN_PRIORITY by the default
   * contructor.
   */
   public void run() {
     //Activate ServerAmbient
     presentMessage[0]=new Semaphore(0,"");
     availableCells[0]=new Semaphore(dimQueue,"");
     processesQueue[0]=new Queue(dimQueue,availableCells,presentMessage);
     SA=new ServerAmbient(state,processesQueue,GSA);

     //Activate processes handler and SendVRML
     SV=new SendVRML(this, GSV);
     GP=new ProcessesHandler(processesList,SV,SA,resolutionTime,printerTime,agentsTime,realTime);

     try {
       //attending a client connnection and create a new Thread
       while(true) {
         Socket client_socket=server_socket.accept();
         Connection c=new Connection(client_socket, this);
       }
     } catch (IOException e) {
         fail(e, "[ERROR] when the server was attending a connection with a client");
       }
  }

  /**
   * Print fail information and exit from server program
   *
   * @param e   the exception that fails server
   * @param msg the message to print in err file
   */
  public static void fail(Exception e, String msg) {
    System.err.println(msg + "\n  [java error code]: " +  e);
    System.exit(1);
  }
}


/**
 * This is the class that mantains the connection between the client and
 * server and executes the comunication with it
 */
class Connection extends Thread {
  /**
   * Contains the list of commands available for clients. It will be
   * initilaized using <code>clientCommands</code> <i>Hashtable</i>.
   */
  public static String availableCommands="";

  /**
   * Contains the <i>socket</i> that cliet uses to comunicate
   */
  protected Socket client;

  /**
   * This is the input client-socket stream
   */
  protected DataInputStream in;

  /**
   * This is the output client-socket stream
   */
  protected PrintStream out;

  /**
   * Contains the IP client address
   */
  protected InetAddress clientAddress;

  /**
   * Contais the local client port for connecting to server
   */
  protected int clientPort;

  /**
   * Contains a string name of the client
   */
  protected String clientName;

  /**
   * This contains a line read by the socket
   */
  protected String line;

  /**
   * Contains the available client commands; the hash key is the name of the
   * command, the value is an <code>ClientCommand</code> object
   */
  protected Hashtable clientCommands;

  //copy istance of Server program

  /**
   * Contains the list of agents processes
   * (It's a reference to <code>Server</code>'s)
   */
  protected Hashtable processesList;

  /**
   * Contains the process for serving an ambient
   * (It's a reference to <code>Server</code>'s)
   */
  protected ServerAmbient ServAmb;

  /**
   * This is the array of semaphore that contains the available resources
   * of the index-array type (It's a reference to <code>Server</code>'s)
   */
  protected Semaphore[] availableCells;

  /**
   * This is the array of semaphore that contains the occupied resources
   * of the index-array type (It's a reference to <code>Server</code>'s)
   */
  protected Semaphore[] presentMessage;

  /**
   * A queue of message changed by client agents and the server
   * (It's a reference to <code>Server</code>'s)
   */
  protected Queue[] processesQueue;

  /**
   * Max dimension for the queue
   */
  protected int dimQueue;

  /**
   * This is the <i>abstract class</i> to build your static ambient
   * (It's a reference to <code>Server</code>'s)
   */
  protected Ambient staticAmb;

  /**
   * Contains instance of the actual states of agents
   */
  protected Hashtable state;

  /**
   * This is the thread that updates the simulator state and send the VRML
   * view to the client
   */
  protected SendVRML SV;

  /**
   * Contains the time handler of processes
   */
  protected ProcessesHandler GP;

  /**
   * Contains the list of client users that are connected
   */
  protected Hashtable connectedUsers;

  /**
   * Contains the list of nodes of virtual world
   */
  protected Hashtable nodesList;

  /**
   * Print fail information and exit from server program
   *
   * @param e   the exception that fails server
   * @param msg the message to print in err file
   */
  public static void fail(Exception e, String msg) {
    System.err.println(msg + "\n  [java error code]: " +  e);
    System.exit(1);
  }

  /**
   * initialize the streams and start thread
   */
  public Connection(Socket client_socket, Server server) {
    logActivity("Client connection accepted. Try to build an handler thread...");

    client=client_socket;
    connectedUsers=server.connectedUsers;
    nodesList=server.nodesList;
    processesList=server.processesList;
    ServAmb=server.SA;
    availableCells=server.availableCells;
    presentMessage=server.presentMessage;
    processesQueue=server.processesQueue;
    dimQueue=server.dimQueue;
    staticAmb=server.staticAmb;
    state=server.state;
    SV=server.SV;
    GP=server.GP;

    clientCommands=server.clientCommands;
    // initialize available commands string
    availableCommands="";
    Enumeration el=clientCommands.elements();
    while (el.hasMoreElements()) {
      availableCommands+="\n"+((ClientCommand)el.nextElement()).getText();
    }
    availableCommands+="\n ";
    System.out.println(availableCommands);

    try {
      in = new DataInputStream(client.getInputStream());
      logActivity("Builded an input stream for the new socket    [OK]");
      out = new PrintStream(client.getOutputStream());
      logActivity("Builded an output stream for the new socket   [OK]");

      clientName = in.readLine();    //read the name that client send
    } catch (IOException e) {
        logActivity("[ERROR] don't able to build I/O stream => closing socket ...");
        try {
          client.close();
          logActivity("    Socket CLOSED      [OK]");
        } catch (IOException e2) {}
        this.stop();    //stop thread
        return;
      }

    clientAddress=client.getInetAddress();  //get client IP address
    clientPort=client.getPort();            //get client port

    if ((clientName == null))
      clientName=clientAddress+":"+clientPort;  //name of "IP_Addr:Port" type

    connectedUsers.put(clientAddress +":"+ clientPort, this);  //insert the new users in Hash table
    logActivity("Connection with '" +clientName+"' done");

    // send to the client the server information
    out.println("---------------------------------------------------\n"
                +"Welcome!!  you are connected with "
                + server.about
                +"\n---------------------------------------------------\n");
    out.println("Available command:"+availableCommands);

    // send to all the client that there's a new users
    broadcast("INFO: '"+ clientName + "' is connected.");

    this.setPriority(MAX_PRIORITY);

    //build static part of Virtual Lab
    staticAmb.buildStatic(this);

    this.start();
  }

  /**
   * Gets the names of the connected users
   *
   * @return the string of all the connected users
   */
  public String whoString() {
    String result ="\nCONNECTED USERS:\n";
    Enumeration e =connectedUsers.keys();
    while (e.hasMoreElements())	{
      String key = (String) e.nextElement();
      Connection thisClient = (Connection) connectedUsers.get(key);
      result += thisClient.clientName+" (" + key + ")\n";
    }
    return result;
  }

  /**
   * This is the thread that reads command from socket and runs them
   */
  public void run() {
    boolean esc;
    try {
      for(;;) {
        line= in.readLine();    //read from socket

        logActivity("Receved: " + line); //write to server consol

        esc=false;
        if (line != null) {
          esc=runCommand(line);
        }
        if ((esc) | (line == null)) {
          break;
	}
      }
    } catch (IOException e) {
        logException (e, "[ERROR] don't able to read with the socket of: \n     "
                     + clientName+"("+clientAddress+":"+clientPort+")");
      }
      finally {
        //erase all users nodes from NodesList HashTable
        String object;
        Enumeration e;
        Vector objDel=new Vector();
        Nodes n;

        e=nodesList.keys();
        while (e.hasMoreElements()) {
          object=(String)e.nextElement();
          n=(Nodes)nodesList.get(object);
          if (n.userName.equals(clientName)) {
            nodesList.remove(object);
            objDel.addElement(object);
          }
        }

        //erase all nodes from the other users world
        int i=0;
        String S="";
        for (i=0;i<objDel.size();i++) {
          object=(String)objDel.elementAt(i);
          S+="# DEL "+object+"\n";
        }
        broadcast(S);

        logActivity ("The connection with "
                    + clientAddress + ":" + clientPort
                    + " don't work rigth. It will be closed");
        try {
          client.close();
          logActivity ("[OK] Close the socket with "
                      + clientAddress+":"+clientPort);
        } catch (IOException e2) {
            logException (e2, "[ERROR] closing the socket with "
                         + clientAddress + ":" + clientPort);
          }
          finally {
            client=null;   //erase all reference (=> memory can be free)

            logActivity("The 'Broken socket' is been ERASED");

            connectedUsers.remove(clientAddress+":"+clientPort); //erase from HashTable
            broadcast ("The users '" + clientName + "' ("
                      + clientAddress + ":" + clientPort
                      + ") has close the connection.");
          }
      }
  }

  /**
   * Print exception information
   *
   * @param e   the exception that fails server
   * @param msg the message to print in err file
   */
  public void logException (Exception e, String message) {
    System.err.println(message + ": " +  e);
  }

  /**
   * Print log activities information
   *
   * @param message the message to print
   */
  public void logActivity (String message) {
    System.out.println(message);
  }

  /**
   * Send a string command to all the connected users
   *
   * @param s the string command to send
   */
  public void broadcast (String s) {
    Enumeration e = connectedUsers.keys(); //obtain all hashtable keys
    while (e.hasMoreElements()) {
      String key = (String) e.nextElement();
      Connection thisClient = (Connection) connectedUsers.get(key);
      thisClient.out.println(s);
      thisClient.out.flush();
      //logActivity ("Broadcasting: " + s);
    }
  }

  /**
   * Execute a command that a user client pass to it.
   * The string command are tokenized and passed to the appropriate
   * <code>ClientCommand</code> object that is stored in the
   * <code>clientCommands</code> <i>hashtable</i>.
   *
   * @param comm the string that rappresent the command
   * @return true if we must exit from connection, false otherwise
   *
   * @see ClientCommand
   * @see Server#clientCommands
   */
  private boolean runCommand(String comm) {
    StringTokenizer ST=new StringTokenizer(comm," ",false);
    String header="";
    Object command;
    boolean ret=false;

    if (ST.countTokens()>0) {
     header=ST.nextToken();
     command=clientCommands.get(header);
     if (command!=null) ret=((ClientCommand)command).runCommand(ST, this);
       else broadcast(clientName+"> "+comm);
    }
    return ret;
  }

  /**
   * Read a file and return a string rappresentation of it
   *
   * @param fileName the string name of the file
   * @return the string that contains the files
   */
  public String readFile(String fileName) {
    String s="";
    String str="";
    int i;

    try {
      BufferedReader inputFile=new BufferedReader(new FileReader(fileName));
      s=inputFile.readLine();
      while (s != null) {
        if ((i=s.indexOf("#"))==-1) {
          str+=s+" ";
        } else if (i!=0) str+=s.substring(0,i-1); // delete the comment
        s=inputFile.readLine();
      }
    } catch (FileNotFoundException e) {
        logException(e,"[ERROR] File "+fileName+" not found");
        str="Group {} # null vrml body of object\n";
      }
      catch(IOException IOe) {
        logException(IOe,"[ERROR] don't able to read the file "+fileName);
        str="Group {} # null vrml body of object\n";
      }
    return str;
  }
}


/**
 * This is a <i>class</i> used to store information useful to retrive
 * information about node/user
 */
class Nodes {
  /**
   * Contains a string to use for building a node in memory
   */
  String strNode;

  /**
   * contains the name of the user that hold the node
   */
  String userName;

  /**
   * Initilize object with string
   *
   * @param node the string for the node
   * @param user the string for the user name
   */
  public Nodes(String node, String user) {
    strNode=node;
    userName=user;
  }
}


/**
 * This abstract class execute a command that the client write in the applet.
 * To do this, you must derived it and implements the <code>runCommand</code>
 * method.
 * All the client commands that you build are manage by <code>clientCommands</code>
 * <i>hashtable</i> of <code>Server</code>.
 *
 * @see Server#clientCommands
 */
abstract class ClientCommand {

  /**
   * Contain the command as text help string like "ADD : adding a ..."
   */
  protected String command;

  /**
   * Contain the extended description of the command with the meanings of the
   * parameters like "ADD param1 param2 \n param1=... \n param2=..."
   */
  protected String description;

  /**
   * contain the name of command used as key by <i>hashtable</i>
   */
  protected String name;

  /**
   * Constructor that initialize the <code>ClientCommand</code>:
   *
   * @param comm the text help string
   * @param desc the extended description of command
   */
  public ClientCommand(String comm, String desc) {
    StringTokenizer ST=new StringTokenizer(comm," ",false);
    command=comm;
    description=desc;
    name=ST.nextToken();
  }

  /**
   * Return the name of the command; it must be used for the <i>hashtable</i>
   * hash value.
   *
   * @return the name of command as one token of a string
   */
  public String getName() {
    return name;
  }

  /**
   * Return the text description of the command.
   *
   * @return the text string
   */
  public String getText() {
    return command;
  }

  /**
   * Return an extended description of command
   *
   * @return the description
   */
  public String getDescription() {
    return description;
  }

  /**
   * This is the abstract method to implements that execute your command.
   * It received the <code>StringTokenizer</code> line that the client had
   * insert, so you can extract your parameter.
   *
   * @param ST the line passed by client
   * @param connection a copy of <code>Connection</code> istance to permit
   *        you to use it's variables
   * @return a boolean that indicate if we must exit from <code>RunCommand</code>
   *         of <code>Connection</code>
   */
  public abstract boolean runCommand(StringTokenizer ST, Connection connection);
}


/**
 * Implements the client command that exit from simulator closing the connection
 */
class ExitClientCommand extends ClientCommand {

  /**
   * Initialize object for EXIT command
   */
  public ExitClientCommand() {
    super("EXIT      : close connection",
          "EXIT : close connection");
  }

  /**
   * Comunicate that we want to exit
   *
   * @param ST never used for exit command
   * @param connection a copy of <code>Connection</code> istance to permit
   *        you to use it's variables
   * @return true becouse we exit
   */
  public boolean runCommand(StringTokenizer ST, Connection connection) {
    connection.logActivity ("'EXIT' command ==> CLOSE!");
    return true;
  }
}


/**
 * Implements the client command that shows who are connected now
 */
class WhoClientCommand extends ClientCommand {

  /**
   * Initialize object for WHO command
   */
  public WhoClientCommand() {
    super("WHO       : list the connected users",
          "WHO : list the connected users");
  }

  /**
   * List the connected client
   *
   * @param ST never used for WHO command
   * @param connection a copy of <code>Connection</code> istance to permit
   *        you to use it's variables
   * @return false becouse we must not exit
   */
  public boolean runCommand(StringTokenizer ST, Connection connection) {
    connection.out.println(connection.whoString());
    connection.out.flush();
    return false;
  }
}


/**
 * Implements the client command that rename your actual name
 */
class NameClientCommand extends ClientCommand {

  /**
   * Initialize object for NAME command
   */
  public NameClientCommand() {
    super("NAME      : rename your actual name",
          "NAME new_name \n new_name=[string]=new name to assign to the client");
  }

  /**
   * Change actual client name with new name
   *
   * @param ST first token is used for new name
   * @param connection a copy of <code>Connection</code> istance to permit
   *        you to use it's variables
   * @return false becouse we must not exit
   */
  public boolean runCommand(StringTokenizer ST, Connection connection) {
    String msg;
    if (ST.countTokens()!=1) {
      connection.out.println("Invalid number of parameter");
      connection.out.flush();
      return false;
    }
    String name=ST.nextToken();
    msg="INFO: '"+connection.clientName+"' RENAMED in '"
                     +name+"' ("
                     +connection.clientAddress+":"+
                     +connection.clientPort+")\n";
    // set the new name and send to all
    connection.clientName=name;
    connection.broadcast(msg);
    return false;
  }
}


/**
 * Implements the client modify command that modify a vrml object position
 */
class ModifyClientCommand extends ClientCommand {

  /**
   * Initialize object for MODIFY command
   */
  public ModifyClientCommand() {
    super("MODIFY    : modify an vrml object",
          "MODIFY object_name vrml_field type <..>  : modify an vrml object\n"+
          "object_name=[string]=name of the object who modify some attribute\n"+
          "vrml_field=[string]=the field used by EAI (eg. SFVECT3F, SFROTATION)\n"+
          "type=[string]=the type of operation to execute (eg. set_translation)\n"+
          "...=[...]=varius parameters that are used by the operation" );
  }

  /**
   * modify a position of one object in the simulator
   *
   * @param ST the tokens contain the command parameters
   * @param connection a copy of <code>Connection</code> istance to permit
   *        you to use it's variables
   * @return false becouse we must not exit
   */
  public boolean runCommand(StringTokenizer ST, Connection connection) {
    String objectName="";
    String comm="";
    Nodes n;

    objectName=ST.nextToken();
    n=(Nodes)connection.nodesList.get(objectName);
    if (n != null) {
      if (n.userName.equals(connection.clientName)) {
        while (ST.hasMoreTokens()) {
          comm+=ST.nextToken();
        }
        connection.broadcast(comm);
      } else {
          connection.out.println("Object ("+objectName+") don't be Modified \n becouse it's of "+n.userName);
        }
    } else {
        connection.out.println("Object ("+objectName+") don't exist");
      }
    return false;
  }
}


/**
 * Implements the client add command that add a vrml object in 3D position
 */
class AddClientCommand extends ClientCommand {

  /**
   * Initialize object for ADD command
   */
  public AddClientCommand() {
    super("ADD       : add an vrml object\n",
          "ADD object_name wrl_name pos_x pos_y pos_z : add an vrml object\n"+
          "object_name=[string]=the name we want for the object\n"+
          "wrl_name=[string]=the name of vrml file that describe the object\n"+
          "pos_x=[double]=position x of object\n"+
          "pos_y=[double]=position y of object\n"+
          "pos_z=[double]=position z of object");
  }

  /**
   * Add an object to the simulator
   *
   * @param ST the tokens contain the command parameters
   * @param connection a copy of <code>Connection</code> istance to permit
   *        you to use it's variables
   * @return false becouse we must not exit
   */
  public boolean runCommand(StringTokenizer ST, Connection connection) {
    String objectName="";
    String objectType="";
    String vrmlString="";
    Nodes n;
    float[] position=new float[3];

    objectName=ST.nextToken();
    if (((Nodes)connection.nodesList.get(objectName)) == null) {
      objectType=ST.nextToken();
      int i=0;
      for (i=0;i<3;i++) {
        position[i]=(Float.valueOf(ST.nextToken())).floatValue();
      }
      vrmlString="# NEW "+objectName+" "+connection.readFile(objectType+".wrl");

      n=new Nodes(vrmlString,connection.clientName);
      connection.nodesList.put(objectName,n);
      connection.broadcast(vrmlString);
      connection.broadcast("# MODIFY "+objectName+" SFVEC3F set_translation "+position[0]+" "+position[1]+" "+position[2]);
    } else {
        connection.out.println("Object "+objectName+" already exist.\n Change name!!!");
      }
    return false;
  }
}


/**
 * Implements the client command that delete one object
 */
class DelClientCommand extends ClientCommand {

  /**
   * Initilaize object for DEL command
   */
  public DelClientCommand() {
    super("DEL       : del an object from simulator",
          "DEL object_name : del an object from simulator\n"+
          "object_name=[string]=the name of object to delete");
  }

  /**
   * Delete an object from simulator
   *
   * @param ST the next token is the name of object to delete
   * @param connection a copy of <code>Connection</code> istance to permit
   *        you to use it's variables
   * @return false becouse we must not exit
   */
  public boolean runCommand(StringTokenizer ST, Connection connection) {
    String objectName="";
    Nodes n;

    objectName=ST.nextToken();
    n=(Nodes)connection.nodesList.get(objectName);
    if (n != null) {
      if (n.userName.equals(connection.clientName)) {
        connection.nodesList.remove(objectName);
        connection.broadcast("# DEL "+objectName);
      } else {
          connection.out.println("Object ("+objectName+") can't be erased\nbecouse it's of "+n.userName);
        }
    } else {
        connection.out.println("Object ("+objectName+") don't exist");
      }
    return false;
  }
}


/**
 * Implements the client command  that help for a list of commands
 */
class HelpClientCommand extends ClientCommand {

  /**
   * Initilaize object for HELP command
   */
  public HelpClientCommand() {
    super("HELP      : show list of command or description of a command",
          "HELP command : show list of command or description of a command\n"+
          "command=[string]=show a description of the command if it exists, else the list of command");
  }

  /**
   * Show a help line only for the user that require it
   *
   * @param ST a token who get help
   * @param connection a copy of <code>Connection</code> istance to permit
   *        you to use it's variables
   * @return false becouse we must not exit
   */
  public boolean runCommand(StringTokenizer ST, Connection connection) {
    String comm;
    ClientCommand elem;

    if (ST.countTokens()>0) comm=ST.nextToken();
      else comm="";
    Enumeration el=connection.clientCommands.elements();

    while (el.hasMoreElements()) {
      if (comm.equals((elem=(ClientCommand)el.nextElement()).getName())) {
        connection.out.println(elem.getDescription()+"\n ");
        return false;
      }
    }
    connection.out.println(connection.availableCommands);
    return false;
  }
}


/**
 * This is an empty <i>class</i> used only to documentate how a agent may
 * work with the other <i>classes</i>.
 * You must insert this code in your <i>class</i>:
 * <ul>
 * <li> An constructor that initialize your agent variables (initial position,
 *      final goal...) and receive the <code>processesQueue</code> tile of
 *      <code>Server</code> (that is the <code>agentQueue</code> of
 *      <code>ServerAmbient/GenericServerAmbient</code>) that is
 *      necessary for a correct use of comunication (so an identifier
 *      progressive number and the name of robot must be present to use the
 *      tile correctly).
 *      This constructor may finish with this instruction:
 *   <ul>
 *   <li>   <code>setPriority(MAX_PRIORITY)</code>
 *   <li>   <code>start()</code>
 *   </ul>
 *      that set the correct priority and start <i>thread</i> body.
 * <li> a <code>run</code> method that execute a loop until the goal was
 *      achieved when the agents read the information he need from
 *      <code>ServerAmbient</code> using the <code>processesQueue</code> tile
 *      and set the new information about itself calculated with your behavior
 *      based strategy.
 * </ul>
 * @see ServerAmbient
 * @see Server#processesQueue
 */
abstract class Agent extends Thread {
}


/**
 * This is an <i>abstract class</i> that performe the basic function of a
 * client applet.
 * You must derive this class and implement your personal function.
 * It's important know that your applet and the vrml view must be in the same
 * page to work correct, so the html page will be like this:
 * <p><blockquote><pre>
 *
 *  &lt;HTML>
 *  &lt;HEAD>
 *  &lt;TITLE>Client Applet</TITLE>
 *  &lt;/HEAD>
 *
 *  &lt;BODY>
 *  &lt;EMBED SRC="World.wrl" WIDTH=400 HEIGHT=300 NAME="wrl">
 *  &lt;APPLET code="MyClient.class" width=511 height=377 MAYSCRIPT>
 *  &lt;PARAM name=port value=6472>
 *  &lt;PARAM name=host value=localhost>
 *  &lt;PARAM name=showRoundTrip value=false>
 *  &lt;/APPLET>
 *  &lt;/BODY>
 * </pre></blockquote>
 * <p>
 * The <i>World.wrl</i> vrml file will be like this:
 * <p><blockquote><pre>
 *  #VRML V2.0 utf8
 *
 *  DEF World Group {
 *    children [
 *      Viewpoint {
 *        position 10 30 -10
 *        orientation 1 0 0 -1.5707
 *        description "Up View"
 *      },
 *      Viewpoint {
 *        position 10,5,10
 *        description "Front View"
 *      },
 *      DEF Room Group {
 *        children [
 *          Transform {
 *            translation 10,0.005,-10
 *            children [
 *              DEF Floor Shape {
 *                geometry Box {
 *                  size 20,.01,20
 *                }
 *              }
 *            ]
 *          }
 *        ]
 *      }
 *    ]
 *  }
 * </pre></blockquote>
 * <p>
 * In this examples was create a world containing a room and a shape, and
 * two viepoints. You can create all you need using the vrml syntax, but the
 * <code>DEF world Group {</code> must be present for the correct use of the
 * program.
 */
abstract class Client extends Applet {

  /**
   * This contains an text that shows who has create the client.
   * It must be initialize by you in you <code>init</code> method.
   */
  public static String about;

  public static final int DEFAULT_PORT=6472;
  public static final String DEFAULT_HOST="localhost";  //localhost identify 127.0.0.0
  public static final boolean DEFAULT_SHOWROUNDTRIP=false;

  /**
   * Max number of char that the <code>TextArea</come> may contain.
   * After this value the old will be cut from the head
   */
  public static final int MAX_CHARS=10000;

  /**
   * Create a new random number generator. This may used by the instances of
   * the applet to create a unique identifier.
   */
  public static Random idGen=new Random();

  /**
   * Contains the instance of <code>Listener</code> <i>class</i>
   */
  protected Listener listener=null;

  /**
   * Contains the instance of  <code>Typist</code> <i>class</i>
   */
  protected Typist typist=null;

  /**
   * Contains the port number used by the socket
   */
  protected int port;

  /**
   * This is the string that contains the host information
   */
  protected String host;

  protected int maxChars;

  /**
   * Contains the <i>socket</i> to comunicate with the server
   */
  protected Socket s=null;

  /**
   * Contains the information like 'login@machine'
   */
  protected String remote_name;

  /**
   * The identifier ID of current instance
   */
  protected int istance_ID=idGen.nextInt();

  /**
   * Indicate if a RTT will be visible
   */
  protected boolean showRoundTripInitial;

  // --- graphical controls ---------------

  /**
   * This is the control used to insert information to send to the server.
   * It must be present in your applet graphical interface for a correct
   * use of this applet.
   */
  TextField winLocal;

  /**
   * This <code>Label</code> is used with the <code>TextField</code> and it
   * used only by <code>graphicAppletInit</code> method.
   */
  Label typePrompt;

  /**
   * This is the area used to write the information that the server send us.
   * It must be present in your applet graphical interface for a correct
   * use of this applet.
   */
  TextArea winAll;

  /**
   * This is the control to choose if you want to show the RTT.
   * If you don't want this feature you can not use this control, but you
   * must initialize the <code>showRoundTripInitial</code> correctly.
   */
  Checkbox showRoundTripCheck;


  // --- VRML variables ---------------

  /**
   * Contains the instance to the <code>Browser</code> <i>class</i>.
   * This <i>class</i> is used to comunicate between your applet and an
   * VRML browser only if these are in the same HTML page.
   */
  Browser browser;

  /**
   * Contains a list of vrml nodes add to the VRML browser
   */
  Hashtable nodesList;

  /**
   * Contains the tokens available for SFVec3F VRML field
   */
  protected Hashtable tableSFVec3f;

  /**
   * Contains the tokens available for SFColor VRML field
   */
  protected Hashtable tableSFColor;

  /**
   * Contains the tokens available for SFRotation VRML field
   */
  protected Hashtable tableSFRotation;

  /**
   * Contains the tokens available for SFFloat VRML field
   */
  protected Hashtable tableSFFloat;

  /**
   * Contains the tokens available for SFBool VRML field
   */
  protected Hashtable tableSFBool;

  /**
   * Contains the tokens available for SFVec2f VRML field
   */
  protected Hashtable tableSFVec2f;

  /**
   * Contains the tokens available for SFNode VRML field
   */
  protected Hashtable tableSFNode;

  /**
   * Contains the tokens available for MFNode VRML field
   */
  protected Hashtable tableMFNode;

  /**
   * Contains the tokens available for SFTime VRML field
   */
  protected Hashtable tableSFTime;

  /**
   * Contains the tokens available for SFInt32 VRML field
   */
  protected Hashtable tableSFInt32;

  /**
   * Specify the type of parameter that the applet want
   */
  public String[][] getParameterInfo() {
    String[][] info = {
      // name, type, description
      {"host", "string", "server host name"},
      {"port", "integer", "port used for the connection"},
      {"showRoundTrip", "boolean", "initial value for RTT checkbox"},
      {"maxChars", "integer", "max number of chars before cutting"}
    };
    return info;
  }

  /**
   * return the information about the autor of the client
   *
   * @return a text helped string
   */
  public String getAppletInfo() {
    return about;
  }

  /**
   * Initialize the applet.
   * Your <i>class</i> must extend this method to initialize the undefined
   * variables, after you can call this with <i>super</i>.
   */
  public void init() {
    System.err.println("Initializing applet ...  [OK]");
    System.err.println(about);


    // try to read parameters from applet
    try	{
      port=Integer.parseInt(getParameter("port"));
    } catch (NumberFormatException e) {
        port=DEFAULT_PORT;
      }

    host=getParameter("host");
    if (host==null) {
      host=DEFAULT_HOST;
    }

    showRoundTripInitial=Boolean.valueOf(getParameter("showRoundTrip")).booleanValue();

    try	{
      maxChars=Integer.parseInt(getParameter("maxChars"));
    } catch (NumberFormatException e) {
        maxChars=MAX_CHARS;
      }

    System.out.println("host = "+host
                       + "\nport = "+port
                       + "\nshowRoundTrip = "+showRoundTripInitial
                       + "\nmaxChars = "+maxChars );
    buildFieldTable();
  }

  /**
   * destroy()
   */
  public void destroy() {
    System.err.println("running destroy().    [OK]");
  }

  /**
   * finalize()
   */
  public void finalize() throws Throwable {
    System.err.println("running finalize().   [OK]");
  }

  /**
   * Initilize graphic object in the applet.
   * You can overwrite by extension this method, but you must init the
   * <code>TextField</code> and <code>TextArea</code>, and optionally the
   * <code>CheckBox</code> for RTT.
   */
  public void graphAppletInit() {
    setLayout(null);
    resize(511,377);

    //writing box
    winLocal=new TextField(44);
    winLocal.setFont(new Font("Courier",Font.PLAIN,12));
    add(winLocal);
    winLocal.reshape(126,315,371,30);

    //label for writing box
    typePrompt=new Label("Send:", Label.RIGHT);
    typePrompt.setFont(new Font("Helvetica",Font.PLAIN,12));
    add(typePrompt);
    typePrompt.reshape(7,330,112,15);

    //message window
    winAll=new TextArea(19,60);
    winAll.setFont(new Font("Courier",Font.PLAIN,12));
    add(winAll);
    winAll.reshape(7,4,497,308);
    winAll.setBackground(Color.white);

    //CheckBox for RTT
    showRoundTripCheck=new Checkbox("Show RTT");
    add(showRoundTripCheck);
    showRoundTripCheck.reshape(126,353,154,22);
    showRoundTripCheck.setState(showRoundTripInitial);

    winAll.setEditable(false);
    winAll.appendText(about+"\n");

  }

  /**
   * this method activate a connection with the server, then initilize
   * the graph of the applet and last activate the </>threads</i>
   * <code>Listener</code> and </code>Typist</code>
   *
   * @see Listener
   * @see Typist
   */
  public void start(){
    System.err.println("Start() executing... ");

    // Activate the socket connection and send info to server

    try	{
      s=new Socket(host, port);     //activate the socket
      // obtain user property: a security exeption can occurs becouse this is an applet
      String local_user="?";
      try {
	    local_user=System.getProperty("user.name");
      } catch (SecurityException e1) {local_user="?"+idGen.nextInt();}

      DataOutputStream os=new DataOutputStream(s.getOutputStream());

      //send local user name to the server
      os.writeBytes(local_user);
      os.writeByte('\n');
      os.flush();

      remote_name=s.getInetAddress()+":"+s.getPort();

      System.out.println("Connecting to the server '"+ remote_name+"' is done\n");
    } catch (IOException e) {
        System.err.println("Can't activate the socket connection with "+remote_name);
        this.cleanUp();
	return;
      }

    graphAppletInit();

    //Initialize Browser VRML2.0 corrent instance
 /**   JSObject win=JSObject.getWindow(this);
    JSObject doc=(JSObject) win.getMember("document");
    JSObject embeds=(JSObject) doc.getMember("embeds");
    browser=(Browser) embeds.getSlot(0);
*/ browser=Browser.getBrowser(this);
    nodesList=new Hashtable();

    //activacting threads
    listener=new Listener(s, winAll, this);
    typist=new Typist(s, winLocal, this);
  }

  /**
   * This is called by the applet when it stops
   */
  public void stop() {
    System.err.println("I'm in stop() ...  [OK]");
    cleanUp();
    super.stop();
  }

  /**
   * Clean all the active processes before exiting
   */
  public void cleanUp() {
    System.err.println("Executing cleanUp() ...");

    // Save the value of Checkbox state if another call to the applet
    if (showRoundTripCheck != null) {
      showRoundTripInitial=showRoundTripCheck.getState();
    }

    // stop threads, remove windows and delete socket
    if (typist != null && typist.running) {
      typist.stop();
    }
    typist=null;
    System.err.println("Typist thread --> STOPPED & ERASED");

    if (listener != null && listener.running) {
      System.err.println("Closing the Listener...");
      listener.stop();
    }
    listener=null;
    System.err.println("Listener Thread --> STOPPED & ERASED");

    removeAll();  //erase all the instance of graphics

    try {
      if (s != null) {
        s.close();
        System.err.println("socket closed.  [OK]");
      }
    } catch (IOException e) {
        System.err.println("[ERROR] Can't close the socket: " + e);
      }

    s=null;
    remote_name="";
    repaint();
  }

  /**
   * If RETURN key was pressed, the contents of <code>TextArea</code> is send
   * to the server. This type of solution is <b>deprecated</b> but it is not
   * converted to actual solution becouse when the </>Swing</i> become real in
   * SUN JDK 1.2, some features of <i>Client</i> class probably will to be adapted
   * to the new filosofy again.
   *
   * @param e the event that occurs
   * @param key the key pressed
   */
  public boolean keyUp(Event e, int key) {
    int flags=e.modifiers;   //verify if there's key as CTRL,ALT ...
    if ((e.id==Event.KEY_RELEASE) && (e.key=='\n')) {
      if (typist != null) {
        typist.sendLine();
      }
      return true;    // I handled it myself
    } else {
        return false; // I want someone else to handle it
      }
  }


  /**
   * Draw a graphics message about server connection not available
   *
   * @param g the window of applet where draw in graphics
   */
  public void paint(Graphics g) {
    if (typist==null && listener==null && s==null) {
      g.setFont(new Font("Helvetica",Font.BOLD,18));
      g.drawString("Connection with server not available. Threads stopped.", 20, 180);
    }
  }

  /**
   * Insert a text in the applet text window and resize it if buffer length
   * is over <code>maxChars</code> chars.
   *
   * @param t the string to insert in window
   */
  public void insertText(String t) {
    TextArea w=winAll;
    String text=w.getText();
    while (text.length()+t.length()>maxChars) {
      int upTo=text.indexOf("\n");
      w.replaceText("", 0, upTo+1);
      text=w.getText();
    }
    w.appendText(t);
  }

  /**
   * Builds the tables used to find rapidally if a field is of VRML type
   */
  protected void buildFieldTable() {

    // creates elements for SFVec3f fields of VRML
    tableSFVec3f=new Hashtable();

    tableSFVec3f.put("set_attenuation", new Integer(1));
    tableSFVec3f.put("set_axisOfRotation", new Integer(2));
    tableSFVec3f.put("set_center", new Integer(3));
    tableSFVec3f.put("set_direction", new Integer(4));
    tableSFVec3f.put("set_location", new Integer(5));
    tableSFVec3f.put("set_offset", new Integer(6));
    tableSFVec3f.put("set_scale", new Integer(7));
    tableSFVec3f.put("set_size", new Integer(8));
    tableSFVec3f.put("set_translation", new Integer(9));

    tableSFVec3f.put("attenuation", new Integer(101));
    tableSFVec3f.put("axisOfRotation", new Integer(102));
    tableSFVec3f.put("center", new Integer(103));
    tableSFVec3f.put("direction", new Integer(104));
    tableSFVec3f.put("location", new Integer(105));
    tableSFVec3f.put("offset", new Integer(106));
    tableSFVec3f.put("scale", new Integer(107));
    tableSFVec3f.put("size", new Integer(108));
    tableSFVec3f.put("translation", new Integer(109));

    // creates elements for SFRotation fields of VRML
    tableSFRotation=new Hashtable();

    tableSFRotation.put("set_offset", new Integer(1));
    tableSFRotation.put("set_orientation", new Integer(2));
    tableSFRotation.put("set_rotation", new Integer(3));
    tableSFRotation.put("set_scaleOrientation", new Integer(4));

    tableSFRotation.put("offset", new Integer(101));
    tableSFRotation.put("orientation", new Integer(102));
    tableSFRotation.put("rotation", new Integer(103));
    tableSFRotation.put("scaleOrientation", new Integer(104));

    // creates elements for SFColor fields of VRML
    tableSFColor=new Hashtable();

    tableSFColor.put("set_color", new Integer(1));
    tableSFColor.put("set_diffuseColor", new Integer(2));
    tableSFColor.put("set_emissiveColor", new Integer(3));
    tableSFColor.put("set_specularColor", new Integer(4));

    tableSFColor.put("color", new Integer(101));
    tableSFColor.put("diffuseColor", new Integer(102));
    tableSFColor.put("emissiveColor", new Integer(103));
    tableSFColor.put("specularColor", new Integer(104));

    // creates elements for SFFloat fields of VRML
    tableSFFloat=new Hashtable();

    tableSFFloat.put("set_ambientIntensity", new Integer(1));
    tableSFFloat.put("set_beamWidth", new Integer(2));
    tableSFFloat.put("set_cutOffAngle", new Integer(3));
    tableSFFloat.put("set_diskAngle", new Integer(4));
    tableSFFloat.put("set_fieldOfView", new Integer(5));
    tableSFFloat.put("set_fraction", new Integer(6));
    tableSFFloat.put("set_intensity", new Integer(7));
    tableSFFloat.put("set_maxAngle", new Integer(8));
    tableSFFloat.put("set_maxBack", new Integer(9));
    tableSFFloat.put("set_maxFront", new Integer(10));
    tableSFFloat.put("set_minAngle", new Integer(11));
    tableSFFloat.put("set_minBack", new Integer(12));
    tableSFFloat.put("set_minFront", new Integer(13));
    tableSFFloat.put("set_pitch", new Integer(14));
    tableSFFloat.put("set_priority", new Integer(15));
    tableSFFloat.put("set_radius", new Integer(16));
    tableSFFloat.put("set_rotation", new Integer(17));
    tableSFFloat.put("set_shininess", new Integer(18));
    tableSFFloat.put("set_speed", new Integer(19));
    tableSFFloat.put("set_trasparency", new Integer(20));

    tableSFFloat.put("ambientIntensity", new Integer(101));
    tableSFFloat.put("beamWidth", new Integer(102));
    tableSFFloat.put("cutOffAngle", new Integer(103));
    tableSFFloat.put("diskAngle", new Integer(104));
    tableSFFloat.put("fieldOfView", new Integer(105));
    tableSFFloat.put("intensity", new Integer(107));
    tableSFFloat.put("maxAngle", new Integer(108));
    tableSFFloat.put("maxBack", new Integer(109));
    tableSFFloat.put("maxFront", new Integer(110));
    tableSFFloat.put("minAngle", new Integer(111));
    tableSFFloat.put("minBack", new Integer(112));
    tableSFFloat.put("minFront", new Integer(113));
    tableSFFloat.put("pitch", new Integer(114));
    tableSFFloat.put("priority", new Integer(115));
    tableSFFloat.put("radius", new Integer(116));
    tableSFFloat.put("rotation", new Integer(117));
    tableSFFloat.put("shininess", new Integer(118));
    tableSFFloat.put("speed", new Integer(119));
    tableSFFloat.put("trasparency", new Integer(120));

    // creates elements for SFBool fields of VRML
    tableSFBool=new Hashtable();

    tableSFBool.put("set_autoOffset", new Integer(1));
    tableSFBool.put("set_bind", new Integer(2));
    tableSFBool.put("set_collide", new Integer(3));
    tableSFBool.put("set_enable", new Integer(4));
    tableSFBool.put("set_loop", new Integer(5));
    tableSFBool.put("set_jump", new Integer(6));
    tableSFBool.put("set_on", new Integer(7));

    tableSFBool.put("autoOffset", new Integer(101));
    tableSFBool.put("collide", new Integer(103));
    tableSFBool.put("enable", new Integer(104));
    tableSFBool.put("loop", new Integer(105));
    tableSFBool.put("jump", new Integer(106));
    tableSFBool.put("on", new Integer(107));

    // creates elements for SFVec2f fields of VRML
    tableSFVec2f=new Hashtable();

    tableSFVec2f.put("set_crossSection", new Integer(1));
    tableSFVec2f.put("set_maxPosition", new Integer(2));
    tableSFVec2f.put("set_minPosition", new Integer(3));
    tableSFVec2f.put("set_scale", new Integer(4));
    tableSFVec2f.put("set_translation", new Integer(5));

    tableSFVec2f.put("maxPosition", new Integer(102));
    tableSFVec2f.put("minPosition", new Integer(103));
    tableSFVec2f.put("scale", new Integer(104));
    tableSFVec2f.put("translation", new Integer(105));

    // creates elements for SFTime fields of VRML
    tableSFTime=new Hashtable();

    tableSFTime.put("set_cycleInterval", new Integer(1));
    tableSFTime.put("set_startTime", new Integer(2));
    tableSFTime.put("set_stopTime", new Integer(3));

    tableSFTime.put("cycleInterval", new Integer(101));
    tableSFTime.put("startTime", new Integer(102));
    tableSFTime.put("stopTime", new Integer(103));

    // creates elements for SFInt32 fields of VRML
    tableSFInt32=new Hashtable();

    tableSFInt32.put("set_whichChoice", new Integer(1));

    tableSFInt32.put("whichChoice", new Integer(101));

    // creates elements for SFNode fields of VRML
    tableSFNode=new Hashtable();

    tableSFNode.put("set_appearance", new Integer(1));
    tableSFNode.put("set_color", new Integer(2));
    tableSFNode.put("set_coord", new Integer(3));
    tableSFNode.put("set_fontStyle", new Integer(4));
    tableSFNode.put("set_geometry", new Integer(5));
    tableSFNode.put("set_material", new Integer(6));
    tableSFNode.put("set_normal", new Integer(7));
    tableSFNode.put("set_source", new Integer(8));
    tableSFNode.put("set_texCoord", new Integer(9));
    tableSFNode.put("set_texture", new Integer(10));
    tableSFNode.put("set_textureTransform", new Integer(11));

    tableSFNode.put("appearance", new Integer(101));
    tableSFNode.put("color", new Integer(102));
    tableSFNode.put("coord", new Integer(103));
    tableSFNode.put("fontStyle", new Integer(104));
    tableSFNode.put("geometry", new Integer(105));
    tableSFNode.put("material", new Integer(106));
    tableSFNode.put("normal", new Integer(107));
    tableSFNode.put("source", new Integer(108));
    tableSFNode.put("texCoord", new Integer(109));
    tableSFNode.put("texture", new Integer(110));
    tableSFNode.put("textureTransform", new Integer(111));

    // creates elements for MFNode fields of VRML
    tableMFNode=new Hashtable();

    tableMFNode=new Hashtable();

    tableMFNode.put("set_addChildren", new Integer(1));
    tableMFNode.put("set_children", new Integer(2));
    tableMFNode.put("set_choice", new Integer(3));
    tableMFNode.put("set_level", new Integer(4));
    tableMFNode.put("set_removeChildren", new Integer(5));

    tableMFNode.put("children", new Integer(102));
    tableMFNode.put("choice", new Integer(103));
    tableMFNode.put("level", new Integer(104));

  }

  /**
   * Parse a string command send by server for executing some commands to the
   * virtual world and to the relative structures.
   * In this implementation is not possible to a programmer to extends the
   * classes of command to performes without modify this method (or
   * overwriting all the method), becouse we thinks that when all VRML2.0
   * features are coded in this method, no further adjustement seems to be
   * neccessary.
   * If next version of VRML use other useful commands or you need another
   * features not implemented, you must modify this method.
   * (however actualy not all VRML2.0 features are coded, becouse they are
   * not used until now). <p>
   * Available commands are:
   * <ul>
   * <li> # NEW       obj_name vrml_define_object_string
   * <li> # DEL       obj_name
   * <li> # MODIFY    obj_name SFBOOL      type  boolean
   * <li> # MODIFY    obj_name SFTIME      type  time
   * <lI> # MODIFY    obj_name SFINT32     type  int
   * <li> # MODIFY    obj_name SFFLOAT     type  float
   * <li> # MODIFY    obj_name SFVEC2F     type  num_1 num_2
   * <li> # MODIFY    obj_name SFVEC3F     type  num_x num_y num_z
   * <li> # MODIFY    obj_name SFCOLOR     type  red   green blue
   * <li> # MODIFY    obj_name SFROTATION  type  num_x num_y num_z num_a
   * <li> # ADDSFNODE obj_name parent_name type  vrml_string
   * <li> # ADDMFNODE obj_name parent_name type  vrml_string
   * <li> # ERASE     obj_name
   * <li> % ...      a text string that must not be printed in the window
   * <li> all remaining is a text string to print
   * The NEW creates object using "World" DEF tag and is used for creating
   * the first structures of one object. Infact the MODIFY command modifies
   * some attribute of Vrml object only if they are the external tags.
   * So you must create the object using the ADDSFNODE or ADDMFNODE if you
   * need to change the neested caracteristics.
   * Use ERASE to delete nodes creates with ADDxxNODE and DEL for NEW
   * </ul>
   *
   * @param s the text string with command
   */
  public void runCommand(String s) {
    CodeTable vrmlTab=new CodeTable();

    StringTokenizer ST=new StringTokenizer(s," [],",false);
    int code;

    String vrmlString="";
    String header;
    Node world;
    Node node;
    Node[] vecNode;
    float vec2[]=new float[2];   // used for SFVec2f
    float vec3[]=new float[3];   // used for SFVec3f, SFColor ...
    float vec4[]=new float[4];   // used for SFDirection

    String nodeName="";   // name of node to create
    String parentNode=""; // name of parent node
    String nodeType="";   // type of node to create
    String fieldName="";  // name of operation to the field (like set_center)
    String fieldType="";  // type of VRML field (like SFColor)

    System.out.println("I'm executing: "+s);
    if (ST.countTokens()>0)
      header=ST.nextToken();
    else
      header="";

    if (header.equals("%")) {  // this is a non printable string useful for debug
      return;
    }

    if (!header.equals("#")) { // this is a printable string
      insertText(s+"\n");
      return;
    }

    header=ST.nextToken();

    // Identify the messagge that was arrived
    if ((header.equals("NEW")) && (ST.countTokens()>0)) {
      nodeName=ST.nextToken();
      vrmlString=s.substring(6+nodeName.length());
      vecNode=browser.createVrmlFromString(vrmlString);
      nodesList.put(nodeName,vecNode[0]);

      try {
        world=browser.getNode("World");
      } catch (InvalidNodeException e) { return; }
      try {
        ((EventInMFNode)world.getEventIn("addChildren")).setValue(vecNode);
      } catch (InvalidEventInException e) {}
    }

    if ((header.equals("DEL")) && (ST.countTokens()>0)) {
      nodeName=ST.nextToken();
      node=(Node) nodesList.get(nodeName);
      nodesList.remove(nodeName);
      try {
        world=browser.getNode("World");
      } catch (InvalidNodeException e) { return; }
      vecNode=new Node[1];
      vecNode[0]=node;
      try {
        ((EventInMFNode)world.getEventIn("removeChildren")).setValue(vecNode);
      } catch (InvalidEventInException e) {}
    }

    if ((header.equals("ADDSFNODE")) && (ST.countTokens()>3)) {
      nodeName=ST.nextToken();        // name of node to add
      parentNode=ST.nextToken();      // name of node where add
      nodeType=ST.nextToken();        // type of SFNode to add
      world=(Node) nodesList.get(parentNode);
      vrmlString=s.substring(14+nodeName.length()+parentNode.length()+nodeType.length());
      vecNode=browser.createVrmlFromString(vrmlString);

      if ((Integer)tableSFNode.get(nodeType)==null)
        return;                        // there's error

      try {
        ((EventInSFNode)world.getEventIn(nodeType)).setValue(vecNode[0]);
      } catch (InvalidEventInException e) {}
      nodesList.put(nodeName,vecNode[0]);
    }

    if ((header.equals("ADDMFNODE")) && (ST.countTokens()>3)) {
      nodeName=ST.nextToken();        // name of node to add
      parentNode=ST.nextToken();      // name of node where add
      nodeType=ST.nextToken();        // type of MFNode to add
      world=(Node) nodesList.get(parentNode);
      vrmlString=s.substring(14+nodeName.length()+parentNode.length()+nodeType.length());
      vecNode=browser.createVrmlFromString(vrmlString);

      if ((Integer)tableMFNode.get(nodeType)==null)
        return;                        // there's error

      try {
        ((EventInMFNode)world.getEventIn(nodeType)).setValue(vecNode);
      } catch (InvalidEventInException e) {}
      nodesList.put(nodeName,vecNode[0]);
    }

    if ((header.equals("ERASE")) && (ST.countTokens()>0)) {
      nodeName=ST.nextToken();        // name of node to delete from list
      nodesList.remove(nodeName);
    }

    if ((header.equals("MODIFY")) && (ST.countTokens()>3)) {
      nodeName=ST.nextToken();
      node=(Node) nodesList.get(nodeName);

      fieldType=ST.nextToken();
      fieldName=ST.nextToken();
      code=vrmlTab.Code(fieldType);
      switch (code) {
        case FieldTypes.SFVEC3F:
          if (ST.countTokens()<3) break;      // there's error
          vec3[0]=(new Float(ST.nextToken())).floatValue();
          vec3[1]=(new Float(ST.nextToken())).floatValue();
          vec3[2]=(new Float(ST.nextToken())).floatValue();

          if ((Integer)tableSFVec3f.get(fieldName)==null)
             break;                        // there's error

          //set the parameter in the node
          try {
            ((EventInSFVec3f)node.getEventIn(fieldName)).setValue(vec3);
          } catch (InvalidEventInException e) {}

          //Insert in the hashtable the node with the new parameters
          nodesList.put(nodeName,node);
          break;

        case FieldTypes.SFROTATION:
          if (ST.countTokens()<4) break;      // there's error
          vec4[0]=(new Float(ST.nextToken())).floatValue();
          vec4[1]=(new Float(ST.nextToken())).floatValue();
          vec4[2]=(new Float(ST.nextToken())).floatValue();
          vec4[3]=(new Float(ST.nextToken())).floatValue();

          if ((Integer)tableSFRotation.get(fieldName)==null)
             break;                        // there's error

          //set the paramenter in the node
          try {
            ((EventInSFRotation)node.getEventIn(fieldName)).setValue(vec4);
          } catch (InvalidEventInException e) {}

          //Insert in the hashtable the node with the new parameters
          nodesList.put(nodeName,node);
          break;

        case FieldTypes.SFCOLOR:
          if (ST.countTokens()<3) break;      // there's error
          vec3[0]=(new Float(ST.nextToken())).floatValue();
          vec3[1]=(new Float(ST.nextToken())).floatValue();
          vec3[2]=(new Float(ST.nextToken())).floatValue();

          if ((Integer)tableSFColor.get(fieldName)==null)
             break;                        // there's error

          //set the parameter in the node
          try {
            ((EventInSFColor)node.getEventIn(fieldName)).setValue(vec3);
          } catch (InvalidEventInException e) {}

          //Insert in the hashtable the node with the new parameters
          nodesList.put(nodeName,node);
          break;

        case FieldTypes.SFVEC2F:
          if (ST.countTokens()<2) break;      // there's error
          vec2[0]=(new Float(ST.nextToken())).floatValue();
          vec2[1]=(new Float(ST.nextToken())).floatValue();

          if ((Integer)tableSFVec2f.get(fieldName)==null)
             break;                        // there's error

          //set the parameter in the node
          try {
            ((EventInSFVec2f)node.getEventIn(fieldName)).setValue(vec2);
          } catch (InvalidEventInException e) {}

          //Insert in the hashtable the node with the new parameters
          nodesList.put(nodeName,node);
          break;

        case FieldTypes.SFINT32:
          if (ST.countTokens()<1) break;     // there's error
          int i=(new Integer(ST.nextToken())).intValue();

          if ((Integer)tableSFInt32.get(fieldName)==null)
             break;                        // there's error

          //set the parameter in the node
          try {
            ((EventInSFInt32)node.getEventIn(fieldName)).setValue(i);
          } catch (InvalidEventInException e) {}

          //Insert in the hashtable the node with the new parameters
          nodesList.put(nodeName,node);
          break;

        case FieldTypes.SFFLOAT:
          if (ST.countTokens()<1) break;     // there's error
          float f=(new Float(ST.nextToken())).floatValue();

          if ((Integer)tableSFFloat.get(fieldName)==null)
             break;                        // there's error

          //set the parameter in the node
          try {
            ((EventInSFFloat)node.getEventIn(fieldName)).setValue(f);
          } catch (InvalidEventInException e) {}

          //Insert in the hashtable the node with the new parameters
          nodesList.put(nodeName,node);
          break;

        case FieldTypes.SFBOOL:
          if (ST.countTokens()<1) break;     // there's error
          boolean b=(new Boolean(ST.nextToken())).booleanValue();

          if ((Integer)tableSFBool.get(fieldName)==null)
             break;                        // there's error

          //set the parameter in the node
          try {
            ((EventInSFBool)node.getEventIn(fieldName)).setValue(b);
          } catch (InvalidEventInException e) {}

          //Insert in the hashtable the node with the new parameters
          nodesList.put(nodeName,node);
          break;
      }
    }
  }
}



/**
 * A <i>thread-class</i> that handle the text typist in an
 * <code>TextField</code> and send message to the server
 */
class Typist extends Thread {

  /**
   * This is the output stream where we write on
   */
  protected PrintStream sout;

  /**
   * Contain the line that the client users write
   */
  protected TextField window;

  /**
   * This flag indicates if typist is running
   */
  protected boolean running=false;

  /**
   * Contains the instance to the applet
   */
  protected Client applet;

  /**
   * This is a line identifier used by RTT
   */
  protected long lineId=0;

  /**
   * Initilaize object: create new streams and start thread
   *
   * @param s the socket to comunicate
   * @param window a <code>TextFiled</code> where we insert line
   * @param applet an applet of the <code>Client<code> type
   */
   public Typist(Socket s, TextField window, Client applet) {
     System.err.println("I try to initialize the typist...     [OK]");
     this.window=window;
     this.applet=applet;
     try {
       sout=new PrintStream(s.getOutputStream());
       System.err.println("Typist thread had open the output stream throw the server.   [OK]");
     } catch (IOException e) {
         System.err.println("[ERROR] Typist thread can't open the output stream throw the server");
         running=false;
         applet.cleanUp();
         return;
       }
     this.start();
  }

  /**
   * This only set a variable to comunicate that it's running ok, then exit
   * from thread body becouse the method to call is <code>sendline</code> and
   * it is done by the <code>Client</code> <i>class</i>
   */
  public void run() {
    System.err.println("Typist thread running ...     [OK]");
    running=true;
  }

  /**
   * This method is provide only for the garbage collection handler for
   * release the not available space
   */
  protected void finalize() throws Throwable {
    sout.close();  //close output stream
    sout=null;    //free the memory
    System.err.println("(Typist thread) Closing output stream.     [OK]");
    System.out.println("(Typist thread) TERMINATED!.     [OK]");
  }

  /**
   * Send the readed line to the output stream
   */
  public void sendLine() {
    String line=window.getText();
    window.setText("");          //erase TextField message

    line=line.trim()+"\n";      //erase head and tile spaces

    //if RTT is required, send the transmission "istante"
    if (applet.showRoundTripCheck.getState()) {
      long timeSent=(new Date()).getTime();   // millisecond time from 1/1/1970 0:0

      //send a line for RTT
      sout.print("TIMESTAMP istance_ID:"+applet.istance_ID
                 +" lineid:"+lineId
                 +" timesent:"+timeSent+"\n");
    }

    //send readed line
    sout.print(line);
    sout.flush();

    lineId++;
  }
}


/**
 * Listener <i>thread-class</i> of input socket stream.
 * It's execute this two steps:
 * <ul>
 * <li> listen and write all that arrive in an <code>TextArea</code>
 * <li> calculate the RTT if required
 * </ul>
 */
class Listener extends Thread {

  /**
   * This is the input socket stream used to comunicate
   */
  protected DataInputStream sin;

  /**
   * Contains the reference to an <code>TextArea</code> where we write
   * the message that server send us
   */
  protected TextArea window;

  /**
   * This flag indicates if listener is running
   */
  protected boolean running=false;

  /**
   * Contains the instance to the applet
   */
  protected Client applet;

  /**
   * Initialize object and start the <i>thread</i>
   *
   * @param s the socket to comunicate
   * @param window a <code>TextArea</code> where we write line
   * @param applet an <i>applet</i> of the <code>Client</code> type
   */
  public Listener(Socket s, TextArea window, Client applet) {
    System.out.println("Initializing a Listener Thread ...    [OK]");
    this.window=window;
    this.applet=applet;
    try	{
      sin=new DataInputStream(s.getInputStream());   //input socket stream
      System.out.println("Listener thread had opened the input stream.   [OK]");
    } catch (IOException e) {
        System.err.println ("[ERROR] Listener thread can't open the input stream: " + e);
	running=false;
        applet.cleanUp();
        return;
      }
    this.start();
  }

  /**
   * Listen the socket and write message line in the <code>TextArea</code>
   */
  public void run() {
    System.out.println("Listener thread running ...   [OK]");
    running=true;

    String line;
    long timeReceived;

    while (true) {
      try {
        line=sin.readLine();   //read (blocking) from socket line
        timeReceived=(new Date()).getTime();  //arrive time for RTT
      } catch (IOException e) {
          System.err.println("[ERROR] Listener: can't read from socket: " + e);
          running=false;
          applet.cleanUp();
          return;
        }
      // Verify if the socket connection was closed (EOF)
      if (line==null) {
        System.out.println("Listener receved 'EOF' from server: CLOSING listner");
        running=false;
        applet.cleanUp();
        return;
      }

      //calculate RTT
      String roundTripInfo="";
      if (applet.showRoundTripCheck.getState()) {
        int match=line.indexOf("TIMESTAMP istance_ID:"+applet.istance_ID);
        if (line.indexOf("TIMESTAMP istance_ID:"+ applet.istance_ID) != -1) {
          int i=line.indexOf("lineid:");
          int j=line.indexOf("timesent:");

          String lineIdString=line.substring(i+"lineid:".length(), j).trim();

          int lineId=Integer.parseInt(lineIdString);
          long timeSent=Long.parseLong(line.substring(j+"timesent:".length()));

          long roundTrip=timeReceived-timeSent;

          roundTripInfo="[roundtrip: "+roundTrip+" ms] ";
        }
      }

      //write lines only if they aren't timestamp messagges...
      if (line.indexOf("TIMESTAMP istance_ID:") == -1) {
        StringTokenizer StLine=new StringTokenizer(line,"\n\r",false);
        while (StLine.hasMoreTokens()) {
          applet.runCommand(StLine.nextToken());
        }
      }
      applet.insertText(roundTripInfo);
    }
  }
}


/**
 * Create an association between vrml types and an integer using association
 * of <i>class</i> in <code>vrml.external.fields</code>
 **/
class CodeTable {
  /**
   * Contain the actual valid vrml type
   */
  private final String[] Types= {
    "UnknownType","SFBOOL","SFIMAGE","SFTIME","SFCOLOR","MFCOLOR",
    "SFFLOAT","MFFLOAT","SFINT32","MFINT32","SFNODE","MFNODE",
    "SFROTATION","MFROTATION","SFSTRING","MFSTRING","SFVEC2F",
    "MFVEC2F","SFVEC3F","MFVEC3F"};

  /**
   * This is the <code>Hashtable</code> that contains the codes
   */
  private Hashtable Tab = new Hashtable();

  /**
   * Initialize the table of codes
   */
  public CodeTable() {
    int i=0;
    for (i=0;i<20;i++) {
      Tab.put(Types[i], new Integer(i));
    }
  }

  /**
   * Return the code associate to the input vrml type
   *
   * @param s the string containing the vrml type
   * @return the integer associated to the vrml type
   */
  public int Code(String s) {
    return ((Integer)Tab.get(s.toUpperCase())).intValue();
  }
}

